/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.assemblies;

import ist.palcom.resource.descriptor.AltServiceDecl;
import ist.palcom.resource.descriptor.AltServiceDeclList;
import ist.palcom.resource.descriptor.PRDAssemblyD;
import ist.palcom.resource.descriptor.PRDAssemblyVer;
import ist.palcom.resource.descriptor.AssignAction;
import ist.palcom.resource.descriptor.ConnectionDecl;
import ist.palcom.resource.descriptor.ConnectionDeclList;
import ist.palcom.resource.descriptor.ConstantUse;
import ist.palcom.resource.descriptor.DeviceDecl;
import ist.palcom.resource.descriptor.DeviceDeclList;
import ist.palcom.resource.descriptor.DeviceUse;
import ist.palcom.resource.descriptor.EventHandlerClause;
import ist.palcom.resource.descriptor.EventHandlerList;
import ist.palcom.resource.descriptor.EventHandlerScript;
import ist.palcom.resource.descriptor.Identifier;
import ist.palcom.resource.descriptor.InvokeAction;
import ist.palcom.resource.descriptor.MissingUse;
import ist.palcom.resource.descriptor.ParamUse;
import ist.palcom.resource.descriptor.SelfTestAction;
import ist.palcom.resource.descriptor.SendMessageAction;
import ist.palcom.resource.descriptor.ServiceAddress;
import ist.palcom.resource.descriptor.ServiceDecl;
import ist.palcom.resource.descriptor.ServiceDeclList;
import ist.palcom.resource.descriptor.ServiceUse;
import ist.palcom.resource.descriptor.SingleServiceDecl;
import ist.palcom.resource.descriptor.ThisProviderService;
import ist.palcom.resource.descriptor.ThisService;
import ist.palcom.resource.descriptor.VariableDecl;
import ist.palcom.resource.descriptor.VariableList;
import ist.palcom.resource.descriptor.VariableUse;
import ist.palcom.resource.function.FunctionInfoXMLFactory;
import ist.palcom.xml.XMLException;
import ist.palcom.xml.XMLRepresentable;

public class AssemblyXMLFactory extends FunctionInfoXMLFactory {
	
	protected XMLRepresentable createFromTagName(String tagName)
	throws XMLException {
        if (tagName.equals("AssemblyInfo")) {
        	System.err.println("AssemblyInfo XML tag used instead of PRDAssembly. Pls change to PRDAssembly");
            return new PRDAssemblyVer();
        }else if (tagName.equals("AssemblyDescriptor")) {
        	System.err.println("AssemblyDescriptor XML tag used instead of PRDAssembly. Pls change to PRDAssemblyList");
            return new PRDAssemblyD();
        }else if (tagName.equals("PRDAssemblyVer")) {
            return new PRDAssemblyVer();
        }else if (tagName.equals("PRDAssemblyD")) {
            return new PRDAssemblyD();
        } else if (tagName.equals("DeviceDeclList")) {
			return new DeviceDeclList();
        } else if (tagName.equals("ServiceDeclList")) {
			return new ServiceDeclList();
        } else if (tagName.equals("ConnectionDeclList")) {
			return new ConnectionDeclList();
        } else if (tagName.equals("DeviceDecl")) {
			return new DeviceDecl();
        } else if (tagName.equals("SingleServiceDecl")) {
			return new SingleServiceDecl();
        } else if (tagName.equals("ServiceDecl")) {
			return new ServiceDecl();
        } else if (tagName.equals("AltServiceDeclList")) {
			return new AltServiceDeclList();
        } else if (tagName.equals("AltServiceDecl")) {
			return new AltServiceDecl();
        } else if (tagName.equals("Identifier")) {
			return new Identifier();
        } else if (tagName.equals("ServiceUse")) {
			return new ServiceUse();
        } else if (tagName.equals("DeviceUse")) {
			return new DeviceUse();
        } else if (tagName.equals("ThisService")) {
			return new ThisService();
        } else if (tagName.equals("ThisProviderService")) {
			return new ThisProviderService();
//        } else if (tagName.equals("ServiceOnDevice")) {
//			return new ServiceOnDevice();
        } else if (tagName.equals("ConnectionDecl")) {
			return new ConnectionDecl();
        } else if (tagName.equals("EventHandlerScript")) {
			return new EventHandlerScript(new VariableList(), new EventHandlerList());
        } else if (tagName.equals("VariableList")) {
			return new VariableList();
        } else if (tagName.equals("EventHandlerList")) {
			return new EventHandlerList();
        } else if (tagName.equals("VariableDecl")) {
			return new VariableDecl();
        } else if (tagName.equals("EventHandlerClause")) {
			return new EventHandlerClause();
        } else if (tagName.equals("AssignAction")) {
			return new AssignAction();
        } else if (tagName.equals("SendMessageAction")) {
			return new SendMessageAction();
        } else if (tagName.equals("InvokeAction")) {
			return new InvokeAction();
        } else if (tagName.equals("SelfTestAction")) {
			return new SelfTestAction();
        } else if (tagName.equals("VariableUse")) {
			return new VariableUse();
        } else if (tagName.equals("ParamUse")) {
			return new ParamUse();
        } else if (tagName.equals("ConstantUse")) {
			return new ConstantUse();
        } else if (tagName.equals("MissingUse")) {
			return new MissingUse();
        } else if (tagName.equals(ServiceAddress.class.getName())) {
			return new ServiceAddress();
		} else {
			return super.createFromTagName(tagName);
		}
	}
}
