/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.assemblies;

import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.resource.descriptor.PRDAssemblyD;
import ist.palcom.resource.descriptor.PRDAssemblyVer;
import ist.palcom.resource.descriptor.ServiceProxy;
import ist.palcom.services.CommandEventListener;

import java.io.IOException;

public interface IAssemblyManager extends CommandEventListener {
	public static final String URN_SUFFIX="AssemblyManager";
	public static final String SERVICE_NAME="AssemblyManager";
	public static final String COMMAND_LOADED_COUNT_IN = "loaded count";
	public static final String COMMAND_LOADED_COUNT_OUT = "count";
	public static final String COMMAND_LIST_LOADED = "list_loaded";
	public static final String COMMAND_STORED_ASSEMBLIES = "stored list";
	public static final String COMMAND_STORED_COUNT_IN = "stored count";
	public static final String COMMAND_STORED_COUNT_OUT = "count_stored";
	public static final String COMMAND_LIST_STORED = "list_stored";
	public static final String COMMAND_LOADED_ASSEMBLIES = "loaded list";
	public static final String COMMAND_UNLOAD_ALL = "unload all";
	public static final String COMMAND_UNLOAD = "unload";
	public static final String COMMAND_LOAD = "load";
	public static final String COMMAND_READ_LOADED = "read_loaded";
	public static final String COMMAND_STORE = "store";
//	public static final String COMMAND_READ_STORED = "read_stored";
//	public static final String COMMAND_READ = COMMAND_READ_LOADED;
	public static final String COMMAND_ASSEMBLYINFO = "ai";
	public static final String COMMAND_UPDATE = "update";
	public static final String COMMAND_ADD_DECL = "addDecl";
	public static final String COMMAND_REM_DECL = "removeDecl";
	public static final boolean ENABLE_UPDATE_SERVICE = false;
//	private static final String COMMAND_ADD_DEVICE = "addService";
//	private static final String COMMAND_ADD_SERVICE = "addService";
//	private static final String COMMAND_ADD_CONNECTION = "addConnection";

	public byte[] readAssembly(int index);

	public void doAddDeclCommand(String idxStr, String decl);

	public void doRemDeclCommand(String idxStr, String decl);

	public void loadAssembly(PRDAssemblyVer ai);

	public void stopAssembly(Assembly a);

	public void removeAssemblies();

	public void start() throws IOException;

	public void stop() throws IOException;

	public ServiceProxy getServiceProxy();
	
	public Assembly checkIfUpdateToRunning(PRDAssemblyVer ai);

	public PRDAssemblyD checkIfUpdateToStored(PRDAssemblyVer ai);

	public void registerContingencyManagerThread(PalcomThread t);

	/**
	 * Unregister the receiver thread
	 * 
	 * @param t -
	 *            the thread
	 */
	public void unregisterContingencyManagerThread(PalcomThread t);

	/**
	 * Returns a palcom thread responsible for receiving and executing edit
	 * commands for (running) assemblies
	 */
	public PalcomThread getEditCommandReceiverThread();
}
