/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.assemblies;

import ist.palcom.palcomthreads.Event;
import ist.palcom.resource.descriptor.Info;
import ist.palcom.resource.descriptor.PRDData;

public class ResourceResponseEvent extends Event {

	public static final int ADD = 1;
	public static final int CHANGE = 2;
	public static final int REMOVE = 3;
	
	private PRDData decl;
	private Info info;
	private Info oldInfo;
	private int status;

	public ResourceResponseEvent(PRDData d, int status) {
		this.decl = d;
		this.info = null;
		this.status = status;
	}

	public ResourceResponseEvent(PRDData d, Info i, int status) {
		this.decl = d;
		this.info = i;
		this.status = status;
	}
	
	public ResourceResponseEvent(PRDData d, Info info, Info oldInfo, int status) {
		this(d, info,status);
		this.oldInfo = oldInfo;
	}
	
	public PRDData getDecl() {
		return decl;
	}
	
	public Info getInfo() {
		return info;
	}

	/** only used when isChanged()
	 */
	public Info getOldInfo() {
		return oldInfo;
	}
	
	public boolean isAdded() {
		return status == ADD;
	}

	public boolean isChanged() {
		return status == CHANGE;
	}

	public boolean isRemoved() {
		return status == REMOVE;
	}
}
