/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.assembly.manager;

import ist.palcom.assemblies.Assembly;
import ist.palcom.assemblies.AssemblyDeviceContext;
import ist.palcom.assemblies.AssemblyParser;
import ist.palcom.assemblies.EditCommandEvent;
import ist.palcom.assemblies.IAssemblyManager;
import ist.palcom.assemblies.ScriptedAssembly;
import ist.palcom.assemblies.UpdateStateMachine;
import ist.palcom.debugging.DebugDispatcher;
import ist.palcom.device.DeviceContext;
import ist.palcom.discovery.CommandEvent;
import ist.palcom.discovery.InfoEvent;
import ist.palcom.palcomthreads.Event;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.resource.descriptor.AbstractXMLRepresentable;
import ist.palcom.resource.descriptor.Command;
import ist.palcom.resource.descriptor.ControlItem;
import ist.palcom.resource.descriptor.DeviceID;
import ist.palcom.resource.descriptor.Group;
import ist.palcom.resource.descriptor.GroupInfo;
import ist.palcom.resource.descriptor.List;
import ist.palcom.resource.descriptor.LocalSID;
import ist.palcom.resource.descriptor.PRDAssemblyVer;
import ist.palcom.resource.descriptor.PRDAssemblyD;
import ist.palcom.resource.descriptor.PRDDevice;
import ist.palcom.resource.descriptor.PRDService;
import ist.palcom.resource.descriptor.PRDSubList;
import ist.palcom.resource.descriptor.Param;
import ist.palcom.resource.descriptor.ServiceID;
import ist.palcom.resource.descriptor.ServiceInstanceID;
import ist.palcom.resource.descriptor.ServiceProxy;
import ist.palcom.resource.descriptor.URN;
import ist.palcom.resource.descriptor.URNLeaf;
import ist.palcom.services.AbstractService;
import ist.palcom.services.CommandListenerThread;
import ist.palcom.services.SelfTestProvider;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;


public class AssemblyManager extends AbstractService implements IAssemblyManager {
	private CommandListenerThread inCommandThread;
//	private ArrayList<Assembly> assemblies;
	private ArrayList assemblies;
	private EditCommandThread editCommandThread;
	
	public AssemblyManager(DeviceContext context, String serviceName) throws IOException {
		super(context, serviceName, true, false, PRDService.UNICAST, "P1",
                false, "1", "Assembly manager service", createAssembliesServiceProxy(serviceName), 
                new LocalSID(context.getDeviceInfo().getDeviceID(), context.getNewLocalSIDString()));
		ServiceID serviceID = new ServiceID(
        		new DeviceID("X:1"), "TF1", new DeviceID("X:1"), "TF1", null, null, null, null);
		setServiceInstanceID(new ServiceInstanceID(context.getDeviceInfo().getDeviceID(), serviceID, 
				context.getNewInstanceNumber(serviceID)));
//		assemblies = new ArrayList<Assembly>();
		assemblies = new ArrayList();
        inCommandThread = new CommandListenerThread(this);
        context.getScheduler().scheduleThread(inCommandThread);
        getServiceProxy().addInCommandReceiver(inCommandThread);  
        editCommandThread = new EditCommandThread();
        
        if(ENABLE_UPDATE_SERVICE) {
			setupUpdateService();
		}
//        PalcomPriorityThread ct = new PalcomPriorityThread() {
//		        	public void run() {
//		        		System.out.println("CT running");
//		        		while(true) {
//		        			Event e = waitEvent();
//		        			if(e instanceof InfoEvent) {
//		        				System.out.println("CMT:"+((InfoEvent)e).getInfo().getAssemblyLabel());
//		        			}
//		        		}
//		        	}
//		        };
//		registerContingencyManagerThread(ct);
//		context.getScheduler().scheduleThread(ct);
	}

	protected SelfTestProvider getSelfTestService(String urn) {
    	return null;
    }
	
	/**
	 * This constructor makes it possible to instantiate the AssemblyManager from the ServiceManager
	 */
	public AssemblyManager(DeviceContext context) throws IOException {
		this(context, SERVICE_NAME);
	}

    private static ServiceProxy createAssembliesServiceProxy(String controlName) {
        ServiceProxy sd = new ServiceProxy(controlName);
        Group loadedGroup = new Group(new GroupInfo(), new List());
        loadedGroup.getInfo().setID("Loaded assemblies");
        Group storedGroup = new Group(new GroupInfo(), new List());
        storedGroup.getInfo().setID("Stored assemblies");
        Group group = new Group(new GroupInfo(), new List());
        group.getInfo().setID("Commands");
        sd.addControlItemAndInfo(group);
        sd.addControlItemAndInfo(loadedGroup);
        sd.addControlItemAndInfo(storedGroup);
        {
        Command listCmdIn = new Command(COMMAND_LIST_LOADED, Command.DIRECTION_IN);
        loadedGroup.addControlItemAndInfo(listCmdIn);
        Command listCmdOut = new Command(COMMAND_LOADED_ASSEMBLIES, Command.DIRECTION_OUT);
        listCmdOut.addParamAndInfo(new Param("Assembly list", 
                "text/plain", "Ingen data".getBytes()));
        loadedGroup.addControlItemAndInfo(listCmdOut);
        Command countCmd = new Command(COMMAND_LOADED_COUNT_IN, Command.DIRECTION_IN);
        loadedGroup.addControlItemAndInfo(countCmd);
        Command countCmd2 = new Command(COMMAND_LOADED_COUNT_OUT, Command.DIRECTION_OUT);
        countCmd2.addParamAndInfo(new Param("val","text/plain","".getBytes()));
        loadedGroup.addControlItemAndInfo(countCmd2);
        }
        {
            Command listCmdIn = new Command(COMMAND_LIST_STORED, Command.DIRECTION_IN);
            storedGroup.addControlItemAndInfo(listCmdIn);
            Command listCmdOut = new Command(COMMAND_STORED_ASSEMBLIES, Command.DIRECTION_OUT);
            listCmdOut.addParamAndInfo(new Param("Assembly list", 
                    "text/plain", "Ingen data".getBytes()));
            storedGroup.addControlItemAndInfo(listCmdOut);
            Command countCmd = new Command(COMMAND_STORED_COUNT_IN, Command.DIRECTION_IN);
            storedGroup.addControlItemAndInfo(countCmd);
            Command countCmd2 = new Command(COMMAND_STORED_COUNT_OUT, Command.DIRECTION_OUT);
            countCmd2.addParamAndInfo(new Param("val","text/plain","".getBytes()));
            storedGroup.addControlItemAndInfo(countCmd2);
            }
        Command unloadCmd = new Command(COMMAND_UNLOAD, Command.DIRECTION_IN);
        unloadCmd.addParamAndInfo(new Param("Index", "text/plain", "".getBytes()));
        group.addControlItemAndInfo(unloadCmd);
        Command quitCmd = new Command(COMMAND_UNLOAD_ALL, Command.DIRECTION_IN);
        group.addControlItemAndInfo(quitCmd);   
        
        Command updateCmd = new Command(COMMAND_UPDATE, Command.DIRECTION_OUT);
        updateCmd.addParamAndInfo(new Param("AssemblyInfo", "text/xml", "".getBytes()));
        group.addControlItemAndInfo(updateCmd);

       
        Group group2 = new Group(new GroupInfo(), new List());
        group2.getInfo().setID("Loading");
        sd.addControlItemAndInfo(group2);
        Command loadCmd = new Command(COMMAND_LOAD, Command.DIRECTION_IN);
        loadCmd.addParamAndInfo(new Param("AssemblyInfo", "text/xml", "".getBytes()));
        group2.addControlItemAndInfo(loadCmd);
        
        Command readCmd = new Command(COMMAND_READ_LOADED, Command.DIRECTION_IN);
        readCmd.addParamAndInfo(new Param("Index", "text/plain", "".getBytes()));
        group2.addControlItemAndInfo(readCmd);

        Command aiCmd = new Command(COMMAND_ASSEMBLYINFO, Command.DIRECTION_OUT);
        aiCmd.addParamAndInfo(new Param("AssemblyInfo", "text/xml", "".getBytes()));
        group2.addControlItemAndInfo(aiCmd);

        Group group4 = new Group(new GroupInfo(), new List());
        group4.getInfo().setID("Storing");
        sd.addControlItemAndInfo(group4);
        Command storeCmd = new Command(COMMAND_STORE, Command.DIRECTION_IN);
        storeCmd.addParamAndInfo(new Param("XML", "text/xml", "".getBytes()));
        group4.addControlItemAndInfo(storeCmd);
        
//        Command readStoredCmd = new Command(COMMAND_READ_STORED, Command.DIRECTION_IN);
//        readStoredCmd.addParamAndInfo(new Param("Index", "text/plain", "".getBytes()));
//        group4.addControlItemAndInfo(readStoredCmd);
//
//        Command aiStCmd = new Command(COMMAND_ASSEMBLYINFO_STORED, Command.DIRECTION_OUT);
//        aiStCmd.addParamAndInfo(new Param("XML", "text/xml", "".getBytes()));
//        group4.addControlItemAndInfo(aiStCmd);
        
        Group group3 = new Group(new GroupInfo(), new List());
        group3.getInfo().setID("Editing");
        sd.addControlItemAndInfo(group3);
        
        Command addDeclCmd = new Command(COMMAND_ADD_DECL, Command.DIRECTION_IN);
        addDeclCmd.addParamAndInfo(new Param("AssemblyIndex", "text/plain", "".getBytes()));
        addDeclCmd.addParamAndInfo(new Param("Declaration", "text/xml", "".getBytes()));
        group3.addControlItemAndInfo(addDeclCmd);

        Command remDeclCmd = new Command(COMMAND_REM_DECL, Command.DIRECTION_IN);
        remDeclCmd.addParamAndInfo(new Param("AssemblyIndex", "text/plain", "".getBytes()));
        remDeclCmd.addParamAndInfo(new Param("Declaration", "text/xml", "".getBytes()));
        group3.addControlItemAndInfo(remDeclCmd);

        

//        Command addDevCmd = new Command(COMMAND_ADD_DEVICE, Command.DIRECTION_IN);
//        addDevCmd.addParamAndInfo(new Param("AssemblyIndex", "text/plain", "".getBytes()));
//        addDevCmd.addParamAndInfo(new Param("DeviceName", "text/plain", "".getBytes()));
//        addDevCmd.addParamAndInfo(new Param("DeviceURN", "text/plain", "".getBytes()));
//        group3.addControlItemAndInfo(addDevCmd);
//        
//        Command addSvcCmd = new Command(COMMAND_ADD_SERVICE, Command.DIRECTION_IN);
//        addSvcCmd.addParamAndInfo(new Param("AssemblyIndex", "text/plain", "".getBytes()));
//        addSvcCmd.addParamAndInfo(new Param("LocalName", "text/plain", "".getBytes()));
//        addSvcCmd.addParamAndInfo(new Param("ServiceName", "text/plain", "".getBytes()));
//        addSvcCmd.addParamAndInfo(new Param("DeviceUse", "text/plain", "".getBytes()));
//        addSvcCmd.addParamAndInfo(new Param("URNSuffix", "text/plain", "".getBytes()));
//        group3.addControlItemAndInfo(addSvcCmd);
//
//        Command addConCmd = new Command(COMMAND_ADD_SERVICE, Command.DIRECTION_IN);
//        addConCmd.addParamAndInfo(new Param("AssemblyIndex", "text/plain", "".getBytes()));
//        addConCmd.addParamAndInfo(new Param("AssemblyIndex", "text/plain", "".getBytes()));
//        addConCmd.addParamAndInfo(new Param("AssemblyIndex", "text/plain", "".getBytes()));
//        addConCmd.addParamAndInfo(new Param("AssemblyIndex", "text/plain", "".getBytes()));
//        addConCmd.addParamAndInfo(new Param("AssemblyIndex", "text/plain", "".getBytes()));
//
//        group3.addControlItemAndInfo(addConCmd);


        return sd;
    }
    public void commandReceived(CommandEvent e) {
    	String command = e.getCommand().getID();
    	try {
    		if (command.equals(COMMAND_LIST_LOADED)) {
    			doAssemblyCommand();
    		}else if(command.equals(COMMAND_LOADED_COUNT_IN)) {
    			doCountCommand();
    		}else if(command.equals(COMMAND_LOAD)){
    			String assemblyText = new String(e.getCommand().getParam(0).getData());
    			PRDAssemblyVer ai = (PRDAssemblyVer) AssemblyParser.parseAssemblyXML(assemblyText);
    			loadAssembly(ai);
    			doAssemblyCommand();
    		}else if(command.equals(COMMAND_READ_LOADED)){
//    			System.out.println("COMMAND_READ");
    			String idxStr = new String(e.getCommand().getParam(0).getData());
    			doAssemblyInfoCommand(idxStr);
    		}else if(command.equals(COMMAND_ADD_DECL)){
    			System.out.println("COMMAND_ADD_DECL");
    			String idxStr = new String(e.getCommand().getParam(0).getData());
    			String decl = new String(e.getCommand().getParam(1).getData());
    			doAddDeclCommand(idxStr, decl);
    		}else if(command.equals(COMMAND_REM_DECL)){
    			System.out.println("COMMAND_REM_DECL");
    			String idxStr = new String(e.getCommand().getParam(0).getData());
    			String decl = new String(e.getCommand().getParam(1).getData());
    			doRemDeclCommand(idxStr, decl);
    		}else if(command.equals(COMMAND_UNLOAD)){
    			System.out.println("COMMAND_UNLOAD");
    			String idxStr = new String(e.getCommand().getParam(0).getData());
    			doUnloadAssemblyCommand(idxStr);
    			doAssemblyCommand();
    		}else if(command.equals(COMMAND_UNLOAD_ALL)){
    			System.out.println("COMMAND_QUIT");
    			removeAssemblies();
    			doAssemblyCommand();
    			//stop();
//    		}else if(command.equals(COMMAND_UPDATE)){
//    			System.out.println("COMMAND_UPDATE");
//    			doUpdateCommand(e.getCommand().getParam(0).getData());
    		}
    	} catch (Exception ex) {ex.printStackTrace();}
	}

	protected void doUpdateCommand(PRDAssemblyVer ai) {
        if(ENABLE_UPDATE_SERVICE) {
        	try {
        		doCommand(COMMAND_UPDATE,ai.getXMLData());
        	} catch (Exception e) {
        		System.err.println("AssemblyManager.doUpdateCommand: "+e.toString());
        	}
        }
	}

	private void doCountCommand() {
		String line;
		if(assemblies.isEmpty())
			line = "0";
		else {
			line = Integer.toString(assemblies.size());
		}
		doCommand(COMMAND_LOADED_COUNT_OUT, line);		
	}

	private void doAssemblyCommand() {
		String line;
		if(assemblies.isEmpty())
			line = "no assemblies";
		else {
			line = "";
			for (Iterator iter = assemblies.iterator(); iter.hasNext();) {
				Assembly a = (Assembly) iter.next();
				line = line + assemblies.indexOf(a)+":"+ a.assemblyVer.getName();
				if(iter.hasNext()) line = line+"\n";
			}
			//assemblies.get(0).assemblyInfo.getName();
		}
		doCommand(COMMAND_LOADED_ASSEMBLIES, line);
	}

    private void doUnloadAssemblyCommand(String idxStr) {
    	try {
    		int index = Integer.parseInt(idxStr);
    		Assembly a = (Assembly) assemblies.remove(index);
			stopAssembly(a);
    	} catch (Exception e) {
    		e.printStackTrace();
    	}
    }

	
    private void doAssemblyInfoCommand(String idxStr) {
    	try {
    		int index = Integer.parseInt(idxStr);
    		byte[] data = readAssembly(index);
			doCommand(COMMAND_ASSEMBLYINFO, data);
    	} catch (Exception e) {
    		e.printStackTrace();
    	}
    }

	public byte[] readAssembly(int index) {
		Assembly a = (Assembly) assemblies.get(index);
		byte[] data = a.assemblyVer.getXMLData();
		return data;
	}

    public void doAddDeclCommand(String idxStr, String decl) {
    	try {
    		int index = Integer.parseInt(idxStr);
    		Assembly a = (Assembly) assemblies.get(index);
    		AbstractXMLRepresentable tmp = AssemblyParser.parseAssemblyXML(decl);
    		tmp.addToAssembly(a);
    		DebugDispatcher.debug("doAddDeclCommand:"+tmp, DebugDispatcher.CAT_ASSEMBLY);
    		// TODO: add better condition...
//    		if( (tmp instanceof ConnectionDecl) || (tmp instanceof ServiceDecl)) {
//        		DebugDispatcher.debug("doAddDeclCommand: calling doRequestConnections",DebugDispatcher.CAT_ASSEMBLY);
        		//XXX Terrible hack
//        		a.assemblyInfo.setThisReferencesToDeviceURN(context.getDeviceInfo().getDeviceURN());
        		//TODO: how to propagate the information that a serviceDecl has changed
        		// to the resource manager?
//    			a.doRequestConnections();
//    		}
    	} catch (Exception e) {
    		e.printStackTrace();
    	}
    }
    
	public void doRemDeclCommand(String idxStr, String decl) {
		try {
			int index = Integer.parseInt(idxStr);
			Assembly a = (Assembly) assemblies.get(index);
			AbstractXMLRepresentable tmp = AssemblyParser.parseAssemblyXML(decl);
			tmp.removeFromAssembly(a);
			DebugDispatcher.debug("doRemDeclCommand:"+tmp, DebugDispatcher.CAT_ASSEMBLY);
			// TODO: add better condition...
//			if( (tmp instanceof ConnectionDecl) || (tmp instanceof ServiceDecl)) {
			DebugDispatcher.debug("doRemDeclCommand: calling doRequestConnections",DebugDispatcher.CAT_ASSEMBLY);
			a.doRequestConnections();
//			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
    
	protected void doCommand(String commandID, String text) {
	    ControlItem tmp = getServiceProxy().findItem(commandID);
	    if(tmp instanceof Command) {
	        Command command = (Command) tmp;

	        if (command != null) {
	            command.getParam(0).setData(text.getBytes());
	            command.invoke();
	        }
	    }    	
	}

	protected void doCommand(String commandID, byte[]data) {
	    ControlItem tmp = getServiceProxy().findItem(commandID);
	    if(tmp instanceof Command) {
	        Command command = (Command) tmp;

	        if (command != null) {
	            command.getParam(0).setData(data);
	            command.invoke();
	        }
	    }
	}

	
	public void loadAssembly(PRDAssemblyVer ai) {
		try {
//			changeThisReferecesToCurrentDevice(ai, aiURN);
			//TODO: how is instantiation to be done?
			ai.setThisReferencesToDeviceID(context);
			ai.resetTree();
			Assembly a = new ScriptedAssembly((AssemblyDeviceContext) context, ai, 1000);
			DebugDispatcher.debug("Starting "+a+"("+ai.getName()+")",DebugDispatcher.CAT_ASSEMBLY);
			a.start();
			assemblies.add(a);
			doUpdateCommand(ai);
			//Inform contingency manager about the start of a new assembly
			if(contingencyThread!=null) {
				contingencyThread.putEvent(new InfoEvent(ai,true));
			}
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void stopAssembly(Assembly a) {
	    context.getScheduler().scheduleThread(new StopThread(a));
		//Inform contingency manager that an assembly has been stopped
		if(contingencyThread!=null) {
			contingencyThread.putEvent(new InfoEvent(a.assemblyVer,false));
		}
	}

    /** The PalcomThread that calls stopDevice. */
    private class StopThread extends PalcomPriorityThread {
    	Assembly a;
    	public StopThread(Assembly a) {
    		this.a = a;
    	}
        public void run() {
            try {
                a.stop();
    			context.removeService(a);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    
	public void removeAssemblies() {
//		for (Assembly a : assemblies) {
		for (Iterator iter = assemblies.iterator(); iter.hasNext();) {
			Assembly a = (Assembly) iter.next();			
			DebugDispatcher.debug("Stopping "+a,DebugDispatcher.CAT_ASSEMBLY);
			stopAssembly(a);
		}
		assemblies.clear();
	}

	public void start() throws IOException {
		super.start();
		context.getScheduler().scheduleThread(editCommandThread);
		if(ENABLE_UPDATE_SERVICE) {
			updateCustomer.start();
		}
	}
	
	public void stop() throws IOException {
		removeAssemblies();
		editCommandThread.terminate();
		if(ENABLE_UPDATE_SERVICE) {
			updateCustomer.stop();
		}
        inCommandThread.terminate();
		super.stop();
	}

	private UpdateStateMachine updateSM;
	private AbstractService updateCustomer;
	
	private class StateMachineThread extends PalcomPriorityThread {
		public void run() {
			while(true) {
				Event e = waitEvent();
				if(e instanceof InfoEvent && ((InfoEvent)e).getInfo() instanceof PRDDevice) {
					//TODO: discriminate...
					PRDDevice device = (PRDDevice)((InfoEvent)e).getInfo();
					context.getResourceManager2().updatePRDServiceListReceiver(new LocalSID(device.getDeviceID(),null), this);
//					context.getResourceManager2().updateConnectionInfoReceiver(((InfoEvent)e).getInfo().getDeviceURN(), this);
				}
				updateSM.handleEvent(e);
			}
		}
	}
	
	private void setupUpdateService() {
		final StateMachineThread t =new StateMachineThread();
//		try {
//			TODO: Why doesn't connecting work when updateCustomer is a subservice of AssemblyManager?
//			URN uu = getServiceInfo().getURN().createSubUrn(new URNLeaf("update"));
//			updateCustomer = new ControlCustomer(context, uu,"UpdateService") {
			throw new Error("AssemblyManager.updateCustomer broken by protocol changes.");
//			updateCustomer = new AbstractService(context,
//                    new URNLeaf("update"), "UpdateService",
//                    PRDService.ROLE_CUSTOMER, ServiceProxy.CONTENT_TYPE) {
//				public void start() throws IOException {
//					updateCustomer.addServiceProxyListener(t);
//					super.start();
//					context.getScheduler().scheduleThread(t);
//					context.getResourceManager2().updatePRDDeviceReceiver(t);
//				}
//				public void stop() throws IOException {
//					t.terminate();
//					context.getResourceManager2().removePRDDeviceReceiver(t);
//					super.stop();
//				}
//			};
//			context.addServiceAtDeviceLevel(updateCustomer);
//			//addSubservice(updateCustomer);
//			URN updateURN = updateCustomer.getServiceInfo().getURN();
//			updateSM = new UpdateStateMachine(updateURN, (AssemblyDeviceContext) context, t);
//		} catch (IOException e) {
//			e.printStackTrace();
//		}		
	}
	
	/** Aux method: iterate over the assemblies, and check if
	 *  ai is an update to any of them.
	 * @param ai
	 * @return the parent of ai, or null
	 */
	public Assembly checkIfUpdateToRunning(PRDAssemblyVer ai) {
		for (Iterator iter = assemblies.iterator(); iter.hasNext();) {
			Assembly tmp = (Assembly)iter.next();
			if(tmp.assemblyVer.isUpdate(ai)) {
				return tmp;
			}
		}
		return null;
	}
	
	public PRDAssemblyD checkIfUpdateToStored(PRDAssemblyVer ai) {
		return null;
		
	}
	
	private PalcomThread contingencyThread;
	
	/** Register a thread that receives requests for contingency
	 * monitoring for assemblies loaded onto this assembly manager.
	 * 
	 * @param t - The receiver thread
	 */
	public void registerContingencyManagerThread(PalcomThread t) {
		if(contingencyThread != null) {
			System.err.println("WARNING! AssemblyManager.registerContingencyManagerThread: this replaces the previously registered thread.");
		}
		contingencyThread = t;
	}
	
	/** Unregister the receiver thread
	 * 
	 * @param t - the thread
	 */
	public void unregisterContingencyManagerThread(PalcomThread t) {
		contingencyThread = null;
	}
	
	/** Returns a palcom thread responsible for receiving and executing
	 * edit commands for (running) assemblies
	 */
	public PalcomThread getEditCommandReceiverThread() {
		return editCommandThread;
	}
	
	private class EditCommandThread extends PalcomPriorityThread {
		public void run() {
			while(true) {
				Event e = waitEvent();
				if(e instanceof EditCommandEvent) {
					((EditCommandEvent)e).execute(assemblies);
				}
			}
		}
	}
}
