package ist.palcom.assembly.manager;

import ist.palcom.collections.LinearSet;
import ist.palcom.palcomthreads.Monitor;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.resource.descriptor.AbstractResourceAddress;
import ist.palcom.resource.descriptor.ConnectionInfo;
import ist.palcom.resource.descriptor.Info;
import ist.palcom.resource.descriptor.ServiceAddress;
import ist.palcom.resource.descriptor.ServiceInfo;

import java.util.Iterator;

public class InfoMonitor extends Monitor {

	private LinearSet knownServices;

	public InfoMonitor(PalcomScheduler scheduler) {
		super(scheduler);
		knownServices = new LinearSet();
	}

	public void addInfo(Info i) {
		try {
			enterMonitor();	
//			System.err.println("InfoMonitor.add:"+i);
			if(i instanceof ServiceInfo) {
				knownServices.add(i);
//				System.err.println("... size ="+knownServices.toArray().length);
			}
//			else {
//				System.err.println("WARNING! infoMonitor.addInfo ignoring "+i);
//			}

		} finally {
			exitMonitor();
		}
	}

	public void removeInfo(Info i) {
		try {
			enterMonitor();
			System.out.println("InfoMonitor.remove:"+i);
			if(i instanceof ServiceInfo) {
				removeInfo((ServiceInfo)i);
//				System.err.println("... size ="+knownServices.toArray().length);
			} else {
				System.err.println("WARNING! infoMonitor.removeInfo ignoring "+i);
			}

		} finally {
			exitMonitor();
		}
	}

	public ServiceInfo getServiceInfo(AbstractResourceAddress sa) {
		try {
			enterMonitor();	
			ServiceInfo result = null;
			Iterator it = knownServices.iterator();
			while(it.hasNext() && result == null) {
				ServiceInfo tmp = (ServiceInfo) it.next();
				if(tmp.getServiceAddress().equals(sa)) {
					result = tmp;
				}
			}
			return result;
		} finally {
			exitMonitor();
		}
	}

	private void removeInfo(ServiceInfo i) {
		AbstractResourceAddress sa = i.getAddress();
		Iterator it = knownServices.iterator();
		while(it.hasNext()) {
			ServiceInfo tmp = (ServiceInfo) it.next();
			if(tmp.getServiceAddress().equals(sa)) {
				knownServices.remove(tmp);
				return;
			}
		}

	}

	public ConnectionInfo checkRequest(ConnectionRequest cr) {
		try {
			enterMonitor();
			ConnectionInfo result = cr.checkAgainstServices(knownServices.iterator());
			return result;
		} finally  {
			exitMonitor();
		}
	}

	public void tryLoadingSoftwareComponent(ConnectionRequest cr) {
		try {
			enterMonitor();
			cr.checkSoftwareManagers(knownServices.iterator());
		} finally  {
			exitMonitor();
		}
	}
}
