///*
//   Copyright (c) 2004-2007, The PalCom Project consortium.
//   See http://www.ist-palcom.org/consortium.
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or without
//   modification, are permitted provided that the following conditions are
//   met:
//
// * Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following
//         disclaimer in the documentation and/or other materials provided
//         with the distribution.
//
// * Neither the name of the PalCom Project nor the names of its
//         contributors may be used to endorse or promote products derived
//         from this software without specific prior written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// */
//
//package ist.palcom.assembly.manager;
//
//import ist.palcom.assemblies.Assembly;
//import ist.palcom.assemblies.AssemblyDeviceContext;
//import ist.palcom.assemblies.IResourceManager;
//import ist.palcom.assemblies.ResourceRequestEvent;
//import ist.palcom.assemblies.ResourceResponseEvent;
//import ist.palcom.collections.LinearMap;
//import ist.palcom.debugging.DebugDispatcher;
//import ist.palcom.device.DeviceContext;
//import ist.palcom.device.RemoteConnectHelper;
//import ist.palcom.discovery.InfoEvent;
//import ist.palcom.info.ModifiableInfoHandler;
//import ist.palcom.palcomthreads.Event;
//import ist.palcom.palcomthreads.PalcomPriorityThread;
//import ist.palcom.palcomthreads.PalcomThread;
//import ist.palcom.palcomthreads.PeriodicEvent;
//import ist.palcom.resource.descriptor.AbstractServiceDecl;
//import ist.palcom.resource.descriptor.AltServiceDecl;
//import ist.palcom.resource.descriptor.AltServiceDeclList;
//import ist.palcom.resource.descriptor.ConnectionDecl;
//import ist.palcom.resource.descriptor.ConnectionDeclList;
//import ist.palcom.resource.descriptor.DeviceID;
//import ist.palcom.resource.descriptor.LocalSID;
//import ist.palcom.resource.descriptor.PRDAssemblyVer;
//import ist.palcom.resource.descriptor.PRDConnection;
//import ist.palcom.resource.descriptor.PRDData;
//import ist.palcom.resource.descriptor.PRDDevice;
//import ist.palcom.resource.descriptor.PRDService;
//import ist.palcom.resource.descriptor.PRDServiceList;
//import ist.palcom.resource.descriptor.ServiceDecl;
//import ist.palcom.resource.descriptor.ServiceExp;
//import ist.palcom.resource.descriptor.ServiceID;
//import ist.palcom.resource.descriptor.ServiceInstanceID;
//import ist.palcom.resource.descriptor.ServiceProxy;
//import ist.palcom.resource.descriptor.ServiceUse;
//import ist.palcom.resource.descriptor.SingleServiceDecl;
//import ist.palcom.resource.descriptor.ThisService;
//import ist.palcom.resource.descriptor.URN;
//import ist.palcom.services.AbstractService;
//
//import java.io.IOException;
//import java.util.ArrayList;
//import java.util.Iterator;
//
///**
// * 
// * A class responsible for providing Assemblies with resources
// * 
// * This is the multi-threaded version, with one thread monitoring 
// * each request.
// * 
// * Currently, only connections are supported: An assembly sends requests
// * for connections, and the ResourceManager responds with an ConnectionInfo,
// * if the connection is possible (i.e., the required services have been
// * discovered)
// * 
// * If established resources go down, dependent assemblies are informed
// * 
// * For alternative resources, if a better alternative appears, dependent assemblies
// * are informed that a change is possible 
// * 
// * @author sven
// *
// */
//public class ResourceManager3 extends AbstractService implements IResourceManager{
//
//	public static final String SERVICE_NAME = "ResourceManager";
//
//	private ResourceThread resourceThread;
//
//	private InfoThread infoThread;
//
//	private ModifiableInfoHandler remoteInfoHandler;
//
//	private LinearMap connectionDeclToThread;
//
//	public ResourceManager3(DeviceContext context, String serviceName) throws IOException {
//		super(context, serviceName, true, false, PRDService.UNICAST, "P1", 
//				false, "1", "", createServiceProxy(serviceName), new LocalSID(
//						context.getDeviceInfo().getDeviceID(), context.getNewLocalSIDString()));
//		ServiceID serviceID = new ServiceID(
//				new DeviceID("X:1"), "TF2", new DeviceID("X:1"), "TF2", null, null, null, null);
//		setServiceInstanceID(new ServiceInstanceID(context.getDeviceInfo().getDeviceID(), serviceID, 
//				context.getNewInstanceNumber(serviceID)));
//		resourceThread = new ResourceThread();
//		infoThread = new InfoThread();
//		remoteInfoHandler = new ModifiableInfoHandler();
//		connectionDeclToThread = new LinearMap();
//		context.getResourceManager2().updatePRDDeviceReceiver(infoThread);
////		((AssemblyDeviceContext) context).getResourceAddressingManager().listenForDevices(infoThread);
//		context.getScheduler().scheduleThread(resourceThread);
//		context.getScheduler().scheduleThread(infoThread);
//	}
//
//	/**
//	 * This constructor makes it possible to start the ResourceManager from the ServiceManager
//	 */
//	public ResourceManager3(DeviceContext context) throws IOException {
//		this(context, SERVICE_NAME);
//	}		
//	public void start() throws IOException {
//		super.start();
//	}
//
//	public void stop() throws IOException {
//		resourceThread.terminate();
//		infoThread.terminate();
//		super.stop();
//	}
//
//	/** tryLoadingSoftwareComponent 
//	 * 
//	 * Connect to the SoftwareComponentManager on a device with a missing service
//	 * and attempt to load the service with the specified URN.
//	 */
//
////	protected void tryLoadingSoftwareComponentAsync(Assembly assembly, AssemblyInfo ai, URN urn) {
////	// Lookup device declaration in assembly info
////	DeviceDecl dev = ai.lookupDevice(urn.getDeviceURN());
////	if(dev != null) {
////	URN swcmanurn = dev.getURN().createSubUrn(
////	new URNLeaf(SoftwareComponentManagerConstants.URN_SUFFIX));
////	ConnectionInfo ci = new ConnectionInfo(swcmanurn, assembly.getServiceInfo().getURN());
////	ConnectionInfo current = (ConnectionInfo) remoteInfoHandler
////	.getInfoThatIsAboutTheSameEntity(ci);
////	// check for or establish connection to the SoftwareComponentManager
////	if (current == null) {
////	// Establish connection
////	context.getRemoteConnectThread().putEvent(new InfoEvent(ci, true));
////	//FIXME: this should be divided into two handlers, the second part
////	//triggered by discovery of the requested connection...
////	}
////	// Send the load command
////	ServiceProxy sp = assembly.getServiceProxyForUrn(swcmanurn);
////	if (sp != null) {
////	ControlItem tmp = sp.findItem("load");
////	if(tmp instanceof Command) {
////	Command command = (Command) tmp;
////	command.getParam(0).setData(urn.getString().getBytes());
//////	logMsg("[OUT] trying to load SWC "+urn);
////	DebugDispatcher.debug("ResourceManager trying to load (async) SWC "+urn, DebugDispatcher.CAT_RESOURCE);
////	command.invoke();
////	} else {
////	DebugDispatcher.debug("tryLoading (async)...:Not a command "+tmp, DebugDispatcher.CAT_RESOURCE);
////	}
////	}
////	// Close connection
////	context.getRemoteConnectThread().putEvent(new InfoEvent(ci, false));
////	}
////	}
//	protected void tryLoadingSoftwareComponent(Assembly assembly, PRDAssemblyVer ai, ServiceInstanceID siid) {
//		throw new Error("ResourceManager3.tryLoadingSoftwareComponent broken by URN refactoring");
////		// Lookup device declaration in assembly info
////		DeviceDecl dev = ai.lookupDevice(urn.getDeviceURN());
////		if(dev != null) {
////		URN swcmanurn = dev.getURN().createSubUrn(
////		new URNLeaf(ServiceManagerConstants.URN_SUFFIX));
////		PRDConnection ci = new PRDConnection(swcmanurn, assembly.getServiceInfo().getURN());
//
////		//XXX
////		if(!remoteInfoHandler.connectionIsPossible(ci)) return;
//
////		// This call is blocking...
////		rch.doConnect(ci, 5000, 20000);
//
////		// Send the load command
////		ServiceProxy sp = assembly.getServiceProxyForUrn(swcmanurn);
////		if (sp != null) {
////		ControlItem tmp = sp.findItem("load");
////		if(tmp instanceof Command) {
////		Command command = (Command) tmp;
////		command.getParam(0).setData(urn.getString().getBytes());
//////		logMsg("[OUT] trying to load SWC "+urn);
////		DebugDispatcher.debug("ResourceManager trying to load SWC "+urn, DebugDispatcher.CAT_RESOURCE);
////		command.invoke();
////		} else {
////		DebugDispatcher.debug("tryLoading...:Not a command "+tmp, DebugDispatcher.CAT_RESOURCE);
////		}
////		}
////		// Close connection
////		try {
////		context.getRemoteConnectManager().requestDisconnect(ci);
////		} catch (IOException e) {
////		e.printStackTrace();
////		}
////		}
//	}
//
//	/**
//	 * A thread that receives ResourceRequests, and sets up or
//	 * terminates handler threads for them 
//	 */
//	private class ResourceThread extends PalcomPriorityThread {
//
//		public void run() {
//			while (true) {
//				Event tmpe = waitEvent();
//				if(tmpe instanceof ResourceRequestEvent) {
//					DebugDispatcher.debug("ResourceManager :"+tmpe,DebugDispatcher.CAT_RESOURCE);
//					processResourceRequest((ResourceRequestEvent) tmpe);
//				} 
//			}
//		}
//
//	}
//
//	private class InfoThread extends PalcomPriorityThread {
//		public void run() {
//			//XXX Hack: the remoteInfoHandler uses callbacks. Forward events
//			//to the resourceThread
////			remoteInfoHandler.addInfoListener( new InfoListener() {
//
////			public void infoReceived(InfoEvent event) {
////			resourceThread.putEvent(event);
////			}				
////			});
//			while (true) {
//				InfoEvent event = (InfoEvent) waitEvent();
//				remoteInfoHandler.infoReceived(event);
//				if(event.getInfo() instanceof PRDDevice) {
//					DeviceID deviceID = ((PRDDevice)event.getInfo()).getDeviceID();
//					context.getResourceManager2().updatePRDServiceListReceiver(new LocalSID(deviceID, null), this);
//					context.getResourceManager2().updatePRDConnectionReceiver(deviceID, this);
//				}
//			}
//		}
//	}
//
//	/** aux method
//	 * 
//	 * take care of differences between single and alternative declarations
//	 */
//	private void doProcessConnectionDecl(ConnectionDecl connectionDecl, ResourceRequestEvent rre) {
//		ServiceExp prov = connectionDecl.getProvider();
//		ServiceExp cust = connectionDecl.getCustomer();
//		try {
//			DebugDispatcher.debug("ResourceManager.doProcessConnectionDecl: "+connectionDecl+"("+rre.isActive()+")", DebugDispatcher.CAT_RESOURCE);
//			if(rre.isActive()) {
//				PalcomThread t;
//				if(servicesAreSingle(prov, cust)) {
//					SingleConnectionRequest singleConnectionRequest = new SingleConnectionRequest(connectionDecl, rre.getAssembly(), rre.getReceiver());
//					t = new ConnectionRequestThread(singleConnectionRequest);
//				} else {
//					AltConnectionRequest altConnectionRequest = new AltConnectionRequest(connectionDecl, rre.getAssembly(), rre.getReceiver());
//					t = new ConnectionRequestThread(altConnectionRequest);
//				}
//				context.getScheduler().scheduleThread(t);
//				connectionDeclToThread.put(connectionDecl, t);
//			} else {
//				cancelConnectionRequest(connectionDecl);
//			} 
//		} catch(IllegalArgumentException e) {
//			DebugDispatcher.debug("ResourceManager.doProcessConnectionDecl: "+e.getMessage(), DebugDispatcher.CAT_RESOURCE);
//		} catch(Exception e) {
//			DebugDispatcher.debug("ResourceManager.doProcessConnectionDecl: ("+prov+", "+cust+")"+e, DebugDispatcher.CAT_RESOURCE);
//			e.printStackTrace();
//		}
//	}
//
//	protected void processResourceRequest(ResourceRequestEvent rre) {
//		PRDData d = rre.getDeclaration();
//		if(d instanceof ConnectionDecl) {
//			doProcessConnectionDecl((ConnectionDecl)d, rre);
//		} else if( d instanceof ConnectionDeclList) {
////			System.out.println("ResourceManager.addResourceRequest:"+d);
//			ConnectionDeclList cdl = (ConnectionDeclList) d;
//			for(int i=0; i<cdl.getNumConnectionDecl(); i++){
//				ConnectionDecl connectionDecl = cdl.getConnectionDecl(i);
//				doProcessConnectionDecl(connectionDecl, rre);
//			}
//		} else {
//			DebugDispatcher.debug("ResourceManager.addResourceRequest: unknown type:"+d, DebugDispatcher.CAT_RESOURCE);
//		}
//		// We want events also for already discovered services
//		//XXX Only do this if a request was actually added
//		remoteInfoHandler.fireAllInfos();
//	}
//
//	private void cancelConnectionRequest(ConnectionDecl decl) {
//		PalcomThread t = (PalcomThread) connectionDeclToThread.remove(decl);
//		if(t != null) {
//			DebugDispatcher.debug("ResourceManager.cancelConnectionRequest: Killing thread for "+decl.getAssemblyLabel(), DebugDispatcher.CAT_RESOURCE);
//			t.terminate();
//		}
//	}
//
//	/** Check if a pair of serviceExp:s refer to single (i.e., not alternative) declarations 
//	 */
//	public boolean servicesAreSingle(ServiceExp prov, ServiceExp cust) {
//		boolean provIsSingle = serviceExpRefersToSingle(prov);
//		boolean custIsSingle = serviceExpRefersToSingle(cust);
//		return provIsSingle && custIsSingle;
//	}
//
//	public boolean serviceExpRefersToSingle(ServiceExp se) {
//		boolean seIsSingle=false;;
//		if(se instanceof ThisService) {
////			DebugDispatcher.debug("ResourceManager.serviceExpRefersToSingle: this=true", DebugDispatcher.CAT_RESOURCE);
//			seIsSingle = true;
//		} else if(se instanceof ServiceUse) {
//			seIsSingle = serviceUseRefersToSingle((ServiceUse)se);
////			DebugDispatcher.debug("ResourceManager.serviceExpRefersToSingle:"+se.toString()+"="+seIsSingle, DebugDispatcher.CAT_RESOURCE);
//		} else {
//			DebugDispatcher.debug("ResourceManager.serviceExpRefersToSingle:"+se.toString()+" not this or ServiceExp?", DebugDispatcher.CAT_RESOURCE);			
//		}
//		return seIsSingle;
//	}
//
//	private boolean serviceUseRefersToSingle(ServiceUse svc) {
//		if(svc != null) {
//			ServiceDecl decl = svc.getDecl();
//			if(decl != null) {
//				AbstractServiceDecl inner = decl.getDecl();
//				return inner instanceof SingleServiceDecl;
//			} else {
//				DebugDispatcher.debug("ResourceManager.serviceUseRefersToSingle: no declaration found for "+svc, DebugDispatcher.CAT_RESOURCE);
//				throw new IllegalArgumentException("No declaration found for Service "+svc.getIDforEditing());
//			}
//		}
//		return false;
//	}
//
//	private static ServiceProxy createServiceProxy(String controlName) {
//		ServiceProxy sd = new ServiceProxy(controlName);
//		return sd;
//	}
//
//	public void putEvent(ResourceRequestEvent event) {
//		resourceThread.putEvent(event);
//	}
//
//	/** 
//	 * Class representing a connection request 
//	 */
//	private abstract class ConnectionRequest {
//		private ConnectionDecl cd;
//		private Assembly assembly;
//		private PalcomThread recv;
//
//		ConnectionRequest(ConnectionDecl cd, Assembly assembly, PalcomThread recv) {
//			this.cd = cd;
//			this.assembly = assembly;
//			this.recv = recv;
//		}
//		public abstract void satisfied();
//
//		public abstract void unsatisfied();
//
//		public abstract PRDConnection getMatchingConnectionInfo(PRDData info);
//
//		public abstract boolean isActiveConnection(PRDConnection info);
//
//		ConnectionDecl getDeclaration() {
//			return cd;
//		}
//
//		Assembly getAssembly() {
//			return assembly;
//		}
//
//		PalcomThread getReceiver() {
//			return recv;
//		}
//
//		/**
//		 * Checks if an InfoEvent affects this ConnectionRequest
//		 * 
//		 * @param ie - the InfoEvent to check against
//		 * @return a reference to a ConnectionRequest that has now been fulfilled, or null 
//		 */
//
//		public abstract ConnectionRequest checkAgainstInfo(InfoEvent ie);
//
//		public abstract boolean isAlternative();
//
//		public String toString() {
//			return this.getClass().toString() + ":"+getDeclaration().toString();
//		}
//		public abstract boolean checkIfPossible();
//	}
//
//	private class SingleConnectionRequest extends ConnectionRequest {
//
//		private boolean sat;
//		SingleConnectionRequest(ConnectionDecl cd, Assembly assembly, PalcomThread recv) {
//			super(cd, assembly, recv);
//		}
//
//		/** Aux method
//		 * 
//		 * @param rr The Resource request to be examined
//		 * @param sl The disovered service list
//		 */
//		private void fireConnectionResponseIfPossible(PRDServiceList sl) {
//			throw new Error("ResourceManager3.fireConnectionResponseIfPossible broken by URN refactoring");
////			for(int i=0; i< sl.getNumPRDServiceListItem(); i++) {
////			URN discoveredURN = sl.getPRDServiceListItem(i).getURN();
////			ConnectionDecl cd = (ConnectionDecl) getDeclaration();
//////			DebugDispatcher.debug("fireConnectionResponseIfPossible:"+discoveredURN+ "("+cd.getProvider().getURN()+" | "+cd.getCustomer().getURN()+")", DebugDispatcher.CAT_RESOURCE);
//
////			// ConnectionDecl includes discovered service
////			if((cd.getProvider().getURN().equals(discoveredURN) || 
////			cd.getCustomer().getURN().equals(discoveredURN)) ) {
//////			DebugDispatcher.debug("***Match", DebugDispatcher.CAT_RESOURCE);
//
////			PRDConnection ci = cd.asInfo();
////			// Look up the current connectionInfo
////			PRDConnection current = (PRDConnection) remoteInfoHandler
////			.getInfoThatIsAboutTheSameEntity(ci);
//
////			if (current == null) {
//////			statusStr = statusStr+"\n"+"[**] "+cd.getProvider()+" --> "+cd.getCustomer();
//////			System.out.println("[NO] "+cd.getProvider()+" --> "+cd.getCustomer());
////			if(remoteInfoHandler.connectionIsPossible(ci)) {
//////			DebugDispatcher.debug("fireConnectionResponseIfPossible: putEvent...", DebugDispatcher.CAT_RESOURCE);
////			getReceiver().putEvent(new ResourceResponseEvent(cd, ci,ResourceResponseEvent.ADD));
////			} else {
//////			DebugDispatcher.debug("connection not possible --- strange..."+ci, DebugDispatcher.CAT_RESOURCE);							
////			}
////			} else {
//////			DebugDispatcher.debug("current != null, we should be connected...", DebugDispatcher.CAT_RESOURCE);													
////			}
////			}
////			}
//		}
//
//
//
//		public ConnectionRequest checkAgainstInfo(InfoEvent ie) {
////			System.err.println("checkAgainstInfo("+ie.getInfo().getClass().getName()+"):"+getDeclaration().getAssemblyLabel()+" -- sat == "+sat);
//			//XXX 
//			if(sat) {
//				return null; //Dont'check if already satisfied
//			}
//			ConnectionRequest result = null;
//			if((ie.getInfo() instanceof PRDServiceList))  {
//				PRDServiceList sl = (PRDServiceList) ie.getInfo();
//				fireConnectionResponseIfPossible(sl);
//			} else if(ie.getInfo() instanceof PRDConnection) {
//				ConnectionDecl cd = (ConnectionDecl) getDeclaration();
//				// A pending connection has been discovered
//				if(cd.asInfo().equals(ie.getInfo())) {
////					discoveredConnections.add(this);
//					result = this;
//				}
//			} else if ( (ie.getInfo() instanceof PRDDevice)) {
//				// For a new device, if we have a pending requests for a service on that
//				// device, which has not been discovered, we attempt to contact its 
//				// SoftwareComponentManager and see if it has a matching URL.
////				DebugDispatcher.debug("checkAgainstInfo..."+ie.getInfo()+"  [DEVICE]", DebugDispatcher.CAT_RESOURCE);
//				ConnectionDecl cd = (ConnectionDecl) getDeclaration();
//				ServiceExp prov = cd.getProvider();
//
//				if(prov instanceof ServiceUse) {
//					ServiceDecl provDecl = ((ServiceUse)prov).getDecl();
//					if(provDecl != null) {
//						SingleServiceDecl asd = (SingleServiceDecl) provDecl.getDecl();
////						XXX Broken
////						if(asd.getServiceAddress().getServiceInstanceID().getDeviceID().equals(((PRDDevice)ie.getInfo()).getDeviceID())) {
////						//FIXME: is this a good idea:
////						DebugDispatcher.debug("checkAgainstInfo("+asd.getAssemblyLabel()+"..."+ie.getInfo()+"  try loading...", DebugDispatcher.CAT_RESOURCE);							
////						PRDAssemblyVer assemblyInfo=cd.av();
////						tryLoadingSoftwareComponent(getAssembly(),assemblyInfo,asd.getServiceAddress().getServiceInstanceID());
////						}
//					}
//				}
//			} 
//			return result;
//		}
//
//		public PRDConnection getMatchingConnectionInfo(PRDData info) {
//			PRDConnection ci = getDeclaration().asInfo();
//			if( info.equals(ci)) {
//				return ci;
//			}
//			return null;
//		}
//
//		public boolean isAlternative() {
//			return false;
//		}
//
//		public void satisfied() {
//			sat=true;
//		}
//
//		public void unsatisfied() {
//			getReceiver().putEvent(new ResourceResponseEvent(getDeclaration(), getDeclaration().asInfo(),ResourceResponseEvent.REMOVE));
//			sat=false;
//		}
//
//		public boolean isActiveConnection(PRDConnection info) {
//			return getMatchingConnectionInfo(info) != null;
//		}
//
//		public boolean checkIfPossible() {
//			return remoteInfoHandler.connectionIsPossible(getDeclaration().asInfo());
//		}
//	}
//
//	private class AltConnectionRequest extends ConnectionRequest {
//
//		private AltServiceDecl currentAlternative;
//		private PRDConnection currentConnectionInfo;
//		private boolean sat;
//
//		AltConnectionRequest(ConnectionDecl cd, Assembly assembly, PalcomThread recv) {
//			super(cd, assembly, recv);
//		}
//
//		/** Aux method	
//		 * 
//		 * @param rr The Resource request to be examined
//		 * @param sl The disovered service list
//		 */
//		private boolean fireConnectionResponseIfPossible(AltServiceDecl asd, PRDServiceList sl, boolean altProvider) {
//			throw new Error("ResourceManager3.fireConnectionResponseIfPossible broken by URN refactoring");
////			boolean result = false;
////			for(int i=0; i< sl.getNumPRDServiceListItem(); i++) {
////			URN discoveredURN = sl.getPRDServiceListItem(i).getURN();
////			URN asdURN = asd.getURN();
////			// ConnectionDecl includes discovered service
//////			DebugDispatcher.debug("AltConnectionRequest.fireConnectionResponseIfPossible: disovered service:"+discoveredURN, DebugDispatcher.CAT_RESOURCE);
////			if( asdURN.equals(discoveredURN) ) {
//////			DebugDispatcher.debug("...matches "+asd.getAssemblyLabel(),DebugDispatcher.CAT_RESOURCE);
////			PRDConnection ci = connectionInfoForASD(altProvider, getDeclaration(), asd);
////			// Look up the current connectionInfo
////			PRDConnection current = (PRDConnection) remoteInfoHandler.getInfoThatIsAboutTheSameEntity(ci);
//
////			if (current == null) {
//////			statusStr = statusStr+"\n"+"[**] "+cd.getProvider()+" --> "+cd.getCustomer();
//////			System.out.println("[NO] "+cd.getProvider()+" --> "+cd.getCustomer());
////			if(remoteInfoHandler.connectionIsPossible(ci)) {
////			if(currentAlternative == null) {
//////			DebugDispatcher.debug("AltConnectionRequest.fireConnectionResponseIfPossible: no previous alternative:", DebugDispatcher.CAT_RESOURCE);
////			currentAlternative = asd;
////			currentConnectionInfo = ci;
////			getReceiver().putEvent(new ResourceResponseEvent(getDeclaration(), ci,ResourceResponseEvent.ADD));
////			} else {
//////			DebugDispatcher.debug("AltConnectionRequest.fireConnectionResponseIfPossible: previous priority:"+currentAlternative.getPriority()+" ["+asd.getPriority()+"]", DebugDispatcher.CAT_RESOURCE);
////			if(asd.equals(currentAlternative)) {
////			//XXX this is meant to mean "retry". does it?
////			getReceiver().putEvent(new ResourceResponseEvent(getDeclaration(), ci, null,ResourceResponseEvent.ADD));																	
////			} else if(asd.getPriority() < currentAlternative.getPriority()) {
////			PRDConnection oldCi = currentConnectionInfo;
////			currentAlternative = asd;
////			currentConnectionInfo = ci;
////			sat = false;
////			getReceiver().putEvent(new ResourceResponseEvent(getDeclaration(), ci, oldCi,ResourceResponseEvent.CHANGE));								
////			} 
//
////			}
////			result =true;
////			} 
////			}
////			}
////			}
////			return result;
//		}
//
//		public synchronized ConnectionRequest checkAgainstInfo(InfoEvent ie) {
//			ConnectionRequest result = null;
//			AltServiceDeclList asdl = null;
//			boolean altProvider = false;
//
//			ConnectionDecl cd = getDeclaration();
//			try {
//				if(!serviceExpRefersToSingle(cd.getProvider())) {
//					asdl = (AltServiceDeclList)cd.getProvider().getDecl().getDecl();
//					altProvider = true;
//				} else if(!serviceExpRefersToSingle(cd.getCustomer())) {
//					//use else if just in case: one has to be true, or else
//					//this should be a SingleConnectionRequest: Perhaps add a
//					//check for this in the constructor?
//					asdl = (AltServiceDeclList)cd.getCustomer().getDecl().getDecl();
//					altProvider = false;
//				}
//			} catch (Exception e) {
//				DebugDispatcher.debug("AltConnectionRequest.checkAgainstInfo:"+e.toString(), DebugDispatcher.CAT_RESOURCE);
//			}
//			// just in case
//			if(asdl==null) {
//				DebugDispatcher.debug("AltConnectionRequest.checkAgainstInfo: adsl == null", DebugDispatcher.CAT_RESOURCE);
//				return null;
//			}
////			for(int i=0; i<asdl.getNumServiceDecl();i++) {
////			AltServiceDecl asd = asdl.getServiceDecl(i);
//			ArrayList al = asdl.getChildrenInPriorityOrder();
//			for (Iterator iter = al.iterator(); iter.hasNext();) {
//				AltServiceDecl asd = (AltServiceDecl) iter.next();
//
//				if((ie.getInfo() instanceof PRDServiceList))  {
//					PRDServiceList sl = (PRDServiceList) ie.getInfo();
//					if(fireConnectionResponseIfPossible(asd, sl, altProvider))
//						break;
//				} else if(ie.getInfo() instanceof PRDConnection) {
//					PRDConnection asdInfo;
//					asdInfo = connectionInfoForASD(altProvider, cd, asd);
//					// A pending connection has been discovered
//					if(!sat && asdInfo.equals(ie.getInfo())) {
//						result = this;
//						break;
//					}
//				}
//			}
//			return result;
//		}
//		// XXX There's some redundant code  here (related to the .asInfo() equivalent).
//
//		private PRDConnection connectionInfoForASD(boolean altProvider, ConnectionDecl cd, AltServiceDecl asd) {
//			throw new Error("ResourceManager3.ACR.connectionInfoForASD broken by URN refactoring");
////			PRDConnection asdInfo;
////			if(altProvider) {
////			asdInfo = new PRDConnection(asd.getURN(), cd.getCustomer().getURN());
////			} else {
////			asdInfo = new PRDConnection(cd.getProvider().getURN(), asd.getURN());						
////			}
////			return asdInfo;
//		}
//
//		public PRDConnection getMatchingConnectionInfo(PRDData info) {
//			ConnectionDecl cd = getDeclaration();
//			AltServiceDeclList asdl=null;
//			boolean altProvider=false;;
//			if(!serviceExpRefersToSingle(cd.getProvider())) {
//				asdl = (AltServiceDeclList)cd.getProvider().getDecl().getDecl();
//				altProvider = true;
//			} else if(!serviceExpRefersToSingle(cd.getCustomer())) {
//				//use else if just in case: one has to be true, or else
//				//this should be a SingleConnectionRequest: Perhaps add a
//				//check for this in the constructor?
//				asdl = (AltServiceDeclList)cd.getCustomer().getDecl().getDecl();
//				altProvider = false;
//			} else {
//				// This is an error
//				DebugDispatcher.debug("AltConnectionRequest.checkAgainstInfo: neither provider nor customer is alternative???",DebugDispatcher.CAT_RESOURCE);
//				return null;
//			}
//			//XXX HERE BE DRAGONS!
//			//If there is a currently active alternative, only match against that
//			//(This is to avoid triggering duplicate alternatives. In particular, we
//			//don't care about "replaced" connections going down.)
////			if(currentAlternative!=null) {
////			ConnectionInfo ci = connectionInfoForASD(altProvider, cd, currentAlternative);
//			if(currentConnectionInfo != null) {
//				if(currentConnectionInfo.equals(info))
//					return currentConnectionInfo;				
//			}else{
//				for(int i=0; i<asdl.getNumServiceDecl();i++) {
//					AltServiceDecl asd = asdl.getServiceDecl(i);
//					PRDConnection ci = connectionInfoForASD(altProvider, cd, asd);
//					if(ci.equals(info))
//						return ci;
//				}
//			} 
//			return null;
//		}
//
//		public boolean isAlternative() {
//			return true;
//		}
//
//		public synchronized void satisfied() {
//			sat = true;
//		}
//
//		public synchronized void unsatisfied() {
//			getReceiver().putEvent(new ResourceResponseEvent(getDeclaration(), currentConnectionInfo,ResourceResponseEvent.REMOVE));
//			currentAlternative = null;
//			currentConnectionInfo = null;
//			sat = false;
//		}
//
//		public boolean isActiveConnection(PRDConnection info) {
//			return info.isAboutTheSameEntityAs(currentConnectionInfo);
//		}
//
//		public boolean checkIfPossible() {
//			//XXX NB! checks only *current* connection
//			return currentConnectionInfo != null && remoteInfoHandler.connectionIsPossible(currentConnectionInfo);
//		}
//	}
//
//	/** A class responsible for monitoring and acting in response
//	 * to changes to the status of a single connection request.
//	 * 
//	 * @author sven
//	 *
//	 */
//	private class ConnectionRequestThread extends PalcomPriorityThread {
//
//		private static final int PERIOD = 1000;
//
//		ConnectionRequest cr;
//
//		public ConnectionRequestThread(ConnectionRequest connectionRequest) {
//			cr = connectionRequest;
//		}
//
//		public void run() {
////			boolean established=false;
//			cr.getDeclaration().getCustomer().listenForDiscovery(context.getResourceManager2(), this, true);
//			cr.getDeclaration().getProvider().listenForDiscovery(context.getResourceManager2(), this, true);
////			XXX			context.getResourceManager2().updatePRDConnectionReceiver(cr.getDeclaration().getCustomer().getServiceAddress().getServiceInstanceID().getDeviceID(), this);
////			context.getResourceManager2().updatePRDConnectionReceiver(cr.getDeclaration().getProvider().getServiceAddress().getServiceInstanceID().getDeviceID(), this);
//			enablePeriodicEvents(PERIOD);
//			while(true) {
//				Event e = waitEvent();
//				if(e instanceof InfoEvent) {
//					InfoEvent ie = (InfoEvent)e;
////					System.err.println("CRT: "+cr.getDeclaration().getAssemblyLabel()+"--"+ie.getInfo().getClass().getSimpleName());
//
//
//					if(ie.infoShouldBeKept()) {
//						ConnectionRequest req = cr.checkAgainstInfo(ie);
//						if(req != null) {
////							established = true;
////							System.out.println("ConnectionRequestThread: fulfilled: "+req.getDeclaration().asInfo());
//							cr.satisfied();
//						}
//					}else if(ie.getInfo() instanceof PRDConnection && !ie.infoShouldBeKept()) {
////						established = false;
//						if(cr.isActiveConnection((PRDConnection) ie.getInfo())) {
////							System.out.println("ConnectionRequestThread: connection went down: "+cr.getDeclaration().asInfo());
//							cr.unsatisfied();
//						}
//
//					} else if(ie.getInfo() instanceof PRDDevice && !ie.infoShouldBeKept()) {
//						//This is a fix for DiscoveryManager failing to send
//						//events for disconnection caused by removal of a service						
////						System.err.println("**** Device removed");
//						if(!cr.checkIfPossible()) {
////							System.err.println("***********NOT POSSIBLE!");
//							cr.unsatisfied();
//						}
//					}
//				} else if (e instanceof PeriodicEvent) {
//					// Check all known infos periodically
//					for (int i = 0; i < remoteInfoHandler.getNumberOfInfos(); i++) {
//						PRDData info = remoteInfoHandler.getInfo(i);
//						ConnectionRequest req = cr.checkAgainstInfo(new InfoEvent(info, true));
//						if(req != null) {
//							cr.satisfied();
//						}
//					}
//				}
//			}
//		}
//
//		public void terminate() {
//			cr.getDeclaration().getCustomer().listenForDiscovery(context.getResourceManager2(), this, false);
//			cr.getDeclaration().getProvider().listenForDiscovery(context.getResourceManager2(), this, false);
////			context.getResourceManager2().removePRDConnectionReceiver(cr.getDeclaration().getCustomer().getServiceAddress().getServiceInstanceID().getDeviceID(), this);
////			context.getResourceManager2().removePRDConnectionReceiver(cr.getDeclaration().getProvider().getServiceAddress().getServiceInstanceID().getDeviceID(), this);
//
//			super.terminate();
//		}
//	}
//}
