/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.base.networking;

/**
 * This class implements core.BluetoothPacket
 * 
 * @author Jacob Frolund Pedersen, frolund@daimi.au.dk
 */

public class BluetoothPacket {

	String receiverHost = "";
	
	String senderHost = "";
	
	byte[] data;

	/**
	 * 
	 * @param data - the data in the package
	 * @param host - address of the receiving host
	 */
	
    public BluetoothPacket(byte[] data, String host) {
           this.receiverHost = host;
           this.data = data;
    }
    
    public BluetoothPacket(){
    	
    }

    public byte[] getData() {
        return data;
    }
    
    public int getDataLength(){
    	return data.length;
    }

    public String getReceiverHost() {
        return receiverHost;
    }
    
    public void setReceiverHost(String address){
    	this.receiverHost = address;
    }

    public byte[] toByteRepresentation(){
    	
    	String sLength = Integer.toString(data.length);
    	byte[] tmp = new byte[1+receiverHost.getBytes().length+1+senderHost.getBytes().length+1+sLength.getBytes().length+data.length];
    	
    	tmp[0] = new Integer(receiverHost.getBytes().length).byteValue();
    	java.lang.System.arraycopy(receiverHost.getBytes(),0,tmp,1,receiverHost.getBytes().length);
    	
    	tmp[receiverHost.getBytes().length+1] = new Integer(senderHost.getBytes().length).byteValue();
    	java.lang.System.arraycopy(senderHost.getBytes(),0,tmp,receiverHost.getBytes().length+2,senderHost.getBytes().length);
    		
    	tmp[receiverHost.getBytes().length+1+senderHost.getBytes().length+1] = new Integer(sLength.getBytes().length).byteValue();    	
    	java.lang.System.arraycopy(sLength.getBytes(),0,tmp,receiverHost.getBytes().length+senderHost.getBytes().length+3,sLength.getBytes().length);
    	java.lang.System.arraycopy(data,0,tmp, receiverHost.getBytes().length+1+senderHost.getBytes().length+1+sLength.getBytes().length+1,data.length);
    	
    	return tmp;
    }
    
    public void fromByteRepresentation(byte[] byteArray){
    	byte[] receiverHostArray = new byte[new Integer(byteArray[0]).intValue()];
    	java.lang.System.arraycopy(byteArray,1,receiverHostArray,0,receiverHostArray.length);
    	receiverHost = new String(receiverHostArray);
    	
    	byte[] senderHostArray = new byte[new Integer(byteArray[receiverHostArray.length+1]).intValue()];
    	java.lang.System.arraycopy(byteArray,senderHostArray.length+2,senderHostArray,0,senderHostArray.length);
    	senderHost = new String(senderHostArray);
    	
    	byte[] dataLengthArray = new byte[new Integer(byteArray[receiverHostArray.length+1+senderHostArray.length+1]).intValue()];
    	java.lang.System.arraycopy(byteArray,receiverHostArray.length+1+senderHostArray.length+2,dataLengthArray,0,dataLengthArray.length);
    	int dataLength = Integer.parseInt(new String(dataLengthArray));
    	
    	data = new byte[dataLength];
    	java.lang.System.arraycopy(byteArray,receiverHostArray.length+1+senderHostArray.length+1+dataLengthArray.length+1,data,0,data.length);
    }
    
    public int getLength() {
        return data.length;
    }
    
    public void setLength(int length){
    	data = new byte[length];
    }

	public String getSenderHost() {
		return senderHost;
	}

	public void setSenderHost(String senderHost) {
		this.senderHost = senderHost;
	}
}
