/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.base.networking;

import java.io.IOException;

import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import javax.microedition.io.StreamConnectionNotifier;

/**
 * Bluetooth BTSPP (Serial Port Profile) server socket.
 * 
 * @author Henrik Gammelmark, geemark@daimi.au.dk
 */
public class BluetoothSPPSocket {
	private static BluetoothSPPSocket instance;
	private BluetoothSPPListener listener;
	private Thread serverThread;
	
	/**
	 * Singleton
	 */
	private BluetoothSPPSocket() {
	}
	
	public static BluetoothSPPSocket getInstance() {
		if (instance == null)
			instance = new BluetoothSPPSocket();
		return instance;
	}
	
	public void start(BluetoothSPPListener listener) {
		this.listener = listener;
		
		serverThread = new Thread(new SPPServerThread());
		serverThread.start();
	}
	
	public void stop() {
		listener = null;
		serverThread.interrupt();
	}
	
	private class SPPServerThread implements Runnable {
		public void run() {
			StreamConnectionNotifier notifier = null;
			StreamConnection conn = null;
			
			try {
				String url = "btspp://localhost:00112233445566778899AABBCCDDEEFF;name=PalCom SPP Server";
				notifier = (StreamConnectionNotifier) Connector.open(url);
				
				while (!Thread.interrupted()) {
					conn = notifier.acceptAndOpen();
					listener.onEstablish(new BluetoothSPPConnection(conn));
				}
				
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
}
