/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.device;

import ist.palcom.palcomthreads.Event;
import ist.palcom.palcomthreads.PalcomThread;

/**
 * Interface specifying the I/O interface between the 
 * (simulated) hardware and the palcom device/service code.
 * 
 * The interface specifies operations to allow both event-based
 * and polling I/O, in both directions.
 * 
 * @author sven
 *
 */
public interface IDeviceIO {

	/* Methods called by Palcom side */

	/**
	 * Send an event to the hardware from the palcom side code.
	 * If listeners have been registered using addOutEventListener,
	 * the hardware-side event handler call-back may run in the 
	 * context of the palcom thread that calls putOutEvent. 
	 * 
	 */
	void putOutEvent(Event e);
	
	/**
	 * non-blocking polling of input events
	 * 
	 */
	Event getInEvent();
	
	/**
	 * blocking polling of input events
	 * 
	 */
	Event waitInEvent();

	/**
	 * register the receiver for events from the hardware side.
	 * N.B.! There can be at most one receiver thread on the
	 * PalcomThread side.
	 */
		
	void setInEventReceiver(PalcomThread t);

	/**
	 * unregister the receiver for events from the hardware side.
	 */
	
	void removeInEventReceiver();
	
    /** close and do cleanup. */
    void close();
    
	/* Methods called by hardware/simulator side */
	
	/**
	 *  send an event from the hardware to the palcom side code
	 */
	void putInEvent(Event e);
	
	/**
	 * Register a listener for events from the palcom side. 
	 * On the hardware side, a set of listeners is allowed. 
	 */
	
	void addOutEventListener(OutEventListener l);

	/**
	 * unregister a listener for events from the palcom side.
	 */
	
	void removeOutEventListener(OutEventListener l);

	/**
	 * non-blocking polling of input events
	 * 
	 */
	
	Event getOutEvent();

	/**
	 * blocking polling of input events
	 * 
	 */
	Event waitOutEvent();
}
