package ist.palcom.hgraph;

import java.util.Iterator;

/**
 * Abstractinvokator implements the basic functionality of Invokator, such as invoke and log
 */
public abstract class AbstractInvokator implements Invokator{
	
	private int numLogs;
	private int writeNum = 0;
	private int actualNum = 0;
	public static final int DEFAULT_NUM_LOGS = 100;
	private HgraphEvent[] logs;
	private IHgraphNode node;
	private String name;
	private String description;
	
	
	public AbstractInvokator(IHgraphNode node, String name, String description){
		this(DEFAULT_NUM_LOGS, node, name, description);
	}
	
	public AbstractInvokator(int numLogs, IHgraphNode node, String name, String description){
		this.numLogs = numLogs;
		logs = new HgraphEvent[numLogs];
		this.node = node;
		this.name = name;
		this.description = description;
	}

	public void invoke(String command, Path path, String returnCommand, Value value){
		HgraphEvent tmp = new HgraphEvent(path, command, returnCommand, value);
		log(tmp);
		node.getFuncFunc().invoke(this, tmp);
	}
	
	protected void log(HgraphEvent event){
		logs[writeNum] = event;
		if(writeNum == (numLogs - 1)){
			writeNum = 0;
		}else{
			writeNum++;
		}
		if(actualNum<numLogs){
			actualNum++;
		}
	}
	public String getDescription() {
		return description;
	}

	public String getName() {
		return name;
	}
	
	public Iterator getLogIterator(){
		return new LogIterator(numLogs, actualNum, writeNum);
	}

	public class LogIterator implements Iterator{

		private int numLogs;
		private int actualNum;
		private int writeNum;
		
		private int ctr = 0;
		
		public LogIterator(int numLogs, int actualNum, int writeNum){
			this.numLogs = numLogs;
			this.actualNum = actualNum;
			this.writeNum = writeNum;
		}
		
		public boolean hasNext() {
			if(ctr > actualNum)
				return true;
			return false;
		}

		public Object next() {
			HgraphEvent tmp = logs[((ctr + writeNum)%numLogs) - 1];
			ctr++;
			return tmp;
		}

		public void remove() {}
	}
}
