/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.hgraph;

/**
 * A Handler is the Hgraph functional equivalent to a provided method.
 * Handlers are invoked through the structure of the Hgraph.
 * They are invoked with a single parameter, an Hgraph event, holding the sender and recipient paths,
 * a command string for the handler to act upon and possibly a value. This value may be simple or complex
 * Handlers can handle multiple different commands
 */
public interface Handler {

	/**
	 * The handle method is invoked by the HgraphNode in which the handler is installed.
	 * This happens once the node receives an event for which it is the designatee.
	 * The handler may freely switch on the command string of the event and act accordingly
	 */
	public void handle(HgraphEvent event);
	
	/**
	 * @return The name of the handler.
	 */
	public String getName();
	
	/**
	 * @return A description of the purpose of the handler in prose. It should as a minimum describe the commands
	 * the handler is able to handle and the effects of it handling those
	 */
	public String getDescription();
	
	/**
	 * @return The signatures of the commands handled by the handler
	 */
	public Signature[] getSignatures();	
}
