/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.hgraph;

import ist.palcom.base.StringBuilder;
import ist.palcom.palcomthreads.Event;

/**
 * HgraphEvents are used for message passing in h-graphs.
 * They consist of a sender path, a recipient path, a message command string and possibly a parameter value
 */
public class HgraphEvent extends Event{

	private String command;
	private String returnCommand;
	private Path fromPath;
	private Path toPath;
	private Value value;

	/**
	 * Creates a HgraphEvent from a recipient path, a command string and a value
	 * @param toPath
	 * @param command
	 * @param value
	 */
	public HgraphEvent(Path toPath, String command, Value value){
		this.toPath = toPath;
		this.command = command;
		this.value = value;
	}
	
	/**
	 * Creates a HgraphEvent from a recipient path, a command string, a return comand and a value
	 * @param toPath
	 * @param command
	 * @param value
	 */
	public HgraphEvent(Path toPath, String command, String returnCommand, Value value){
		this.toPath = toPath;
		this.command = command;
		this.value = value;
		this.returnCommand = returnCommand;
	}

	/**
	 * Creates a HgraphEvent from a recipient path and a command string
	 * @param toPath
	 * @param command
	 */
	public HgraphEvent(Path toPath, String command){
		this.toPath = toPath;
		this.command = command;
	}
	
	/**
	 * Creates a HgraphEvent from a recipient path, a command string and a return command
	 * @param toPath
	 * @param command
	 */
	public HgraphEvent(Path toPath, String command, String returnComand){
		this.toPath = toPath;
		this.command = command;
		this.returnCommand = returnComand;
	}
	
	/**
	 * @return The command of the event
	 */
	public String getCommand(){
		return command;
	}
	
	/**
	 * @return The return command of the event. Null if none exists
	 */
	public String getReturnCommand(){
		return returnCommand;
	}
	
	/**
	 * @return The path of the sender
	 */
	public Path getFromPath(){
		return fromPath;
	}
	
	/**
	 * @return The path of the recpient
	 */
	public Path getToPath(){
		return toPath;
	}
	
	/**
	 * Sets the sender path. This is done as a node is invoked with this event
	 * @param fromPath
	 */
	public void setFromPath(Path fromPath){
		this.fromPath = fromPath;
	}
	
	/**
	 * Sets the recipient path.
	 * @param toPath
	 */
	public void setToPath(Path toPath){
		this.toPath = toPath;
	}
	
	/**
	 * @return The value parameter of the event. It may be null
	 */
	public Value getValue(){
		return value;
	}
	
	/**
	 * Sets the value parameter of the event. It may be null
	 * @param value
	 */
	public void setValue(Value value){
		this.value = value;
	}		
}
