/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.hgraph;


/**
 * NodeHandler handles the basic node commands related to children and naming
 *
 */
public class NodeHandler extends AbstractHandler{

	public static final String name = "Node Handler";
	public static final String description = "Handles names and children";
	private Signature[] handlerSignatures;
	
	/**
	 * Command for setting the name of the node. 
	 */
	public static final String CMD_SETNAME = "cmd_setname";
	
	/**
	 * Command for removing a specific child from the node
	 */
	public static final String CMD_REMOVECHILD = "cmd_removechild";
	
	/**
	 * Command for cheching whether the node has a child with a certain name
	 */
	public static final String CMD_HASCHILD = "cmd_haschild";
	
	/**
	 * Command for creating and adding a new child to the node
	 */
	public static final String CMD_NEWCHILD = "cmd_newchild";
	
	/**
	 * Command for adding a child to the node
	 */
	public static final String CMD_ADDCHILD = "cmd_addchild";
	
	/**
	 * Command for setting the parent of the node
	 */
	public static final String CMD_SETPARENT = "cmd_setparent";
	
	private HgraphNode node;
	
	public NodeHandler(HgraphNode node){
		super(name, description);
		this.node = node;
		handlerSignatures = new Signature[6];
		handlerSignatures[0] = new Signature(new String[]{"string"}, CMD_SETNAME, "int");
		handlerSignatures[1] = new Signature(new String[]{"string"}, CMD_REMOVECHILD, "bolean");
		handlerSignatures[2] = new Signature(new String[]{"string"}, CMD_NEWCHILD, "int");
		handlerSignatures[3] = new Signature(new String[]{"string"}, CMD_HASCHILD,"int");
		handlerSignatures[4] = new Signature(new String[]{"IHgraphNode"}, CMD_SETPARENT,"int");
		handlerSignatures[5] = new Signature(new String[]{"IHgraphNode"}, CMD_ADDCHILD, "int");
	}
	
	public Signature[] getSignatures(){
		return handlerSignatures;
	}
	
	public void handle(HgraphEvent event) {
		if(event.getCommand().equals(CMD_NEWCHILD) && event.getValue().getValue() instanceof String){
			int tmp = node.addNodeChild(new HgraphNode((String)event.getValue().getValue()), event.getFromPath());
			if(event.getReturnCommand() != null){
				node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value(CMD_NEWCHILD, new Integer(tmp)));
			}
		}else if(event.getCommand().equals(CMD_REMOVECHILD)){
			int tmp = node.removeNodeChild((String)event.getValue().getValue(), event.getFromPath());			
			if(event.getReturnCommand() != null){
				node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value(CMD_REMOVECHILD, new Integer(tmp)));
			}
		}else if(event.getCommand().equals(CMD_HASCHILD) && event.getReturnCommand() != null){
			boolean tmp = node.hasChildNamed((String)event.getValue().getValue());
			node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value(CMD_HASCHILD, new Boolean(tmp)));
		}else if(event.getCommand().equals(CMD_SETNAME)){
			int tmp = node.setNodeName((String)event.getValue().getValue(), event.getFromPath());
			if(event.getReturnCommand() != null){
				node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value(CMD_SETNAME, new Integer(tmp)));
			}
		}else if(event.getCommand().equals(CMD_ADDCHILD)){
			int tmp = node.addNodeChild((IHgraphNode)event.getValue().getValue(), event.getFromPath());
			if(event.getReturnCommand() != null){
				node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value(CMD_REMOVECHILD, new Integer(tmp)));
			}			
		}else if(event.getCommand().equals(CMD_SETPARENT)){
			
		}
	}
}
