/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.hgraph;


/**
 * PrintHandler handles commands related to textual representations of the node (and its offspring)
 */
public class PrintHandler extends AbstractHandler{

	/**
	 * Returns a simple representation of the node and its children
	 */
	public static final String CMD_PRINT = "cmd_print";
	
	/**
	 * Returns a full representation of the node
	 */
	public static final String CMD_PRINT_FULL = "cmd_print_full";
	
	/**
	 * Returns a recursive simple representation of the node and its offspring
	 */
	public static final String CMD_PRINT_RECURSIVELY = "cmd_print_recursively";
	
	/**
	 * Returns a recursive full representation of the node and its offspring
	 */
	public static final String CMD_PRINT_FULL_RECURSIVELY = "cmd_print_full_recursively";

	private static final String description = "Handles commands for returning textual representations of the associated node";
	private static final String name = "Print Handler";
	private Signature[] handlerSignatures;
	
	private HgraphNode node;
	
	public PrintHandler(HgraphNode node){
		super(name, description);
		this.node = node;
		handlerSignatures = new Signature[4];
		handlerSignatures[0] = new Signature(new String[]{}, CMD_PRINT, "string");
		handlerSignatures[1] = new Signature(new String[]{}, CMD_PRINT_FULL, "string");
		handlerSignatures[2] = new Signature(new String[]{}, CMD_PRINT_FULL_RECURSIVELY, "string");
		handlerSignatures[3] = new Signature(new String[]{}, CMD_PRINT_RECURSIVELY, "string");
	}

	public Signature[] getSignatures(){
		return handlerSignatures;
	}
	
	public void handle(HgraphEvent event) {
		if(event.getCommand().equals(CMD_PRINT) && (event.getReturnCommand() != null)){
			String tmp = node.toXML();
			node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value(CMD_PRINT, tmp));
		}else if(event.getCommand().equals(CMD_PRINT_FULL) && (event.getReturnCommand() != null)){
			String tmp = node.toXMLFull();
			node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value(CMD_PRINT_FULL, tmp));
		}else if(event.getCommand().equals(CMD_PRINT_RECURSIVELY) && (event.getReturnCommand() != null)){
			String tmp = node.toXMLRecursively();
			node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value(CMD_PRINT_RECURSIVELY, tmp));
		}else if(event.getCommand().equals(CMD_PRINT_FULL_RECURSIVELY) && (event.getReturnCommand() != null)){
			String tmp = node.toXMLFullRecursively();
			node.getFuncFunc().getGenericInvokator().invoke(event.getReturnCommand(), event.getFromPath(), new Value(CMD_PRINT_RECURSIVELY, tmp));
		}
	}
}
