package ist.palcom.hgraph;

import java.util.Iterator;

import ist.palcom.base.StringBuilder;

public class XML {

	public static String INDENTATION = "  "; // = 2
	
	public static final String HANDLERS_TAG = "XNodeHandlers";
	public static final String INVOKATORS_TAG = "XNodeInvokators";
	public static final String LISTERNERS_TAG = "XNodeListeners";
	public static final String VALUES_TAG = "XNodeValues";
	public static final String CHILDREN_TAG = "XNodeChildren";
	public static final String LOGS_TAG = "XNodeInvokatorLogs";
	public static final String INPUT_TAG = "XNodeSignatureInputs";
	public static final String RETURNS_TAG = "XNodeSignatureOutputs";
	
	public static String toXML(IHgraphNode node){
		StringBuilder build = StringBuilder.instantiate();
		openNodeTag(node, build);
		childrenToXML(node, build, INDENTATION);
		valuesToXML(node, build, INDENTATION);
		closeNodeTag(node, build);
		return build.toString();
	}
		
	public static String toXMLRecursively(IHgraphNode node){
		StringBuilder build = StringBuilder.instantiate();
		openNodeTag(node, build);
		Iterator iter = node.getChildIterator();
		while(iter.hasNext()){
			IHgraphNode cNode = (IHgraphNode)iter.next();
			innerToXMLRecursively(cNode, build, INDENTATION);
		}
		valuesToXML(node, build, INDENTATION);
		closeNodeTag(node, build);
		return build.toString();		
	}
	
	private static void innerToXMLRecursively(IHgraphNode node, StringBuilder build, String indent){
		String newIndent = indent + INDENTATION;
		build.append(indent);
		openNodeTag(node, build);
		Iterator iter = node.getChildIterator();
		while(iter.hasNext()){
			IHgraphNode cNode = (IHgraphNode)iter.next();
			innerToXMLRecursively(cNode, build, newIndent);
		}
		valuesToXML(node, build, newIndent);
		build.append(indent);
		closeNodeTag(node, build);		
	}

	
	public static String toXMLFull(IHgraphNode node){
		StringBuilder build = StringBuilder.instantiate();
		openNodeTag(node, build);
		childrenToXML(node, build, INDENTATION);
		valuesToXML(node, build, INDENTATION);
		handlersToXML(node, build, INDENTATION);
		invokatorsToXML(node, build, INDENTATION);
		listenersToXML(node, build, INDENTATION);
		closeNodeTag(node, build);
		return build.toString();
	}
	
	public static String toXMLFullRecursively(IHgraphNode node){
		StringBuilder build = StringBuilder.instantiate();
		openNodeTag(node, build);
		Iterator iter = node.getChildIterator();
		while(iter.hasNext()){
			IHgraphNode cNode = (IHgraphNode)iter.next();
			innerToXMLFullRecursively(cNode, build, INDENTATION);
		}
		valuesToXML(node, build, INDENTATION);
		closeNodeTag(node, build);
		return build.toString();		
	}
	
	private static void innerToXMLFullRecursively(IHgraphNode node, StringBuilder build, String indent){
		String newIndent = indent + INDENTATION;
		build.append(indent);
		openNodeTag(node, build);
		Iterator iter = node.getChildIterator();
		while(iter.hasNext()){
			IHgraphNode cNode = (IHgraphNode)iter.next();
			innerToXMLFullRecursively(cNode, build, newIndent);
		}
		valuesToXML(node, build, indent);
		handlersToXML(node, build, indent);
		invokatorsToXML(node, build, indent);
		listenersToXML(node, build, indent);
		build.append(indent);
		closeNodeTag(node, build);		
	}
		
	private static void openNodeTag(IHgraphNode node, StringBuilder build){
		build.append("<");
		build.append(node.getClass().getName());
		build.append(" name=\"");
		build.append(node.getNodeName());
		if((node.getPath() != null) && (node.getPath().toString() != null)){
			build.append("\" path=\"");
			build.append(node.getPath().toString());
		}
		build.append("\">\n");
	}

	private static void closeNodeTag(IHgraphNode node, StringBuilder build){
		build.append("</");
		build.append(node.getClass().getName());
		build.append(">\n");
	}
	
	private static void childrenToXML(IHgraphNode node, StringBuilder build, String indent){
		Iterator iter = node.getChildIterator();
		if(iter.hasNext()){
			build.append(indent);
			build.append("<" + CHILDREN_TAG + ">\n");
			while(iter.hasNext()){
				IHgraphNode cNode = (IHgraphNode)iter.next();
				build.append(indent);
				build.append(INDENTATION);
				build.append("<");
				build.append(cNode.getClass().getName());
				build.append(" name=\"");
				build.append(cNode.getNodeName());
				if((cNode.getPath() != null) && (cNode.getPath().toString() != null)){
					build.append("\" path=\"");
					build.append(cNode.getPath().toString());
				}
				build.append("\"/>\n");
			}
			build.append("</" + CHILDREN_TAG + ">\n");
		}
	}
	
	private static void valuesToXML(IHgraphNode node, StringBuilder build, String indent){
		Iterator iter = node.getValueFunc().valueIterator();
		if(iter.hasNext()){
			build.append(indent);
			build.append("<" + VALUES_TAG + ">\n");
			while(iter.hasNext()){
				Value cVal = (Value)iter.next();
				build.append(indent);
				build.append(INDENTATION);
				valueToXML(cVal, build);
			}
			build.append(indent);
			build.append("</" + VALUES_TAG + ">\n");
		}
	}

	private static void valueToXML(Value value, StringBuilder build){
		build.append("<");
		build.append(Value.class.getName());
		build.append(" name=\"");
		build.append(value.getName());
		if((value != null) && (value.getValue() != null)){
			build.append("\" valuetype=\"");
			if(value.getValue() instanceof IHgraphNode){
				build.append(IHgraphNode.class.getName());
			}else{
				build.append(value.getValue().getClass().getName());
			}
			build.append("\"");
			if(value.getValue() instanceof String){
				build.append(" value=\""+(String)value.getValue());
				build.append("\"/>\n");
			}else if(value.getValue() instanceof Integer){
				build.append(" value=\""+(Integer)value.getValue());
				build.append("\"/>\n");				
			}else if(value.getValue() instanceof IHgraphNode){
				build.append(">\n");
				build.append(toXMLRecursively((IHgraphNode)value.getValue())); 
				build.append("<");
				build.append(Value.class.getName());
				build.append("/>\n");
			}else{
				build.append(" value=\""+value.getValue().toString());
				build.append("\"/>\n");				
			}
		}else{
			build.append("\">");
		}
	}

	private static void handlersToXML(IHgraphNode node, StringBuilder build, String indent){
		if(node.getFuncFunc().getHandlers() == null){
			return;
		}
		build.append(indent);
		build.append("<" + HANDLERS_TAG + ">\n");
		for(int i = 0; i<node.getFuncFunc().getHandlers().length; i++){
			handlerToXML(node.getFuncFunc().getHandlers()[i], build, indent + INDENTATION);
		}
		build.append(indent);		
		build.append("</" + HANDLERS_TAG + ">\n");
	}
	
	private static void handlerToXML(Handler handler, StringBuilder build, String indent){
		build.append(indent);
		build.append("<");
		build.append(handler.getClass().toString());
		build.append(" name=\"");
		build.append(handler.getName());
		build.append("\" description=\"");
		build.append(handler.getDescription());
		build.append("\">\n");
		if(handler.getSignatures() != null){
			for(int i = 0; i<handler.getSignatures().length; i++){
				signatureToXML(handler.getSignatures()[i], build, indent + INDENTATION);
			}			
		}
		build.append(indent);
		build.append("<");
		build.append(handler.getClass().toString());
		build.append("/>\n");
	}

	private static void signatureToXML(Signature signature, StringBuilder build, String indent){
		build.append(indent);
		build.append("<");
		build.append(signature.getClass().toString());
		build.append(" name=\"");
		build.append(signature.getName());
		build.append("\"/>\n");
		if(signature.getInput() != null){
			for(int i = 0; i<signature.getInput().length; i++){
				build.append(indent);
				build.append(INDENTATION);
				build.append("<");
				build.append(INPUT_TAG);
				build.append(" type=\"");
				build.append(signature.getInput()[i]);
				build.append("\" />\n");
			}
		}
		if(signature.getReturns() != null){
			build.append(indent);
			build.append(INDENTATION);
			build.append("<");
			build.append(RETURNS_TAG);
			build.append(" type=\"");
			build.append(signature.getReturns());
			build.append("\" />\n");
		}
		build.append(indent);
		build.append("</");
		build.append(signature.getClass().toString());
		build.append(">\n");
	}

	
	private static void invokatorsToXML(IHgraphNode node, StringBuilder build, String indent){
		if(node.getFuncFunc().getInvokators() == null){
			return;
		}
		build.append(indent);
		build.append("<" + INVOKATORS_TAG + ">\n");
		for(int i = 0; i<node.getFuncFunc().getInvokators().length; i++){
			invokatorToXML(node.getFuncFunc().getInvokators()[i], build, indent + INDENTATION);
		}
		build.append(indent);
		build.append("</" + INVOKATORS_TAG + ">\n");		
	}
	
	
	private static void invokatorToXML(Invokator invokator, StringBuilder build, String indent) {
		build.append(indent);
		build.append("<");
		build.append(invokator.getClass().toString());
		build.append(" name=\"");		
		build.append(invokator.getName());
		build.append("\" description=\"");
		build.append(invokator.getDescription());
		build.append("\"/>\n");
		Iterator iter = invokator.getLogIterator();
		if(iter.hasNext()){
			String newIndent = indent + INDENTATION;
			build.append(indent);
			build.append("<" + LOGS_TAG + "/>\n");
			while(iter.hasNext()){
				HgraphEvent event = (HgraphEvent)iter.next();
				eventToXML(event, build, indent);
			}
			build.append("</" + LOGS_TAG + "/>\n");
		}
		build.append(indent);
		build.append("<");
		build.append(invokator.getClass().toString());
		build.append("/>\n");
	}

	public static String eventToXML(HgraphEvent event){
		StringBuilder build = StringBuilder.instantiate();
		eventToXML(event, build, "");
		return build.toString();
	}
	
	public static String valueToXML(Value value){
		StringBuilder build = StringBuilder.instantiate();
		valueToXML(value, build);
		return build.toString();
	}

	private static void eventToXML(HgraphEvent event, StringBuilder build, String indent){
		build.append(indent);
		build.append("<");
		build.append(event.getClass().toString());
		build.append(" frompath=\"");
		build.append(event.getFromPath().toString());
		build.append("\" topath=\"");
		build.append(event.getToPath().toString());
		build.append("\" command=\"");
		build.append(event.getCommand());
		if(event.getReturnCommand() != null){
			build.append("\" returncommand=\"");			
			build.append(event.getReturnCommand());
		}
		build.append("\">\n");
		if(event.getValue() != null){
			build.append(indent);
			build.append(INDENTATION);
			valueToXML(event.getValue(), build);
		}
		build.append(indent);
		build.append("<");
		build.append(event.getClass().toString());
		build.append("/>\n");
	}
	
	private static void listenersToXML(IHgraphNode node, StringBuilder build, String indent){
		if(node.getFuncFunc().getListeners() == null){
			return;
		}
		build.append(indent);		
		build.append("<" + LISTERNERS_TAG + ">\n");
		for(int i = 0; i<node.getFuncFunc().getListeners().length; i++){
			listenerToXML(node.getFuncFunc().getListeners()[i], build, indent + INDENTATION);
		}
		build.append(indent);
		build.append("</" + LISTERNERS_TAG + ">\n");		
	}
	
	private static void listenerToXML(Listener listener, StringBuilder build, String indent){
		build.append(indent);
		build.append("<");
		build.append(listener.getClass().toString());
		build.append(" name=\"");
		build.append(listener.getName());
		build.append("\" description=\"");
		build.append(listener.getDescription());
		build.append("\" returnpath=\"");
		build.append(listener.getReturnPath().toString());
		build.append("\"/>\n");
	}	
}