/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.jtst;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class ExceptionTest extends Test {
  public String name() { return "Exception"; }
  public void run() {
    //ist.palcom.base.System.instance().enableConstantGC();

    // Remove these when static initializers are working!
    //Throwable aioob = new java.lang.ArrayIndexOutOfBoundsException();
    //aioob.nullStackTrace();
    //ist.palcom.base.System.instance().addExceptionMapping("ArrayIndexOutOfBoundsException", aioob);
    //ist.palcom.base.System.instance().addExceptionMapping("StringIndexOutOfBoundsException", new java.lang.StringIndexOutOfBoundsException());
    //ist.palcom.base.System.instance().addExceptionMapping("ArrayStoreException", new java.lang.ArrayStoreException());
    //ist.palcom.base.System.instance().addExceptionMapping("ArithmeticException", new java.lang.ArithmeticException());
    //ist.palcom.base.System.instance().addExceptionMapping("NilDoesNotUnderstand", new java.lang.NullPointerException());
    //ist.palcom.base.System.instance().addExceptionMapping("PrimitiveSelfType", new java.lang.IllegalArgumentException("Receiver of primitive message send had wrong type"));
    //ist.palcom.base.System.instance().addExceptionMapping("PrimitiveParameter1Type", new java.lang.IllegalArgumentException("First parameter of primitive message send had wrong type"));
    //ist.palcom.base.System.instance().addExceptionMapping("PrimitiveParameter1Invalid", new java.lang.IllegalArgumentException("First parameter of primitive message send was invalid"));
    //ist.palcom.base.System.instance().addExceptionMapping("PrimitiveParameter2Type", new java.lang.IllegalArgumentException("Second parameter of primitive message send had wrong type"));
    //ist.palcom.base.System.instance().addExceptionMapping("PrimitiveParameter3Type", new java.lang.IllegalArgumentException("Third parameter of primitive message send had wrong type"));

    run1();
    run2();
  }

  private void run1() {
    startLine();
    print(A);
    try { throw new RuntimeException(); } catch (Throwable t) { print(B); }
    try {
      try { throw new IOException(); } finally { print(C); }
    } catch (IOException ioe) {
      print(D);
    }

    while(true) {
      print(E);
      break;
    }
    print(F);

    boolean yes = true;
    boolean cond = true;
    while(yes) {
      if (cond)
        print(G);
      else
        print(H);
      if (yes) {
        yes = cond;
        cond = false;
        continue;
      }
      print("Oops");
    }
    print(I);

    for (int i = 0; i < 4; i++) {
      if (i == 0) {
        print(J);
      } else {
        print(L);
        break;
      }
      if (i == 1)
        continue;
      print(K);
    }
    print(M);

    for (int i = 0; i < 10; i++) {
      print(N);
      try {
        print(O);
        throw new IOException();
      } catch (Exception e) {
        print(P);
        break;
      }
    }
    
    print(Q);
    try {
      print(R);
      for (int i = 0; i < 10; i++) {
        print(S);
        throw new IOException();
      }
    } catch (Exception e) {
      print(T);
    }

    print(U);
    try {
      print(V);
      try {
        print(W);
        throw new IOException();
      } catch (RuntimeException re) {
        print("!");
      } finally {
        print(X);
      }
      print("*");
    } catch (IOException ioe) {
      print(Y);
    } finally {
      print(Z);
    }

    print(0);
    try {
      print(1);
      try {
        print(2);
        throw new IOException();
      } catch (IllegalArgumentException iae) {
        print("%");
      } catch (RuntimeException re) {
        print("!");
      } finally {
        print(3);
      }
      print("*");
    } catch (IllegalArgumentException iae) {
      print("#");
    } catch (IOException ioe) {
      print(4);
    } finally {
      print(5);
    }

    try {
      thrower();
    } catch (Exception e) {
    //  e.printStackTrace();
    }

    println("6789.");
  }
  
  private void run2() {
    Throwable one = null;
    Throwable two = null;
    startLine();
    print(A);

    try {
      print(B);
      (new int[2])[3] = 0;
    } catch (ArrayIndexOutOfBoundsException e) {
      print(C);
      one = e;
      //e.printStackTrace();
    }

    try {
      print(D);
      (new int[2])[3] = 0;
    } catch (ArrayIndexOutOfBoundsException e) {
      print(E);
      two = e;
    }

    if (one != two) {
      print(F);
    }

    if (one instanceof java.lang.ArrayIndexOutOfBoundsException)
      print(G);

    if (two instanceof java.lang.ArrayIndexOutOfBoundsException)
      print(H);

    try {
      print(I);
      int d = 0;
      d = 4 / d;
    } catch (Throwable t) {
      print(J);
    }

    try {
      print(K);
      int d = 0;
      d = 4 / d;
    } catch (ArithmeticException t) {
      print(L);
    }

    try {
      print(M);
      try {
        print(N);
        (new List[2])[3] = new ArrayList();
      } catch (ArithmeticException e) {
        print (X);
      }
    } catch (ArrayIndexOutOfBoundsException e) {
      print(O);
    }

    try {
      print(P);
      try {
        print(Q);
        Object [] a = new List[2];
        a[1] = new Object();
      } catch (ArrayIndexOutOfBoundsException e) {
        print (X);
      }
    } catch (ArrayStoreException e) {
      print(R);
    }

    try {
      List a[] = new List[1];
      print(S);
      a[0].get(0);
    } catch (NullPointerException npe) {
      print(T);
    }


    println("uvwxyz0123456789.");
  }

  private void thrower() {
    throw new IllegalArgumentException();
  }
}


