/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.mal;

import ist.palcom.palcomthreads.Event;

/**
 * This event is raised whenever an unrecoverable error has occured
 * within the Media Abstraction Layer.
 * @author Henrik Gammelmark, geemark@daimi.au.dk
 */
public class MALErrorEvent extends Event {
	private MediaManager source;
	private Exception error;
	
	public MALErrorEvent(Exception error) {
		if (error == null)
			throw new NullPointerException("Arguments cannot be null");
		this.error = error;
	}
	
	void setSource(MediaManager source) {
		this.source = source;
	}
	
	public Exception getError() {
		return error;
	}
	
	/**
	 * Determine the source of the error.
	 * This allows for a single error handler to accomodate
	 * multiple MediaManagers. For instance, this is convenient when
	 * using the {@link CompoundMediaManager}, which propagate errors.
	 */
	public MediaManager getSource() {
		return source;
	}
	
	public String toString() {
		return error.getMessage() + (source == null ? "" : " (Source: " + source + ")");
	}
}
