/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.mal;

/**
 * @author David Svensson, david@cs.lth.se
 */
public class SimpleMessage extends Message {

    public static final byte TYPE_DATA = 'd';

    private byte type;

    private byte[] information;

    public SimpleMessage(byte[] information) {
        this(information, 0, information.length);
    }

    public SimpleMessage(byte[] information, int offset, int informationLength) {
        this(TYPE_DATA, information, offset, informationLength);
    }

    public SimpleMessage(byte type, byte[] information, int offset,
            int informationLength) {
        this.type = type;

        // TODO Always use offset and length, and never copy to a new array.
        // That is more efficient, and makes more sense. All uses of
        // getInformation must use the new interface after that change (cannot
        // use the whole array returned, but have to take offset and length into
        // account).
        if (offset != 0 || informationLength != information.length) {
            this.information = new byte[informationLength];
            System.arraycopy(information, offset, this.information, 0,
                    informationLength);
        } else {
            this.information = information;
        }
    }

    public int getInformationLength() {
        return information.length;
    }

    public byte[] getInformation() {
        return information;
    }

    public byte getType() {
        return type;
    }

    protected int writeInformation(byte[] buffer, int offset) {
        System.arraycopy(information, 0, buffer, offset, information.length);
        return information.length;
    }

    public Message getCopy() {
        SimpleMessage result = new SimpleMessage(type, information, 0,
                information.length);
        result.setReceiverSelector(getReceiverSelector());
        result.setSenderSelector(getSenderSelector());
        result.setSenderURL(getSenderURL());
        result.setReceiverURL(getReceiverURL());
        result.setSenderDeviceID(getSenderDeviceID());
        result.setReceiverDeviceID(getReceiverDeviceID());
        return result;
    }
}
