/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.mal.bt;

/**
 * This class implements ist.palcom.mal.BluetoothCollector
 * 
 * This class is intended to receive all messages received via bluetooth. It is the
 * CommunicationThread used when a PalcomThread uses the function 
 * BluetoothMediaManager.receive(palcomThread). In Udp this function create a socket
 * listening on a default address, this is not posible in Bluetooth, so here the thread(s)
 * will receive all packages received via Bluetooth.
 * 
 * @author Jacob Frolund Pedersen, frolund@daimi.au.dk
 */

import ist.palcom.base.networking.BluetoothPacket;
import ist.palcom.mal.CommunicationThread;
import ist.palcom.mal.MessageArrayWithURLEvent;
import ist.palcom.palcomthreads.Event;
import ist.palcom.resource.descriptor.URL;

public class BluetoothCollector extends CommunicationThread{

	URL url = new URL("");
	
	Event event = null;
	
	BluetoothConnectionManagerThread managerThread;
	
	public BluetoothCollector(URL url, BluetoothConnectionManagerThread managerThread){
		this.url = url;
		this.managerThread = managerThread;
	}
	
	public void run(){
		while (true) {
            event = waitEvent();
            if (event instanceof MessageArrayWithURLEvent) {
            	managerThread.putEvent(event);
			}
		}
	}
	
	public URL getURL() {
		return url;
	}
	
	public void receivePacket(BluetoothPacket p){
		//System.out.println("Received a package");
	}
	
	public Event getEvent(){
		Event tmp = event;
		event = null;
		return tmp;
	}
}