/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.mal.bt.test;

import ist.palcom.communication.CommunicationManager;
import ist.palcom.mal.AbstractMediaManager;
import ist.palcom.mal.CompoundMediaManager;
import ist.palcom.mal.DeviceCacheObject;
import ist.palcom.mal.MediaManager;
import ist.palcom.mal.Message;
import ist.palcom.mal.MessageEvent;
import ist.palcom.mal.bt.BluetoothMediaManager;
import ist.palcom.mal.udp.UDPMediaManager;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.resource.descriptor.DeviceID;
import ist.palcom.resource.descriptor.Selector;
import ist.palcom.routing.BasicRoutingManager;
import ist.palcom.routing.RoutingManager;

/**
 * @author David Svensson, david@cs.lth.se
 */
public class BluetoothCommunicationTest{

    public static void main(String[] args) {
        BluetoothCommunicationTest test = new BluetoothCommunicationTest();

        test.testDiscovery();
        
 /*       test.testCreateMessage();
        System.out.println();
        test.testSendAndReceiveMessage();
        System.out.println();
        test.testMulticast();
        System.out.println();
        test.testCompoundMediaManager();
        System.out.println();
 */   }

    private void testDiscovery(){
        final PalcomScheduler scheduler = new PalcomScheduler();
        PalcomThread t = new PalcomPriorityThread() {
        	
            public void run() {
            	MediaManager manager = new BluetoothMediaManager(scheduler, null);
            	RoutingManager routingManager = new BasicRoutingManager(manager, scheduler);
            	CommunicationManager comManager = new CommunicationManager(scheduler, 
            			routingManager);
            	Selector receiverSelector = comManager.startReceiving2(this);
                while(true){
                MessageEvent event = (MessageEvent) this.waitEvent();
                System.out.println(event.getMessage().toString());
                }
//                comManager.stop();
//                routingManager.stop();
            }
        };
        scheduler.scheduleThread(t);
        scheduler.run();
    }
    
    private void testCreateMessage() {
        System.out.println("--- testCreateMessage ---");

        byte[] data = "Message data".getBytes();
        System.out.println("Message data created with length " + data.length);
        Message message = new Message(data, 0, data.length);

        byte[] result = new byte[message.getTotalLength()];
        System.out.println("Length of result " + result.length);
        message.write(result, 0);
        for (int i = 0; i < result.length; i++) {
            System.out.print((char) result[i]);
        }
        System.out.println();
    }

    private void testSendAndReceiveMessage() {
        System.out.println("--- testSendAndReceiveMessage ---");

        final PalcomScheduler scheduler = new PalcomScheduler();
        PalcomThread t = new PalcomPriorityThread() {
            public void run() {
                AbstractMediaManager manager = new UDPMediaManager(scheduler, null);
                RoutingManager routingManager = new BasicRoutingManager(manager, scheduler);
                CommunicationManager comManager = new CommunicationManager(scheduler, 
                		routingManager);
                Selector receiverSelector = comManager.startReceiving2(this);
                Selector senderSelector = comManager.startReceiving2(this);
                // Sender also starts to listen, so a selector is allocated
                
                final DeviceID senderDeviceID = new DeviceID("X:sender");
                final DeviceID receiverDeviceID = new DeviceID("X:receiver");
                
                comManager.setLocalDeviceID(senderDeviceID);

                // Manually register device URL, so the manager knows about the
                // receiver. In real use, this information is spread via
                // broadcast messages
                manager.registerDevice(receiverDeviceID.toString(), new DeviceCacheObject(manager
                        .getDefaultUnicastURL(), receiverDeviceID.toString(), "W2", "D2", new Selector("1")));
                
                byte[] sentData = "Some data".getBytes();
                Message message = new Message(sentData, 0,
                        sentData.length);

                System.out.println("Sending message of total length "
                        + message.getTotalLength() + " (selector not set)");

                comManager.sendSingleShot(message, receiverDeviceID, 
                		receiverSelector, senderSelector, false);

                waitForMessageAndPrintIt(this);

                comManager.stop();
                routingManager.stop();
            }
        };
        scheduler.scheduleThread(t);
        scheduler.run();
    }

//    private void testMulticast() {
//        System.out.println("--- testMulticast ---");
//
//        final PalcomScheduler scheduler = new PalcomScheduler();
//        PalcomThread t = new PalcomPriorityThread() {
//            public void run() {
//                MediaManager mediaManager = new UDPMediaManager(scheduler, null);
//                CommunicationManager comManager = new CommunicationManager(scheduler, mediaManager);
//                final URN senderURN = new URN("urn:palcom://sender");
//                testSendAndReceiveMulticastMessage(scheduler, comManager,
//                        this, senderURN,
//                        "Multi-cast message 1");
//
//                mediaManager.stop();
//            }
//        };
//        scheduler.scheduleThread(t);
//        scheduler.run();
//    }

    private void testCompoundMediaManager() {
        System.out.println("--- testCompoundMediaManager ---");

        final PalcomScheduler scheduler = new PalcomScheduler();
        PalcomThread t = new PalcomPriorityThread() {
            public void run() {
                AbstractMediaManager udpManager = new UDPMediaManager(scheduler, null);
                CompoundMediaManager compoundManager = new CompoundMediaManager(
                        scheduler, "datagram", udpManager, null);
                RoutingManager routingManager = new BasicRoutingManager(compoundManager, scheduler);
                CommunicationManager communicationManager = new CommunicationManager(scheduler, 
                		routingManager);
                // Try sending and receiving a message using the compound
                // manager
                Selector receiverSelector = communicationManager.startReceiving2(this);
                Selector senderSelector = communicationManager.startReceiving2(this);
                // Sender also starts to listen, so a selector is allocated
                
                final DeviceID senderDeviceID = new DeviceID("X:sender");
                final DeviceID receiverDeviceID = new DeviceID("X:receiver");
                
                communicationManager.setLocalDeviceID(senderDeviceID);

                // Manually register device URL, so the manager knows about the
                // receiver. In real use, this information is spread via
                // broadcast messages
                udpManager.registerDevice(receiverDeviceID.toString(), new DeviceCacheObject(udpManager
                        .getDefaultUnicastURL(), receiverDeviceID.toString(), "W2", "D2", new Selector("1")));
                
                byte[] sentData = "Message data".getBytes();
                Message message = new Message(sentData, 0,
                        sentData.length);
                communicationManager.sendSingleShot(message, receiverDeviceID, 
                		receiverSelector, senderSelector, false);
                waitForMessageAndPrintIt(this);

                compoundManager.stop();
                communicationManager.stop();
                routingManager.stop();
            }
        };
        scheduler.scheduleThread(t);
        scheduler.run();
    }

    private void testSendAndReceiveMulticastMessage(PalcomScheduler scheduler,
            CommunicationManager manager, PalcomThread receiver, String senderDeviceID,
            String data) {
        byte[] dataBytes = data.getBytes();
        Message message = new Message(dataBytes, 0, dataBytes.length);
        Selector returnedSelector = manager.startReceiving2(receiver);
        manager.sendBroadcast(message);
        System.out.println("Message sent");
        waitForMessageAndPrintIt(receiver);
    }

    public static void waitForMessageAndPrintIt(PalcomThread receiver) {
    	System.out.println("Before waiting");
        MessageEvent event = (MessageEvent) receiver.waitEvent();
        System.out.println("After waiting");
        Message m = event.getMessage();
        System.out.println("Message received");
        System.out.println("  Total length " + m.getTotalLength());
        byte[] receivedData = new byte[m.getTotalLength()];
        m.write(receivedData, 0);
        System.out.print("  Data: ");
        for (int i = 0; i < receivedData.length; i++) {
            System.out.print((char) receivedData[i]);
        }
        System.out.println();
    }
}
