/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.mal.ir;

import ist.palcom.base.networking.IRMessage;
import ist.palcom.resource.descriptor.URL;
import ist.palcom.mal.CommunicationThread;
import ist.palcom.mal.Message;
import ist.palcom.mal.MessageWithURLEvent;
import ist.palcom.palcomthreads.PalcomScheduler;

public class IRReceiverThread extends CommunicationThread {
	private IRMediaManager mediaManager;
	private String urlPrefix;
	
	public IRReceiverThread(PalcomScheduler scheduler, IRMediaManager mediaManager, String urlPrefix) {
		this.mediaManager = mediaManager;
		this.urlPrefix = urlPrefix;
	}
	
	public URL getURL() {
		return new URL(urlPrefix + IRMediaManager.BROADCAST_HOST);		// Thread handling all urls
	}
	
	public void run() {
        while (true) {
        	// Periodically poll the IR communications-board for new messages.
        	// Must be done very often (preferably every millisecond), or else transmitted bytes may be lost.
        	IRMessage irMsg = mediaManager.getIRManager().readMessage();
        	if (irMsg != null) {
        		System.out.println("IRReceiverThread: Message ready!");
        		URL url = new URL(urlPrefix + irMsg.getOrigin() + ":" + irMsg.getPortIn());
        		Message msg = new IRMessageWrapper(irMsg, url);
        		mediaManager.getDistributorThread().putEvent(new MessageWithURLEvent(msg, url));
        	}
        	yield();
        }
	}
}
