/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.mal.udp;

import java.io.IOException;

import ist.palcom.base.networking.MulticastSocket;
import ist.palcom.resource.descriptor.URL;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.palcomthreads.PalcomThread;

/**
 * @author David Svensson, david@cs.lth.se
 */
public class UDPMulticastCommunicationThread extends UDPCommunicationThread {

    private String address;

    private URL url;

    public UDPMulticastCommunicationThread(PalcomScheduler scheduler,
            PalcomThread distributor, String address, int port) throws IOException {
        super(scheduler, distributor, new MulticastSocket(port));

        this.address = address;
        url = new URL(URL_PREFIX + address + ":" + port);
        if (UDPMediaManager.isIPMulticast(address)) {
            ((MulticastSocket) socket).joinGroup(address);
        }
    }

    protected void tearDown() {
    	try {
            if (UDPMediaManager.isIPMulticast(address)) {
                ((MulticastSocket) socket).leaveGroup(address);
            }
            super.tearDown();
    		
    	} catch (IOException e) {
    		// Ignore - we are leaving anyway
    	}
    }

    public URL getURL() {
        return url;
    }
}
