/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.mal.udp.test;

import ist.palcom.collections.LinkedList;
import ist.palcom.communication.CommunicationManager;
import ist.palcom.mal.AbstractMediaManager;
import ist.palcom.mal.CompoundMediaManager;
import ist.palcom.mal.DeviceCacheObject;
import ist.palcom.mal.HeartBeatEvent;
import ist.palcom.mal.MediaManager;
import ist.palcom.mal.Message;
import ist.palcom.mal.MessageEvent;
import ist.palcom.mal.MessageHeaderNode;
import ist.palcom.mal.udp.UDPMediaManager;
import ist.palcom.palcomthreads.Event;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.resource.descriptor.DeviceID;
import ist.palcom.resource.descriptor.HeartBeat;
import ist.palcom.resource.descriptor.Selector;
import ist.palcom.routing.AbstractRoutingManager2;
import ist.palcom.routing.BasicRoutingManager;
import ist.palcom.routing.RoutingManager;

/**
 * @author Thomas Forsstrm, thomasf@cs.lth.se
 */
public class UDPCommunicationTest2 {

	private static final String localDeviceCache = "0";

	public static void main(String[] args) {
		UDPCommunicationTest2 test = new UDPCommunicationTest2();

		test.testCreateAdvancedMessage();
		System.out.println();
		test.testSendAndReceiveHeartBeat();
		System.out.println();
		test.testSendAndReceiveMessage();
		System.out.println();
		 test.testUnicastAfterBroadcast();
		 System.out.println();
		test.testReliableMessages();
		System.out.println();
		test.testReliableWithLostMessages();
		System.out.println();
	}

	private void testCreateAdvancedMessage() {
		System.out.println("--- testCreateAdvancedMessage ---");

		String[] data = new String[]{"newDevice", "1"};
		MessageHeaderNode hNode = new MessageHeaderNode(
				MessageHeaderNode.NODE_FORMAT_HBINFOREPLY, data);
		System.out.println("Total length of message header node - HBInfo part " + hNode.getTotalLength());

		String[] versionData = new String[]{"W2"};
		MessageHeaderNode versionNode = new MessageHeaderNode(
				MessageHeaderNode.NODE_FORMAT_VERSION, versionData);
		System.out.println("Total length of message header node - Version part " + versionNode.getTotalLength());

		byte[] array = (hNode.toString() + versionNode.toString()).getBytes();
		Message message = new Message(hNode, array);
		message.insertHeaderNodeLast(versionNode);

		byte[] result = new byte[message.getTotalLength()];
		message.write(result, 0);

		System.out.println("HBInfo Node: " + hNode.toString());
		System.out.println("version Node: " + versionNode.toString());
		System.out.println("Length of result " + result.length);
		System.out.println(new String(result));
//		for (int i = 0; i < result.length; i++) {
//			System.out.print((char) result[i]);
//		}
		System.out.println();
	}

	private void testSendAndReceiveHeartBeat() {
		System.out.println("--- testSendAndReceiveHeartBeat ---");

		final PalcomScheduler scheduler = new PalcomScheduler();
		PalcomThread t = new PalcomPriorityThread() {
			public void run() {
				MediaManager mediaManager = new UDPMediaManager(scheduler, null);
				AbstractRoutingManager2 routingManager = new BasicRoutingManager(mediaManager, scheduler);

				routingManager.registerForHeartBeats(this);
				routingManager.registerAsListener(this);

				//Send a couple of heart beats to ensure that the device gets 
				//registered before listening for answers.
				routingManager.sendHeartBeat(false);
				routingManager.sendHeartBeat(false);
				routingManager.sendHeartBeat(false);

				MessageHeaderNode hNode = new MessageHeaderNode(
						MessageHeaderNode.NODE_FORMAT_HEARTBEATACK,
						new String[]{localDeviceCache, "G"});

				System.out.println("Sending heartbeat message of total length "
						+ hNode.getTotalLength() + " (selector not set)");

//				This only works since it is a heart beat message.
				routingManager.sendBroadcast(new Message(hNode)); 
				waitForMessageAndPrintIt(this);

				routingManager.sendHeartBeat(false);
				waitForMessageAndPrintIt(this);

				routingManager.stop();
			}
		};
		scheduler.scheduleThread(t);
		scheduler.run();
	}

	private void testCompoundMediaManager() {
		System.out.println("--- testCompoundMediaManager ---");

		final PalcomScheduler scheduler = new PalcomScheduler();
		PalcomThread t = new PalcomPriorityThread() {
			public void run() {
				MediaManager udpManager = new UDPMediaManager(scheduler, null);
				CompoundMediaManager compoundManager = new CompoundMediaManager(
						scheduler, "datagram", udpManager, null);
				RoutingManager routingManager = new BasicRoutingManager(compoundManager, scheduler);
				CommunicationManager communicationManager = new CommunicationManager(scheduler, 
						routingManager);

				// Try sending and receiving a message using the compound
				// manager
				Selector receiverSelector = communicationManager.startReceiving2(this);
//				URL receiverURL = compoundManager.startReceiving(this);
				byte[] sentData = "some data".getBytes();
//				communicationManager.sendSingleShot(new Message(sentData), receiverSelector);
				waitForMessageAndPrintIt(this);

				communicationManager.stop();
				routingManager.stop();
			}
		};
		scheduler.scheduleThread(t);
		scheduler.run();
	}

	private void testSendAndReceiveMessage() {
		System.out.println("--- testSendAndReceiveMessage ---");

		final PalcomScheduler scheduler = new PalcomScheduler();
		PalcomThread t = new PalcomPriorityThread() {
			public void run() {
				AbstractMediaManager mediaManager = new UDPMediaManager(scheduler, null);
				RoutingManager routingManager = new BasicRoutingManager(mediaManager, scheduler);
				CommunicationManager manager = new CommunicationManager(scheduler, 
						routingManager);

				Selector receiverSelector = manager.startReceiving2(this);
				Selector senderSelector = manager.startReceiving2(this);
				// Sender also starts to listen, so a selector is allocated

				final DeviceID senderDeviceID = new DeviceID("X:sender");
				final DeviceID receiverDeviceID = new DeviceID("X:receiver");

				manager.setLocalDeviceID(senderDeviceID);

				// Manually register device URL, so the manager knows about the
				// receiver. In real use, this information is spread via
				// broadcast messages
				mediaManager.registerDevice(receiverDeviceID.toString(), new DeviceCacheObject(mediaManager
						.getDefaultUnicastURL(), receiverDeviceID.toString(), "W2", "D1", new Selector("1")));

				byte[] rawData = ("S;3;" + receiverSelector.toString() + ";" + senderSelector.toString() + "d;12;Unicast data").getBytes();
				String[] data = new String[]{receiverSelector.toString(), senderSelector.toString()};
				MessageHeaderNode connectionNode = new MessageHeaderNode(MessageHeaderNode.NODE_FORMAT_SINGLE_SHOT, data);
				Message message = new Message("Unicast data".getBytes());

				System.out.println("Sending message of total length "
						+ message.getTotalLength()
						+ " (no message header nodes counted)");
				manager.sendSingleShot(message, receiverDeviceID, receiverSelector, senderSelector, false);

				waitForMessageAndPrintIt(this);

				manager.stop();
				routingManager.stop();
			}
		};
		scheduler.scheduleThread(t);
		scheduler.run();
	}

	private void testBroadcast() {
		System.out.println("--- testBroadcast ---");

		final PalcomScheduler scheduler = new PalcomScheduler();
		PalcomThread t = new PalcomPriorityThread() {
			public void run() {
				MediaManager mediaManager = new UDPMediaManager(scheduler, null);
				RoutingManager routingManager = new BasicRoutingManager(mediaManager, scheduler);
				CommunicationManager manager = new CommunicationManager(scheduler, 
						routingManager);
				DeviceID deviceID = new DeviceID("X:receiver");
				manager.setLocalDeviceID(deviceID);

				mediaManager.registerForHeartBeats(this);
				mediaManager.sendHeartBeat(false, new LinkedList());

				Selector senderSelector = manager.startReceiving2(this);
				MessageHeaderNode hNode = new MessageHeaderNode(
						MessageHeaderNode.NODE_FORMAT_HEARTBEAT,
						new String[]{localDeviceCache, "G"});
				Message message = new Message(hNode);
				testSendAndReceiveBroadcastMessage(mediaManager, this,
						deviceID, senderSelector,
						message);

				manager.stop();
				routingManager.stop();
			}
		};
		scheduler.scheduleThread(t);
		scheduler.run();
	}

	private void testUnicastAfterBroadcast() {
		System.out.println("--- testUnicastAfterBroadcast ---");

		final PalcomScheduler scheduler = new PalcomScheduler();
		PalcomThread t = new PalcomPriorityThread() {
			public void run() {
				MediaManager mediaManager = new UDPMediaManager(scheduler, null);
				RoutingManager routingManager = new BasicRoutingManager(mediaManager, scheduler);
				CommunicationManager manager = new CommunicationManager(scheduler, 
						routingManager);

				final DeviceID receiverDeviceID = new DeviceID("X:receiver");

				mediaManager.suggestSendBufferSize(69);
				manager.setLocalDeviceID(receiverDeviceID);

				Selector receiverSelector = manager.startReceiving2(this);
				Selector senderSelector = manager.startReceiving2(this);
				// Sender also starts to listen, so a selector is allocated

				routingManager.registerForHeartBeats(this);
				routingManager.sendHeartBeat(false);

				MessageHeaderNode hNode = new MessageHeaderNode(
						MessageHeaderNode.NODE_FORMAT_HEARTBEAT, 
						new String[]{localDeviceCache,"G"});
				Message message = new Message(hNode, hNode.toByteArray());
				// First, we let the receiver send out a broadcast message
				testSendAndReceiveBroadcastMessage(mediaManager, this,
						receiverDeviceID, receiverSelector, message);

//				waitForMessageAndPrintIt(this);

				//The extra heartbeat is sent to ensure that the device is 
				//registered before sending the single-shot message.
				routingManager.sendHeartBeat(false);

//				waitForMessageAndPrintIt(this);
//				waitForMessageAndPrintIt(this);

				// No manual registration of URNs needed after the broadcast.
				// The information about the receiver device has now reached the
				// sender.
				String[] data = new String[]{receiverSelector.toString(), senderSelector.toString()};
				MessageHeaderNode connectionNode = new MessageHeaderNode(MessageHeaderNode.NODE_FORMAT_SINGLE_SHOT, 
						data);
				message = new Message("Unicast.data1".getBytes());
				message.insertHeaderNodeLast(connectionNode);

				while(!mediaManager.isDeviceRegistered(receiverDeviceID.toString())) {
//					System.out.println("Yielding");
					this.yield();
				}

				manager.send(message, receiverDeviceID, false);

				Message receivedMessage = waitForMessageAndReturnIt(this);
				while(receivedMessage == null) {
					receivedMessage = waitForMessageAndReturnIt(this);
				}
//				waitForMessageAndPrintIt(this);

				Message nextMessage = new Message(Message.TYPE_MULTIPART);
				nextMessage.addPart(new Message("Unicast".getBytes()));
				nextMessage.addPart(new Message(".data2".getBytes()));

				connectionNode = new MessageHeaderNode(MessageHeaderNode.NODE_FORMAT_SINGLE_SHOT, 
						data);
				nextMessage.insertHeaderNodeLast(connectionNode);

				manager.send(nextMessage, receiverDeviceID, false);

				receivedMessage = waitForMessageAndReturnIt(this);
				while(receivedMessage == null) {
					receivedMessage = waitForMessageAndReturnIt(this);
				}
//				waitForMessageAndPrintIt(this);

				manager.stop();
				routingManager.stop();
			}
		};
		scheduler.scheduleThread(t);
		scheduler.run();
	}

	private void testReliableMessages() {
		System.out.println("--- testReliableMessages ---");

		final PalcomScheduler scheduler = new PalcomScheduler();
		PalcomThread t = new PalcomPriorityThread() {
			public void run() {
				AbstractMediaManager mediaManager = new UDPMediaManager(scheduler, null);
				AbstractRoutingManager2 routingManager = new BasicRoutingManager(mediaManager, scheduler);
				CommunicationManager manager = new CommunicationManager(scheduler, 
						routingManager);
				
				final DeviceID receiverDeviceID = new DeviceID("X:receiver");
				manager.setLocalDeviceID(receiverDeviceID);
				
				Selector receiverSelector = manager.startReceiving2(this);
				Selector senderSelector = manager.startReceiving2(this);
//				 Sender also starts to listen, so a selector is allocated
				

//				Manually register device URL, so the manager knows about the
				// receiver. In real use, this information is spread via
				// broadcast messages
				mediaManager.registerDevice(receiverDeviceID.toString(), new DeviceCacheObject(mediaManager
						.getDefaultUnicastURL(), receiverDeviceID.toString(), "W2", "D1", new Selector("1")));

				routingManager.registerForHeartBeats(this);

				//Message that will be acked
				String[] data = new String[]{receiverSelector.toString(), senderSelector.toString()};
				MessageHeaderNode connectionNode = new MessageHeaderNode(MessageHeaderNode.NODE_FORMAT_SINGLE_SHOT, 
						data);
				Message message = new Message("Reliable message".getBytes());
				message.insertHeaderNodeLast(connectionNode);
				manager.send(message, receiverDeviceID, true);

				waitForMessageAndPrintIt(this);
//				waitForMessageAndPrintIt(this);

				manager.stop();
				routingManager.stop();
			}

		};
		scheduler.scheduleThread(t);
		scheduler.run();
	}
	
	private void testReliableWithLostMessages() {
		System.out.println("--- testReliableWithLostMessages ---");

		final PalcomScheduler scheduler = new PalcomScheduler();
		PalcomThread t = new PalcomPriorityThread() {
			public void run() {
				AbstractMediaManager mediaManager = new UDPMediaManager(scheduler, null);
				AbstractRoutingManager2 routingManager = new BasicRoutingManager(mediaManager, scheduler);
				CommunicationManager manager = new CommunicationManager(scheduler, 
						routingManager);
				
				final DeviceID receiverDeviceID = new DeviceID("X:receiver");
				manager.setLocalDeviceID(receiverDeviceID);
				
				Selector receiverSelector = manager.startReceiving2(this);
				Selector senderSelector = manager.startReceiving2(this);
//				 Sender also starts to listen, so a selector is allocated
				

//				Manually register device URL, so the manager knows about the
				// receiver. In real use, this information is spread via
				// broadcast messages
				mediaManager.registerDevice(receiverDeviceID.toString(), new DeviceCacheObject(mediaManager
						.getDefaultUnicastURL(), receiverDeviceID.toString(), "W2", "D1", new Selector("1")));

				routingManager.registerForHeartBeats(this);
				
				//This makes the message "disappear" for the receiver
				routingManager.registerAsListener(this);
				
				Message message = new Message("Reliable message that will be dropped".getBytes());
				manager.sendSingleShot(message, receiverDeviceID, receiverSelector, senderSelector, true);

				System.out.print("Lost ");
				waitForMessageAndPrintIt(this);
				
				//Start listening for messages again
				manager.setDistributorThreadAsListener();

				//Message to trigger resending
				message = new Message("Another reliable message".getBytes());
				manager.sendSingleShot(message, receiverDeviceID, receiverSelector, senderSelector, true);

				waitForMessageAndPrintIt(this);
				waitForMessageAndPrintIt(this);

				manager.stop();
				routingManager.stop();
			}

		};
		scheduler.scheduleThread(t);
		scheduler.run();
	}

	private void testSendAndReceiveBroadcastMessage(MediaManager manager, 
			PalcomThread receiver, DeviceID senderDeviceID,
			Selector senderSelector, Message message) {
//		byte[] dataBytes = new byte[message.getTotalLength()];
//		message.write(dataBytes, 0);
		manager.sendBroadcast(message);
//		manager.sendBroadcast(message, senderURN, senderSelector);
		System.out.println("Broadcast message sent");
//		waitForMessageAndPrintIt(receiver);
	}

	/** Special method for reliable messages. Prints and returns the message. */
	public static Message waitForMessageAndReturnIt(PalcomThread receiver) {
		Event event = (Event) receiver.waitEvent();
		if (event instanceof MessageEvent) {
			MessageEvent messageEvent = (MessageEvent) event;
			Message m = messageEvent.getMessage();
			System.out.println("Message received");
			System.out.println("  Total length " + m.getTotalLength());
			byte[] receivedData = new byte[m.getTotalLength()];
			m.write(receivedData, 0);
			System.out.print("  Data: ");
			System.out.println(new String(receivedData));
			return m;
		}
		return null;
	}

	public static void waitForMessageAndPrintIt(PalcomThread receiver) {
		Event event = (Event) receiver.waitEvent();
		if (event instanceof MessageEvent) {
			MessageEvent messageEvent = (MessageEvent) event;
			Message m = messageEvent.getMessage();
			System.out.println("Message received");
			System.out.println("  Total length " + m.getTotalLength());
			byte[] receivedData = new byte[m.getTotalLength()];
			m.write(receivedData, 0);
			System.out.print("  Data: ");
			System.out.println(new String(receivedData));
		} else if (event instanceof HeartBeatEvent) {
			HeartBeat heartBeat = ((HeartBeatEvent) event).getHeartBeat();
			System.out.println("HeartBeat received");
			System.out.println("  Total length " + heartBeat.toString().length());
			System.out.print("  Data: ");
			System.out.println(heartBeat.toString());
		}
	}
}
