/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.palcomthreads;

import ist.palcom.collections.LinkedList;

/**
 * @author David Svensson, david@cs.lth.se
 */
public class Mailbox {
    private PalcomScheduler scheduler;

    private LinkedList events;

    private SortedPalcomThreadQueue waitingQueue;

    public Mailbox() {
        this.scheduler = null;
        events = new LinkedList();
        waitingQueue = new SortedPalcomThreadQueue();
    }

    /** @return The next event in this mailbox, or block until one arrives. */
    public Event waitEvent() {
        while (events.isEmpty()) {
            PalcomThread current = scheduler.getCurrentThread();
            waitingQueue.sortInto(new PalcomThreadNotice(current));
            current.state = PalcomThread.STATE_WAITING;
            current.coroutine.suspend();
        }
        return (Event) events.removeFirst();
    }

    /** @return true if there are pending event(s), else return false. */
    public boolean askEvent() {
        return !events.isEmpty();
    }

    /** Deposit an event for this Mailbox. */
    public void putEvent(Event event) {
        events.insertLast(event);
        if (!waitingQueue.isEmpty()) {
            waitingQueue.removeFirst().getThread().schedule();
        }
    }

    /** @return The PalcomScheduler of this Mailbox (if any). */
    public PalcomScheduler getScheduler() {
        return scheduler;
    }

    void setScheduler(PalcomScheduler scheduler) {
        this.scheduler = scheduler;
    }
}