/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.pubsub.test;

import ist.palcom.collections.LinkedList;
import ist.palcom.communication.CommunicationManager;
import ist.palcom.mal.MediaManager;
import ist.palcom.mal.Message;
import ist.palcom.mal.MessageHeaderNode;
import ist.palcom.mal.udp.UDPMediaManager;
import ist.palcom.mal.udp.test.UDPCommunicationTest2;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.pubsub.PubSubManager;
import ist.palcom.resource.descriptor.DeviceID;
import ist.palcom.resource.descriptor.Selector;
import ist.palcom.routing.RoutingManager;
import ist.palcom.routing.util.SimpleRoutingManager;

import java.util.ArrayList;

/**
 * @author David Svensson, david@cs.lth.se
 */
public class PubSubTest {

    public static void main(String[] args) {
        PubSubTest test = new PubSubTest();

        test.testPubSubWithDeviceID();
        System.out.println();
        test.testConnectionBroadcast();
        System.out.println();
    }

    private void testPubSubWithDeviceID() {
        System.out.println("--- testPubSubWithDeviceID ---");

        final PalcomScheduler scheduler = new PalcomScheduler();
        PalcomThread t = new PalcomPriorityThread() {
            public void run() {
            	MediaManager mediaManager = new UDPMediaManager(scheduler, null);
            	mediaManager.sendHeartBeat(false, new LinkedList());
            	ArrayList mediaManagers = new ArrayList();
                mediaManagers.add(mediaManager);

                //Generate hopefully unique mark ID for router.
        		String mark = "abc";
        		
        		RoutingManager routingManager = new SimpleRoutingManager(mediaManagers, scheduler, mark);
        		
                CommunicationManager comManager = new CommunicationManager(scheduler, 
            			routingManager);
                PubSubManager pubSubManager = new PubSubManager(scheduler,
                       comManager);

                final String topic = "My topic";
                pubSubManager.subscribe(topic, this);

                byte[] data = new byte[] { 'M', 'y', ' ', 'd', 'a', 't', 'a' };
                Message message = new Message(data);
                pubSubManager.publish(topic, message);
                System.out.println("Published message on topic " + topic);

                UDPCommunicationTest2.waitForMessageAndPrintIt(this);

                pubSubManager.stop();
                comManager.stop();
                routingManager.stop();
            }
        };
        scheduler.scheduleThread(t);
        scheduler.run();
    }
    
    private void testConnectionBroadcast() {
        System.out.println("--- testConnectionBroadcast ---");

        final PalcomScheduler scheduler = new PalcomScheduler();
        PalcomThread t = new PalcomPriorityThread() {
            public void run() {
            	MediaManager mediaManager = new UDPMediaManager(scheduler, null);
            	mediaManager.sendHeartBeat(false, new LinkedList());
            	ArrayList mediaManagers = new ArrayList();
                mediaManagers.add(mediaManager);

                //Generate hopefully unique mark ID for router.
        		String mark = "abc";
        		
        		RoutingManager routingManager = new SimpleRoutingManager(mediaManagers, scheduler, mark);
        		
                CommunicationManager comManager = new CommunicationManager(scheduler, 
            			routingManager);
               
                final String selS = "2";

                
                byte[] data = new byte[] { 'M', 'o', 'r', 'e', ' ', 'd', 'a', 't', 'a' };
                Message message = new Message(data);
                
                MessageHeaderNode hNode = new MessageHeaderNode(
        				MessageHeaderNode.NODE_FORMAT_CONNECTION, 
        				MessageHeaderNode.FUNCTION_BROADCASTED,
        				new String[]{selS});
        		message.insertHeaderNodeFirst(hNode);
        		
        		comManager.startReceivingBroadcast(new DeviceID(mediaManager.getLocalDeviceID()), new Selector(selS), this);
                
                System.out.println("Send broadcast message from " + mediaManager.getLocalDeviceID() + " selector: " + selS);
                comManager.sendBroadcast(message);
                
                UDPCommunicationTest2.waitForMessageAndPrintIt(this);

                comManager.stop();
                routingManager.stop();
            }
        };
        scheduler.scheduleThread(t);
        scheduler.run();
    }
}
