/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;




public class PRDDevice extends PRDSecondOrder implements Cloneable {
    public void flushCache() {
        super.flushCache();
    }
    public Object clone() throws CloneNotSupportedException {
        PRDDevice node = (PRDDevice)super.clone();
    return node;
    }
    public ASTNode copy() {
      try {
          PRDDevice node = (PRDDevice)clone();
          if(children != null) node.children = (ASTNode[])children.clone();
          return node;
      } catch (CloneNotSupportedException e) {
      }
      System.err.println("Error: Could not clone node of type " + getClass().getName() + "!");
      return null;
    }
    public ASTNode fullCopy() {
        PRDDevice res = (PRDDevice)copy();
        for(int i = 0; i < getNumChild(); i++) {
          ASTNode node = getChildNoTransform(i);
          if(node != null) node = node.fullCopy();
          res.setChild(node, i);
        }
        return res;
    }
    // Declared in Info.jadd at line 41


    public PRDDevice(DeviceID deviceID, String name) {
    	this();
        setName(name);
        setDeviceID(deviceID);
    }

    // Declared in Comparison.jadd at line 193

    public boolean isAboutTheSameEntityAs(PRDData data) {
        if (data == null) {
            return false;
        } else if (!(data instanceof PRDDevice)) {
            return false;
        } else {
            return PRDData.safeEqual(((PRDDevice)data).getDeviceID(), getDeviceID());
        }
    }

    // Declared in Comparison.jadd at line 319


    public boolean equals(Object o) {
        if (o instanceof PRDDevice) {
            PRDDevice s = (PRDDevice) o;
            if (!PRDData.safeEqual(s.getDeviceID(), getDeviceID())) {
                return false;
            }
            if (!PRDData.safeEqual(s.getName(), getName())) {
                return false;
            }
            if (!PRDData.safeEqual(s.getDiscoverySelector(),
                getDiscoverySelector())) {
                return false;
            }
            if (!PRDData.safeEqual(s.getRemoteConnectSelector(),
                getRemoteConnectSelector())) {
                return false;
            }
            if (!PRDData.safeEqual(s.getDeviceVersion(),
                getDeviceVersion())) {
                return false;
            }
            if (s.getDeviceStatus() != getDeviceStatus()) {
                return false;
            }
            return true;
        } else {
            return false;
        }
    }

    // Declared in Comparison.jadd at line 567

	
    public int hashCode() {
        return getDeviceID().getString().hashCode();
    }

    // Declared in PRD.jadd at line 54

    
    public PRDDevice(DeviceID deviceID) {
    	this();
    	setDeviceID(deviceID);
    }

    // Declared in PRD.jadd at line 59

    
    public PRDDevice(String name, DeviceID deviceID, String deviceVersion, byte deviceStatus, 
			Selector discoverySelector, Selector remoteConnectSelector) {
    	this();
    	setName(name);
    	setDeviceID(deviceID);
    	setDeviceVersion(deviceVersion);
    	setDeviceStatus(deviceStatus);
    	setDiscoverySelector(discoverySelector);
    	setRemoteConnectSelector(remoteConnectSelector);
    }

    // Declared in XML.jadd at line 89

    
    public String getTagName() {
        return "D";
    }

    // Declared in XML.jadd at line 234
    
    
    public void initializeFromElement(XmlPullParser parser) {
    	super.initializeFromElement(parser);
        String deviceVersion = parser.getAttributeValue(null, "v");
        if(deviceVersion != null) {
        	setDeviceVersion(deviceVersion);
        } else {
        	setDeviceVersion(null);
        }
        String deviceStatus = parser.getAttributeValue(null, "ds");
        if(deviceStatus != null) {
        	setDeviceStatus(deviceStatus.getBytes()[0]);
        } else {
        	setDeviceStatus((byte)-1);
        }
        String discoverySelectorString = parser.getAttributeValue(null, "dis");
        if (discoverySelectorString != null) {
            setDiscoverySelector(new Selector(discoverySelectorString));
        } else {
            setDiscoverySelector(null);
        }
        String remoteConnectSelectorString = parser.getAttributeValue(null, "rcs");
        if (remoteConnectSelectorString != null) {
            setRemoteConnectSelector(new Selector(remoteConnectSelectorString));
        } else {
            setRemoteConnectSelector(null);
        }
    }

    // Declared in XML.jadd at line 521

    
    public void addChildFromXMLElement(XMLRepresentable child) {
        if (child instanceof DeviceID) {
            setDeviceID((DeviceID) child);
        }
    }

    // Declared in XML.jadd at line 548

    
    public void writeXMLElement(KXmlSerializer serializer) throws IOException {
        serializer.startTag(null, getTagName());
        super.writeAttributes(serializer);
        if(getDeviceVersion() != null) {
        	serializer.attribute(null, "v", getDeviceVersion());
        }
        if(getDeviceStatus() > -1) {
        	serializer.attribute(null, "ds", new String(new byte[] {getDeviceStatus()}));
        }
        if (getDiscoverySelector() != null) {
            serializer.attribute(null, "dis",
                getDiscoverySelector().toString());
        }
        if (getRemoteConnectSelector() != null) {
            serializer.attribute(null, "rcs",
                getRemoteConnectSelector().toString());
        }
        serializer.endTag(null, getTagName());
    }

    // Declared in PalcomResourceDescriptor.ast at line 3
    // Declared in PalcomResourceDescriptor.ast line 52

    public PRDDevice() {
        super();

        setChild(new Opt(), 0);
        setChild(null, 1);
        setChild(null, 2);

    }

    // Declared in PalcomResourceDescriptor.ast at line 13


    // Declared in PalcomResourceDescriptor.ast line 52
    public PRDDevice(String p0, Opt p1, String p2, byte p3, Selector p4, Selector p5) {
        setName(p0);
        setChild(p1, 0);
        setDeviceVersion(p2);
        setDeviceStatus(p3);
        setChild(p4, 1);
        setChild(p5, 2);
    }

    // Declared in PalcomResourceDescriptor.ast at line 22


  protected int numChildren() {
    return 3;
  }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 40
    private String tokenString_Name;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void refined_PalcomResourceDescriptor_setName(String value) {
        tokenString_Name = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getName() {
        return tokenString_Name;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 52
    public void setDeviceIDOpt(Opt opt) {
        setChild(opt, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 6


    public boolean hasDeviceID() {
        return getDeviceIDOpt().getNumChild() != 0;
    }

    // Declared in PalcomResourceDescriptor.ast at line 10


    public DeviceID getDeviceID() {
        return (DeviceID)getDeviceIDOpt().getChild(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 14


    public void setDeviceID(DeviceID node) {
        getDeviceIDOpt().setChild(node, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 17

    public Opt getDeviceIDOpt() {
        return (Opt)getChild(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 21


    public Opt getDeviceIDOptNoTransform() {
        return (Opt)getChildNoTransform(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 52
    private String tokenString_DeviceVersion;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setDeviceVersion(String value) {
        tokenString_DeviceVersion = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getDeviceVersion() {
        return tokenString_DeviceVersion;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 52
    private byte tokenbyte_DeviceStatus;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setDeviceStatus(byte value) {
        tokenbyte_DeviceStatus = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public byte getDeviceStatus() {
        return tokenbyte_DeviceStatus;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 52
    public void refined_PalcomResourceDescriptor_setDiscoverySelector(Selector node) {
        setChild(node, 1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public Selector getDiscoverySelector() {
        return (Selector)getChild(1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public Selector getDiscoverySelectorNoTransform() {
        return (Selector)getChildNoTransform(1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 52
    public void refined_PalcomResourceDescriptor_setRemoteConnectSelector(Selector node) {
        setChild(node, 2);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public Selector getRemoteConnectSelector() {
        return (Selector)getChild(2);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public Selector getRemoteConnectSelectorNoTransform() {
        return (Selector)getChildNoTransform(2);
    }

    // Declared in Hgraph.jadd at line 208

    
	  public void setName(String name){
		refined_PalcomResourceDescriptor_setName(name);
		if(name != null){
			int i = _node.setNodeName(getTagName() + " - " + name, null);
			if(i != IHgraphNode.STATUS_OK){
				System.out.println("Could not set node name: " + name + " , errcode: " + i);
			}else{
				_node.getValueFunc().setValue("name", name, null);
			}		
		}
	}

    // Declared in Hgraph.jadd at line 226


	/*refine PalcomResourceDescriptor public void PRDDevice.setURN(URN urn){
		PalcomResourceDescriptor.PRDDevice.setURN(urn);
		if(urn != null){
			_node.getValueFunc().setValue("urn", urn, null);		
		}
	}*/
	  public void setDiscoverySelector(Selector selector){
		refined_PalcomResourceDescriptor_setDiscoverySelector(selector);
		if(selector != null){
			_node.getValueFunc().setValue("discoverySelector", selector, null);		
		}
	}

    // Declared in Hgraph.jadd at line 232

	  public void setRemoteConnectSelector(Selector selector){
		refined_PalcomResourceDescriptor_setRemoteConnectSelector(selector);
		if(selector != null){
			_node.getValueFunc().setValue("remoteConnectSelector", selector, null);		
		}
	}

}
