/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;


public abstract class PRDSecondOrderVersion extends AbstractXMLRepresentable implements Cloneable {
    public void flushCache() {
        super.flushCache();
    }
    public Object clone() throws CloneNotSupportedException {
        PRDSecondOrderVersion node = (PRDSecondOrderVersion)super.clone();
    return node;
    }
    // Declared in Addressing.jadd at line 59

    
    public String toString() {
    	return (getCreatingVersionPart() + " " + getUpdatingVersionPart() + " " + 
    		getPreviousVersionPart() + " " + getMergedFromVersionPart());
    }

    // Declared in Comparison.jadd at line 528

    
    public boolean equals(Object o) {
		if(o instanceof PRDSecondOrderVersion) {
			PRDSecondOrderVersion sid = (PRDSecondOrderVersion) o;
			if (!PRDData.safeEqual(sid.getCreatingVersionPart(), getCreatingVersionPart())) {
                return false;
            }
            if (!PRDData.safeEqual(sid.getUpdatingVersionPart(), getUpdatingVersionPart())) {
                return false;
            }
            if (!PRDData.safeEqual(sid.getPreviousVersionPart(), getPreviousVersionPart())) {
                return false;
            }
            if (!PRDData.safeEqual(sid.getMergedFromVersionPart(), getMergedFromVersionPart())) {
                return false;
            }
            return true;
		} else {
			return false;
		}
	}

    // Declared in XML.jadd at line 344

    
    public void initializeFromElement(XmlPullParser parser) {
		setCreatingVersionPart(new VersionPart(new DeviceID(parser.getAttributeValue(null, "cdid")),
			parser.getAttributeValue(null, "cn")));
		setUpdatingVersionPart(new VersionPart(new DeviceID(parser.getAttributeValue(null, "udid")),
			parser.getAttributeValue(null, "un")));
		String prevDeviceID = parser.getAttributeValue(null, "pdid");
		String prevNbr = parser.getAttributeValue(null, "pn");
		if(prevDeviceID != null && prevNbr != null) {
			setPreviousVersionPart(new VersionPart(new DeviceID(prevDeviceID), prevNbr));
		}
		String mergedDeviceID = parser.getAttributeValue(null, "mdid");
		String mergedNbr = parser.getAttributeValue(null, "mn");
		if(mergedDeviceID != null && mergedNbr != null) {
			setMergedFromVersionPart(new VersionPart(new DeviceID(mergedDeviceID), mergedNbr));
		}
	}

    // Declared in XML.jadd at line 676

	
	public void writeAttributes(KXmlSerializer serializer) throws IOException {
		serializer.attribute(null, "cdid", getCreatingVersionPart().getDeviceID().toString());
		serializer.attribute(null, "cn", getCreatingVersionPart().getServicePartID());
		serializer.attribute(null, "udid", getUpdatingVersionPart().getDeviceID().toString());
		serializer.attribute(null, "un", getUpdatingVersionPart().getServicePartID());
		if(getPreviousVersionPart() != null && getPreviousVersionPart().getDeviceID() != null &&
			getPreviousVersionPart().getServicePartID() != null) {
			serializer.attribute(null, "pdid", getPreviousVersionPart().getDeviceID().toString());
			serializer.attribute(null, "pn", getPreviousVersionPart().getServicePartID());
		}
		if(getMergedFromVersionPart() != null && getMergedFromVersionPart().getDeviceID() != null
			&& getMergedFromVersionPart().getServicePartID() != null) {
			serializer.attribute(null, "mdid", getMergedFromVersionPart().getDeviceID().toString());
			serializer.attribute(null, "mn", getMergedFromVersionPart().getServicePartID());
		}
	}

    // Declared in PalcomResourceDescriptor.ast at line 3
    // Declared in PalcomResourceDescriptor.ast line 92

    public PRDSecondOrderVersion() {
        super();

        setChild(null, 0);
        setChild(null, 1);
        setChild(null, 2);
        setChild(null, 3);

    }

    // Declared in PalcomResourceDescriptor.ast at line 14


    // Declared in PalcomResourceDescriptor.ast line 92
    public PRDSecondOrderVersion(VersionPart p0, VersionPart p1, VersionPart p2, VersionPart p3) {
        setChild(p0, 0);
        setChild(p1, 1);
        setChild(p2, 2);
        setChild(p3, 3);
    }

    // Declared in PalcomResourceDescriptor.ast at line 21


  protected int numChildren() {
    return 4;
  }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 92
    public void setCreatingVersionPart(VersionPart node) {
        setChild(node, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public VersionPart getCreatingVersionPart() {
        return (VersionPart)getChild(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public VersionPart getCreatingVersionPartNoTransform() {
        return (VersionPart)getChildNoTransform(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 92
    public void setUpdatingVersionPart(VersionPart node) {
        setChild(node, 1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public VersionPart getUpdatingVersionPart() {
        return (VersionPart)getChild(1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public VersionPart getUpdatingVersionPartNoTransform() {
        return (VersionPart)getChildNoTransform(1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 92
    public void setPreviousVersionPart(VersionPart node) {
        setChild(node, 2);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public VersionPart getPreviousVersionPart() {
        return (VersionPart)getChild(2);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public VersionPart getPreviousVersionPartNoTransform() {
        return (VersionPart)getChildNoTransform(2);
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 92
    public void setMergedFromVersionPart(VersionPart node) {
        setChild(node, 3);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public VersionPart getMergedFromVersionPart() {
        return (VersionPart)getChild(3);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public VersionPart getMergedFromVersionPartNoTransform() {
        return (VersionPart)getChildNoTransform(3);
    }

}
