/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;



public class PRDService extends PRDServiceListItem implements Cloneable {
    public void flushCache() {
        super.flushCache();
    }
    public Object clone() throws CloneNotSupportedException {
        PRDService node = (PRDService)super.clone();
    return node;
    }
    public ASTNode copy() {
      try {
          PRDService node = (PRDService)clone();
          if(children != null) node.children = (ASTNode[])children.clone();
          return node;
      } catch (CloneNotSupportedException e) {
      }
      System.err.println("Error: Could not clone node of type " + getClass().getName() + "!");
      return null;
    }
    public ASTNode fullCopy() {
        PRDService res = (PRDService)copy();
        for(int i = 0; i < getNumChild(); i++) {
          ASTNode node = getChildNoTransform(i);
          if(node != null) node = node.fullCopy();
          res.setChild(node, i);
        }
        return res;
    }
    // Declared in Info.jadd at line 37

    public static final String ROLE_PROVIDER = "provider";

    // Declared in Info.jadd at line 39


    public static final String ROLE_CUSTOMER = "customer";

    // Declared in Info.jadd at line 79

    
    public boolean isProvider() {
        return getHasDesc() && !getRemoteConnect();
    }

    // Declared in Info.jadd at line 83


    public boolean isCustomer() {
        return !(getHasDesc() && !getRemoteConnect());
    }

    // Declared in Comparison.jadd at line 214


    public boolean isAboutTheSameEntityAs(PRDData data) {
        if (data == null) {
            return false;
        } else if (data instanceof PRDService) {
            PRDService s = (PRDService) data;
            return PRDData.safeEqual(s.getLocalSID(), getLocalSID());
        } else {
            return false;
        }
    }

    // Declared in Comparison.jadd at line 373

    
    public boolean equals(Object o) {
        if (o instanceof PRDService) {
            PRDService s = (PRDService) o;
            if (!PRDData.safeEqual(s.getName(), getName())) {
                return false;
            }
            if (!PRDData.safeEqual(s.getLocalSID(), getLocalSID())) {
                return false;
            }
            if (s.getHasDesc() != getHasDesc()) {
                return false;
            }
            if (s.getRemoteConnect() != getRemoteConnect()) {
                return false;
            }
             if (!PRDData.safeEqual(s.getProtocol(), getProtocol())) {
                return false;
            }
            if (s.getReliable() != getReliable()) {
                return false;
            }
            if (!PRDData.safeEqual(s.getVersionName(), getVersionName())) {
                return false;
            }
            if (!PRDData.safeEqual(s.getServiceHelpText(), getServiceHelpText())) {
                return false;
            }
            return true;
        } else {
            return false;
        }
    }

    // Declared in Comparison.jadd at line 575


    public int hashCode() {
        return getLocalSID().toString().hashCode();
    }

    // Declared in Info.jadd at line 38


	public static final byte UNICAST = (byte)'1';

    // Declared in Info.jadd at line 40

	
	public static final byte RADIOCAST = (byte)'N';

    // Declared in Info.jadd at line 42

	
	public static final byte GROUPCAST = (byte)'G';

    // Declared in Info.jadd at line 44

	
	public static final byte BROADCAST = (byte)'B';

    // Declared in XML.jadd at line 97


    public String getTagName() {
        return "S";
    }

    // Declared in XML.jadd at line 262

        
    public void initializeFromElement(XmlPullParser parser) {
    	super.initializeFromElement(parser);
        String localSID = parser.getAttributeValue(null, "ls");
        if (localSID != null) {
            setLocalSID(new LocalSID(null, localSID));
        }
        String distribution = parser.getAttributeValue(null, "d");
        if(distribution != null) {
        	 setDistribution(distribution.getBytes()[0]);
        }
        setHasDesc(parser.getAttributeValue(null,
                "hd").equals("1"));
        setRemoteConnect(parser.getAttributeValue(null, "rc").equals("1"));
        setProtocol(parser.getAttributeValue(null, "p"));
        setReliable(parser.getAttributeValue(null, "r").equals("1"));
        setVersionName(parser.getAttributeValue(null, "vn"));
        setServiceHelpText(parser.getAttributeValue(null, "h"));
    }

    // Declared in XML.jadd at line 582


    public void writeXMLElement(KXmlSerializer serializer) throws IOException {
        serializer.startTag(null, getTagName());
		super.writeAttributes(serializer);
        if (getLocalSID() != null && getLocalSID().getServicePartID() != null) {
            serializer.attribute(null, "ls", getLocalSID().getServicePartID());
        }
        serializer.attribute(null, "d", new String(new byte[] {getDistribution()}));
        if(getHasDesc()) {
    		serializer.attribute(null, "hd", "1");
    	} else {
    		serializer.attribute(null, "hd", "0");
    	}
    	if(getRemoteConnect()) {
    		serializer.attribute(null, "rc", "1");
    	} else {
    		serializer.attribute(null, "rc", "0");
    	}
    	if (getProtocol() != null) {
            serializer.attribute(null, "p", getProtocol());
        }
    	if(getReliable()) {
    		serializer.attribute(null, "r", "1");
    	} else {
    		serializer.attribute(null, "r", "0");
    	}
    	if(getVersionName() != null) {
    		 serializer.attribute(null, "vn", getVersionName());
    	}
    	if(getServiceHelpText() != null) {
    		 serializer.attribute(null, "h", getServiceHelpText());
    	}
        serializer.endTag(null, getTagName());
    }

    // Declared in PalcomResourceDescriptor.ast at line 3
    // Declared in PalcomResourceDescriptor.ast line 55

    public PRDService() {
        super();

        setChild(null, 0);

    }

    // Declared in PalcomResourceDescriptor.ast at line 11


    // Declared in PalcomResourceDescriptor.ast line 55
    public PRDService(String p0, LocalSID p1, byte p2, boolean p3, boolean p4, String p5, boolean p6, String p7, String p8) {
        setName(p0);
        setChild(p1, 0);
        setDistribution(p2);
        setHasDesc(p3);
        setRemoteConnect(p4);
        setProtocol(p5);
        setReliable(p6);
        setVersionName(p7);
        setServiceHelpText(p8);
    }

    // Declared in PalcomResourceDescriptor.ast at line 23


  protected int numChildren() {
    return 1;
  }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 40
    private String tokenString_Name;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void refined_PalcomResourceDescriptor_setName(String value) {
        tokenString_Name = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getName() {
        return tokenString_Name;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 55
    public void refined_PalcomResourceDescriptor_setLocalSID(LocalSID node) {
        setChild(node, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public LocalSID getLocalSID() {
        return (LocalSID)getChild(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public LocalSID getLocalSIDNoTransform() {
        return (LocalSID)getChildNoTransform(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 55
    private byte tokenbyte_Distribution;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setDistribution(byte value) {
        tokenbyte_Distribution = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public byte getDistribution() {
        return tokenbyte_Distribution;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 55
    private boolean tokenboolean_HasDesc;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setHasDesc(boolean value) {
        tokenboolean_HasDesc = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public boolean getHasDesc() {
        return tokenboolean_HasDesc;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 55
    private boolean tokenboolean_RemoteConnect;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setRemoteConnect(boolean value) {
        tokenboolean_RemoteConnect = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public boolean getRemoteConnect() {
        return tokenboolean_RemoteConnect;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 55
    private String tokenString_Protocol;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setProtocol(String value) {
        tokenString_Protocol = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getProtocol() {
        return tokenString_Protocol;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 55
    private boolean tokenboolean_Reliable;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setReliable(boolean value) {
        tokenboolean_Reliable = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public boolean getReliable() {
        return tokenboolean_Reliable;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 55
    private String tokenString_VersionName;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setVersionName(String value) {
        tokenString_VersionName = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getVersionName() {
        return tokenString_VersionName;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 55
    private String tokenString_ServiceHelpText;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setServiceHelpText(String value) {
        tokenString_ServiceHelpText = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getServiceHelpText() {
        return tokenString_ServiceHelpText;
    }

    // Declared in Hgraph.jadd at line 184


	  public void setName(String name){
		refined_PalcomResourceDescriptor_setName(name);
		if(name != null){
			int i = _node.setNodeName(getTagName() + " - " + name, null);
			if(i != IHgraphNode.STATUS_OK){
				System.out.println("Could not set node name: " + name + " , errcode: " + i);
			}else{
				_node.getValueFunc().setValue("name", name, null);
			}		
		}
	}

    // Declared in Hgraph.jadd at line 177

		
	/*refine PalcomResourceDescriptor public void PRDService.setSelector(Selector selector){
		PalcomResourceDescriptor.PRDService.setSelector(selector);
		if(selector != null){
			_node.getValueFunc().setValue("selector", selector);		
		}
	}
	
	refine PalcomResourceDescriptor public void PRDService.setRole(String role){
		PalcomResourceDescriptor.PRDService.setRole(role);
		if(role != null){
			_node.getValueFunc().setValue("role", role);		
		}
	}
	
	refine PalcomResourceDescriptor public void PRDService.setContentType(String contentType){
		PalcomResourceDescriptor.PRDService.setContentType(contentType);
		if(contentType != null){
			_node.getValueFunc().setValue("contentType", contentType);		
		}
	}*/
	
	  public void setLocalSID(LocalSID localSID){
		refined_PalcomResourceDescriptor_setLocalSID(localSID);
		if(localSID != null){
			_node.getValueFunc().setValue("localSID", localSID, null);		
		}
	}

}
