/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;


public class Param extends ControlItem implements Cloneable {
    public void flushCache() {
        super.flushCache();
        getBrowserLabel_visited = false;
    }
    public Object clone() throws CloneNotSupportedException {
        Param node = (Param)super.clone();
        node.getBrowserLabel_visited = false;
    return node;
    }
    public ASTNode copy() {
      try {
          Param node = (Param)clone();
          if(children != null) node.children = (ASTNode[])children.clone();
          return node;
      } catch (CloneNotSupportedException e) {
      }
      System.err.println("Error: Could not clone node of type " + getClass().getName() + "!");
      return null;
    }
    public ASTNode fullCopy() {
        Param res = (Param)copy();
        for(int i = 0; i < getNumChild(); i++) {
          ASTNode node = getChildNoTransform(i);
          if(node != null) node = node.fullCopy();
          res.setChild(node, i);
        }
        return res;
    }
    // Declared in ServiceDescription.jadd at line 53


    private byte[] data;

    // Declared in ServiceDescription.jadd at line 95

    

    public Param(String id, String type) {
        setInfo(new ParamInfo(id,type));
    }

    // Declared in ServiceDescription.jadd at line 99


    public Param(String id, String type, byte[] data) {
        setInfo(new ParamInfo(id,type));
        this.data = data;
    }

    // Declared in ServiceDescription.jadd at line 134


    public PRDData getInfoAsInfo() {
        return getInfo();
    }

    // Declared in ServiceDescription.jadd at line 162

    
    public String getType() { 
        return getInfo().getType();
    }

    // Declared in ServiceDescription.jadd at line 166

    
    public int getDataRef() {
        return getInfo().getDataRef();
    }

    // Declared in ServiceDescription.jadd at line 178


    public String getHelp() {
        return getInfo().getHelp();
    }

    // Declared in ServiceDescription.jadd at line 202


    public void setHelp(String help) {
        getInfo().setHelp(help);
    }

    // Declared in ServiceDescription.jadd at line 371


    public void setDataRefsAndAddDataToList(ArrayList dataList) {
        if (data != null) {
            dataList.add(data);
            getInfo().setDataRef(dataList.size());
        } else {
            getInfo().setDataRef(-1);
        }
    }

    // Declared in ServiceDescription.jadd at line 404


    public void setDataAccordingToDataRefs(byte[][] data) {
        if (getDataRef() == -1) {
            setData(null);
        } else {
            setData(data[getDataRef() - 1]);
        }
    }

    // Declared in ServiceDescription.jadd at line 415


    public byte[] getData() {
        return data;
    }

    // Declared in ServiceDescription.jadd at line 419


    public void setData(byte[] data) {
        this.data = data;
    }

    // Declared in ServiceDescription.jadd at line 436


    /** Returns a copy of this param, with the same values for all attributes. */
    public Param getCopy() {
        Param copy = new Param(getID(), getType());
        byte[] copiedData;
        if (data == null) {
            copiedData = null;
        } else {
            copiedData = new byte[data.length];
            System.arraycopy(data, 0, copiedData, 0, data.length);
        }
        copy.setData(copiedData);
        return copy;
    }

    // Declared in Comparison.jadd at line 242

    public boolean isAboutTheSameEntityAs(PRDData data) {
        return getInfo().isAboutTheSameEntityAs(data);
    }

    // Declared in PalcomResourceDescriptor.ast at line 3
    // Declared in PalcomResourceDescriptor.ast line 78

    public Param() {
        super();

        setChild(null, 0);

    }

    // Declared in PalcomResourceDescriptor.ast at line 11


    // Declared in PalcomResourceDescriptor.ast line 78
    public Param(ParamInfo p0) {
        setChild(p0, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 15


  protected int numChildren() {
    return 1;
  }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 78
    public void setInfo(ParamInfo node) {
        setChild(node, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public ParamInfo getInfo() {
        return (ParamInfo)getChild(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public ParamInfo getInfoNoTransform() {
        return (ParamInfo)getChildNoTransform(0);
    }

    protected boolean getBrowserLabel_visited = false;
    // Declared in LabelProviders.jrag at line 113
    public String getBrowserLabel() {
        if(getBrowserLabel_visited)
            throw new RuntimeException("Circular definition of attr: getBrowserLabel in class: ");
        getBrowserLabel_visited = true;
        String getBrowserLabel_value = getBrowserLabel_compute();
        getBrowserLabel_visited = false;
        return getBrowserLabel_value;
    }

    private String getBrowserLabel_compute()  {
		if(getData() != null) {
			if(getType().contains("text"))
				return getID() +"\r\n" + new String(getData());
			else if(getType().equals("image/jpeg")) {
				return "jpeg image";
			} else {
				return "???";
			}
		}else{
			return getID();
		}
	}

}
