/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;


public class ServiceProxy extends Group implements Cloneable {
    public void flushCache() {
        super.flushCache();
    }
    public Object clone() throws CloneNotSupportedException {
        ServiceProxy node = (ServiceProxy)super.clone();
    return node;
    }
    public ASTNode copy() {
      try {
          ServiceProxy node = (ServiceProxy)clone();
          if(children != null) node.children = (ASTNode[])children.clone();
          return node;
      } catch (CloneNotSupportedException e) {
      }
      System.err.println("Error: Could not clone node of type " + getClass().getName() + "!");
      return null;
    }
    public ASTNode fullCopy() {
        ServiceProxy res = (ServiceProxy)copy();
        for(int i = 0; i < getNumChild(); i++) {
          ASTNode node = getChildNoTransform(i);
          if(node != null) node = node.fullCopy();
          res.setChild(node, i);
        }
        return res;
    }
    // Declared in DeviceTree.jadd at line 360


	public ControlService getParentService() {
		return (ControlService) getParent().getParent();
	}

    // Declared in HLAAddress.jadd at line 5


	public AbstractResourceAddress getAddress() {
		return new TemporaryAddress(getLocalSID());
	}

    // Declared in Info.jadd at line 57

    
    public LocalSID getLocalSID() {
        return ((PRDServiceFMDescription) getInfo()).getLocalSID();
    }

    // Declared in Info.jadd at line 61


    public DeviceID getDeviceID() {
        if (getLocalSID() == null) {
            return null;
        } else {
            return getLocalSID().getDeviceID();
        }
    }

    // Declared in ServiceDescription.jadd at line 50


    /** MIME type for control data. */
    public static final String CONTENT_TYPE = "application/x-palcom-control+xml";

    // Declared in ServiceDescription.jadd at line 63
   

    /**
     * List of PalcomThreads that have registered interest in InCommands invoked
     * on this ServiceProxy.
     */
    private ArrayList inCommandReceivers
        = new ArrayList();

    // Declared in ServiceDescription.jadd at line 70


    /**
     * List of PalcomThreads that have registered interest in OutCommands
     * received at this ServiceProxy.
     */
    private ArrayList outCommandReceivers
        = new ArrayList();

    // Declared in ServiceDescription.jadd at line 110

    
    public ServiceProxy(String id) {
    	this();
        setInfo(new PRDServiceFMDescription(id));
    }

    // Declared in ServiceDescription.jadd at line 130

    
    public PRDData getInfoAsInfo() {
        return getInfo();
    }

    // Declared in ServiceDescription.jadd at line 194


    public void setHelp(String help) {
        getInfo().setHelp(help);
    }

    // Declared in ServiceDescription.jadd at line 220

    
    public ControlItem findItem(String id) {
        if (id.equals(getID())) {
            return this;
        } else {
            for (int i = 0; i < getNumControlItem(); i++) {
                ControlItem match = getControlItem(i).findItem(id);
                if (match != null) {
                    return match;
                }
            }
            return null;
        }
    }

    // Declared in ServiceDescription.jadd at line 275


    /** Implements this method for a ServiceProxy, by returning itself. */
    public ServiceProxy getServiceProxy() {
        return this;
    }

    // Declared in ServiceDescription.jadd at line 313


    /**
     * Implements this method for groups (overrides the method in ControlItem).
     * Searches the subtree for matches.
     */
    public Command findMatchingCommand(Command command) {
        for (int i = 0; i < getNumControlItem(); i++) {
            Command match = getControlItem(i).findMatchingCommand(command);
            if (match != null) {
                return match;
            }
        }
        return null;
    }

    // Declared in ServiceDescription.jadd at line 359


    public void setDataRefsAndAddDataToList(ArrayList dataList) {
        for (int i = 0; i < getNumControlItem(); i++) {
            getControlItem(i).setDataRefsAndAddDataToList(dataList);
        }
    }

    // Declared in ServiceDescription.jadd at line 392


    public void setDataAccordingToDataRefs(byte[][] data) {
        for (int i = 0; i < getNumControlItem(); i++) {
            getControlItem(i).setDataAccordingToDataRefs(data);
        }
    }

    // Declared in ServiceDescription.jadd at line 521

    
    /**
     * Update this ServiceProxy with the parameter values of a command,
     * and invokes the command. If there is a matching command in this
     * ServiceProxy, the local command will be updated, and the local
     * command (not the received command) will be sent to registered receivers,
     * by a call to invokeCommand.
     */
    public void update(Command command) {
        Command match = findMatchingCommand(command);
        if (match != null) {
            match.update(command);
            invokeCommand(match);
        }
    }

    // Declared in ServiceDescription.jadd at line 530


    /** Add a receiver for InCommands invoked on this ServiceProxy. */
    public void addInCommandReceiver(PalcomThread receiver) {
        inCommandReceivers.add(receiver);
    }

    // Declared in ServiceDescription.jadd at line 535


    /** Remove a receiver for InCommands. */
    public void removeInCommandReceiver(PalcomThread receiver) {
        inCommandReceivers.remove(receiver);
    }

    // Declared in ServiceDescription.jadd at line 540


    /** Add a receiver for OutCommands received at this ServiceProxy. */
    public void addOutCommandReceiver(PalcomThread receiver) {
        outCommandReceivers.add(receiver);
    }

    // Declared in ServiceDescription.jadd at line 545


    /** Remove a receiver for OutCommands. */
    public void removeOutCommandReceiver(PalcomThread receiver) {
        outCommandReceivers.remove(receiver);
    }

    // Declared in ServiceDescription.jadd at line 555


    /**
     * Invoke a command in this ServiceProxy. The command will be sent to
     * registered receivers, depending on the actual type of the command. If it
     * is an InCommand, it will be sent to InCommand receivers. If it is an
     * OutCommand, it will be sent to OutCommand receivers.
     */
    public void invokeCommand(Command command) {
        // TODO Is copy needed?
        command = command.getCopy();
        if (command.getDirection().equals(Command.DIRECTION_IN)) {
            fireInCommand(command);
        } else if (command.getDirection().equals(Command.DIRECTION_OUT)) {
            fireOutCommand(command);
        }
    }

    // Declared in ServiceDescription.jadd at line 569


    /**
     * Notifies all receivers that have registered interest for notification on
     * InCommands.
     */
    private void fireInCommand(Command command) {
        fireCommand(inCommandReceivers, command);
    }

    // Declared in ServiceDescription.jadd at line 577


    /**
     * Notifies all listeners that have registered interest for notification on
     * OutCommands.
     */
    private void fireOutCommand(Command command) {
        fireCommand(outCommandReceivers, command);
    }

    // Declared in ServiceDescription.jadd at line 584


    /**
     * Notifies all receivers in list about a command.
     */
    private void fireCommand(ArrayList list, Command command) {
        for (int i = 0; i < list.size(); i++) {
            PalcomThread receiver = (PalcomThread) list.get(i);
            receiver.putEvent(new CommandEvent(command, this));
        }
    }

    // Declared in ServiceDescription.jadd at line 603

    
    public void addControlItemAndInfo(ControlItem item) {
        addControlItem(item);
        //getInfo().addControlInfo(item.getInfo());
        if(item instanceof Command) {
            getInfo().addControlInfo(((Command)item).getInfo());
        }else if(item instanceof Group) {
            getInfo().addControlInfo(((Group)item).getInfo());
        }else if(item instanceof Param) {
            getInfo().addControlInfo(((Param)item).getInfo());
        }
    }

    // Declared in Comparison.jadd at line 248

    public boolean isAboutTheSameEntityAs(PRDData data) {
        return getInfo().isAboutTheSameEntityAs(data);
    }

    // Declared in PalcomResourceDescriptor.ast at line 3
    // Declared in PalcomResourceDescriptor.ast line 80

    public ServiceProxy() {
        super();

        setChild(null, 0);
        setChild(new List(), 1);

    }

    // Declared in PalcomResourceDescriptor.ast at line 12


    // Declared in PalcomResourceDescriptor.ast line 80
    public ServiceProxy(GroupInfo p0, List p1) {
        setChild(p0, 0);
        setChild(p1, 1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 17


  protected int numChildren() {
    return 2;
  }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 76
    public void setInfo(GroupInfo node) {
        setChild(node, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public GroupInfo getInfo() {
        return (GroupInfo)getChild(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public GroupInfo getInfoNoTransform() {
        return (GroupInfo)getChildNoTransform(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 76
    public void setControlItemList(List list) {
        setChild(list, 1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 6


    public int getNumControlItem() {
        return getControlItemList().getNumChild();
    }

    // Declared in PalcomResourceDescriptor.ast at line 10


    public ControlItem getControlItem(int i) {
        return (ControlItem)getControlItemList().getChild(i);
    }

    // Declared in PalcomResourceDescriptor.ast at line 14


    public void addControlItem(ControlItem node) {
        List list = getControlItemList();
        list.setChild(node, list.getNumChild());
    }

    // Declared in PalcomResourceDescriptor.ast at line 19


    public void setControlItem(ControlItem node, int i) {
        List list = getControlItemList();
        list.setChild(node, i);
    }

    // Declared in PalcomResourceDescriptor.ast at line 23

    public List getControlItemList() {
        return (List)getChild(1);
    }

    // Declared in PalcomResourceDescriptor.ast at line 27


    public List getControlItemListNoTransform() {
        return (List)getChildNoTransform(1);
    }

}
