/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package ist.palcom.resource.descriptor;
import ist.palcom.resourceAddressing.ResourceAddressingManager;import ist.palcom.debugging.DebugDispatcher;import ist.palcom.device.DeviceContext;import ist.palcom.services.SelfTestProvider;import ist.palcom.assemblies.SemanticError;import java.lang.Object;import ist.palcom.assemblies.Assembly;import java.util.ArrayList;import java.util.Iterator;import java.io.PrintStream;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import java.util.ArrayList;import ist.palcom.mal.Message;import ist.palcom.mal.MultiPartMessage;import ist.palcom.mal.SimpleMessage;import ist.palcom.palcomthreads.PalcomThread;import ist.palcom.discovery.CommandEvent;import ist.palcom.hgraph.IHgraphNode;import ist.palcom.hgraph.HgraphNode;import ist.palcom.hgraph.Path;import ist.palcom.hgraph.ValueFunc;import ist.palcom.hgraph.FuncFunc;import ist.palcom.hgraph.SiblingFunc;import java.util.Iterator;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;import ist.palcom.xml.XMLFactory;import java.io.ByteArrayOutputStream;import java.io.IOException;import org.kxml2.io.KXmlSerializer;import ist.palcom.xml.XMLRepresentable;import org.xmlpull.v1.XmlPullParser;



public class URN extends ASTNode implements Cloneable {
    public void flushCache() {
        super.flushCache();
        getBrowserLabel_visited = false;
    }
    public Object clone() throws CloneNotSupportedException {
        URN node = (URN)super.clone();
        node.getBrowserLabel_visited = false;
    return node;
    }
    public ASTNode copy() {
      try {
          URN node = (URN)clone();
          if(children != null) node.children = (ASTNode[])children.clone();
          return node;
      } catch (CloneNotSupportedException e) {
      }
      System.err.println("Error: Could not clone node of type " + getClass().getName() + "!");
      return null;
    }
    public ASTNode fullCopy() {
        URN res = (URN)copy();
        for(int i = 0; i < getNumChild(); i++) {
          ASTNode node = getChildNoTransform(i);
          if(node != null) node = node.fullCopy();
          res.setChild(node, i);
        }
        return res;
    }
    // Declared in Addressing.jadd at line 39


    /** Prefix for PalCom URNs. */
    public static final String URN_PREFIX = "urn:palcom://";

    // Declared in Addressing.jadd at line 44


    public String toString() {
        return getString();
    }

    // Declared in Addressing.jadd at line 105

    
    public URN(String urnString) {
        if (urnString == null) {
            setLeaf(null);
        } else {
            int startIndex = 0;
            if (urnString.startsWith(URN_PREFIX)) {
                startIndex = URN_PREFIX.length();
            }
            int firstSlashIndex = urnString.indexOf("/", startIndex);
            int lastSlashIndex = urnString.lastIndexOf('/');
            if (firstSlashIndex == -1) {
                setDevicePart(urnString.substring(startIndex));
                setParentPath(null);
                setLeaf(null);
            } else {
                setDevicePart(urnString.substring(startIndex, firstSlashIndex));
                if (lastSlashIndex == firstSlashIndex) {
                    setParentPath(null);
                } else {
                    setParentPath(urnString.substring(firstSlashIndex+1,
                            lastSlashIndex));
                }
                setLeaf(new URNLeaf(urnString.substring(lastSlashIndex + 1)));
            }
        }
    }

    // Declared in Addressing.jadd at line 154

    
    public String getString() {
        ist.palcom.base.StringBuilder builder
        	= ist.palcom.base.StringBuilder.instantiate();
        builder.append(URN_PREFIX);
        builder.append(getDevicePart());
        if (getParentPath() != null) {
            builder.append("/");
            builder.append(getParentPath());
        }
        if (getLeaf() != null) {
            builder.append("/");
            builder.append(getLeaf().getString());
        }
        return builder.toString();
    }

    // Declared in Addressing.jadd at line 174

    
    /**
     * @return The device URN part of this URN, as a new URN, or null if not
     *         defined.
     */
    public URN getDeviceURN() {
        if (getDevicePart() == null) {
            return null;
        } else {
            return new URN(getDevicePart(), null, null);
        }
    }

    // Declared in Addressing.jadd at line 182

    
    public URN createSubUrn(URNLeaf leaf) {
        if (getParentPath() == null) {
            String parentPath = null;
            if (getLeaf() != null) {
                parentPath = getLeaf().getString();
            }
            return new URN(getDevicePart(), parentPath, leaf);
        } else {
            ist.palcom.base.StringBuilder pathBuilder
            	= ist.palcom.base.StringBuilder.instantiate();
            pathBuilder.append(getParentPath());
            pathBuilder.append("/");
            pathBuilder.append(getLeaf().getString());
            return new URN(getDevicePart(),
                pathBuilder.toString(),
                leaf);
        }
    }

    // Declared in Comparison.jadd at line 37

    public boolean equals(Object o) {
        if (o instanceof URN) {
            return AddressPart.safeEqual(((URN) o).getString(), getString());
        } else {
            return false;
        }
    }

    // Declared in Comparison.jadd at line 90


    public int hashCode() {
        return getString().hashCode();
    }

    // Declared in PalcomResourceDescriptor.ast at line 3
    // Declared in PalcomResourceDescriptor.ast line 82

    public URN() {
        super();

        setChild(null, 0);

    }

    // Declared in PalcomResourceDescriptor.ast at line 11


    // Declared in PalcomResourceDescriptor.ast line 82
    public URN(String p0, String p1, URNLeaf p2) {
        setDevicePart(p0);
        setParentPath(p1);
        setChild(p2, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 17


  protected int numChildren() {
    return 1;
  }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 82
    private String tokenString_DevicePart;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setDevicePart(String value) {
        tokenString_DevicePart = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getDevicePart() {
        return tokenString_DevicePart;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 82
    private String tokenString_ParentPath;

    // Declared in PalcomResourceDescriptor.ast at line 3

    public void setParentPath(String value) {
        tokenString_ParentPath = value;
    }

    // Declared in PalcomResourceDescriptor.ast at line 6

    public String getParentPath() {
        return tokenString_ParentPath;
    }

    // Declared in PalcomResourceDescriptor.ast at line 2
    // Declared in PalcomResourceDescriptor.ast line 82
    public void setLeaf(URNLeaf node) {
        setChild(node, 0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 5

    public URNLeaf getLeaf() {
        return (URNLeaf)getChild(0);
    }

    // Declared in PalcomResourceDescriptor.ast at line 9


    public URNLeaf getLeafNoTransform() {
        return (URNLeaf)getChildNoTransform(0);
    }

    protected boolean getBrowserLabel_visited = false;
    // Declared in LabelProviders.jrag at line 53
    public String getBrowserLabel() {
        if(getBrowserLabel_visited)
            throw new RuntimeException("Circular definition of attr: getBrowserLabel in class: ");
        getBrowserLabel_visited = true;
        String getBrowserLabel_value = getBrowserLabel_compute();
        getBrowserLabel_visited = false;
        return getBrowserLabel_value;
    }

    private String getBrowserLabel_compute() {  return  getString();  }

}
