/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.test;

import ist.palcom.communication.CommunicationManager;
import ist.palcom.mal.MediaManager;
import ist.palcom.mal.Message;
import ist.palcom.mal.MessageEvent;
import ist.palcom.mal.udp.UDPMediaManager;
import ist.palcom.messaging.PointToPointManager;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.resource.descriptor.URN;
import ist.palcom.routing.util.AbstractRoutingManager;
import ist.palcom.routing.util.DSDVRoutingManager;
import ist.palcom.routing.util.RoutingPubSubManager;

/**
 * @author Jacob Mahler-Andersen, jmahle@daimi.au.dk
 */
public class DSDVRoutingSimpleTest {
	
	public static void main(String[] args) {
		DSDVRoutingSimpleTest test = new DSDVRoutingSimpleTest();
		
		test.testSimpleRouting();
		System.out.println();
	}
	
	public void testSimpleRouting() {
		 System.out.println("--- testSimpleRouting ---");

	        final PalcomScheduler scheduler = new PalcomScheduler();
	        PalcomThread t = new PalcomPriorityThread() {
	            public void run() {
	                MediaManager mediaManager = new UDPMediaManager(scheduler, null);
	                URN receiverURN = new URN("urn:palcom://blabla");
	                AbstractRoutingManager routingManager = new DSDVRoutingManager(scheduler, mediaManager, new RoutingPubSubManager(scheduler, mediaManager), receiverURN);
	                CommunicationManager comManager = new CommunicationManager(scheduler, routingManager);
	                PointToPointManager pointToPointManager = new PointToPointManager(comManager);
	                
	                PalcomPriorityThread p = new MessageThread();
	                scheduler.scheduleThread(p);
	                pointToPointManager.startReceiving2(p);
	                
	            }
	        };
	        scheduler.scheduleThread(t);
	        System.out.println("Scheduler started");
	        scheduler.run();
	}
    
    private class MessageThread extends PalcomPriorityThread {
        MessageThread() { }

        public void run() {

        	while (true) {
    			MessageEvent event = (MessageEvent) waitEvent();
        		Message msg = event.getMessage();
                
        		byte[] receivedData = new byte[msg.getTotalLength()];
        		msg.write(receivedData, 0);
                
                String data = "";
                
                int offset = msg.getTotalLength() - msg.getDataLength();
                for (int i = offset; i < receivedData.length; i++) {
                    data += (char) receivedData[i];
                }
                //System.out.println("MT: " + data);
            }           
        }
    }
	
}
