/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.util;


import ist.palcom.mal.MessageEvent;
import ist.palcom.palcomthreads.Event;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PeriodicEvent;
import ist.palcom.routing.util.messages.BootstrapMessage;

/**
 * DSDVBoostrap is used to bootstrap the DSDVRoutingManager on a network.
 * When a new peer with a DSDVRoutingManager arrives to the network, it publish
 * it's RoutingURN to all the listeners.
 * 
 * @author Jacob Mahler-Andersen, jmahle@daimi.au.dk
 *
 */
public class DSDVBootstrap {
	
	private static final String topic = "DSDVRouting";
	
	private DSDVRoutingManager manager;
	
	private RoutingPubSubManager pubsub;
	
	private boolean published;
	
	
	/**
	 * TODO: Use multicast directly - not the PubSubManager
	 */
	public DSDVBootstrap(DSDVRoutingManager manager, RoutingPubSubManager pubsub) {
		this.manager = manager;
		this.pubsub = pubsub;
	}
	
	
	/**
	 * Publish bootstrap messages until first message is received (RASCAL issue)  
	 *  
	 * @return PalcomPriorityThread
	 */
	public PalcomPriorityThread startPublish() {
		
		PalcomPriorityThread thread = new PalcomPriorityThread() {
			 public void run() {
				 BootstrapMessage msg = new BootstrapMessage(manager.getRoutingURN(), manager.getRoutingSelector());
				
				 //while (!published) {
				 int i = 0;
				 while (true) {
					 pubsub.publish(topic, msg, manager.getRoutingSelector());
					 //mediaManager.sendBroadcast(multiPartMessage, senderURN, senderSelector);
					 
					 enablePeriodicEvents(2000); // Send every 2 sec
					 waitEvent();
					 disablePeriodicEvents();
					 i++;
				 }
			 }
		};

		return thread;
	}
	
	
	/**
	 * Subscribe on the topic {@link #topic} where new arrivals publish
	 * their RoutingURL. 
	 *  
	 * @return PalcomPriorityThread
	 */
	public PalcomPriorityThread startListening() {
		
		PalcomPriorityThread thread = new PalcomPriorityThread() {
			 public void run() {
				 
				 while (true) {
					 MessageEvent event = (MessageEvent) this.waitEvent();
					 BootstrapMessage message = BootstrapMessage.unmarshall(event.getMessage());
					 
		
					 updateDSDVTable(message);
				 }
				 
			 }
		};
		
		// Subscribe on topic
		pubsub.subscribe(topic, thread);

		return thread;
	}
	
	/*
	 * Update the DSDV-table
	 */
	private void updateDSDVTable(BootstrapMessage message) {
		manager.updateStatus(message.getRoutingURN());
		
		// Do not add our own Routing-URN
		if (message.getRoutingURN().equals(manager.getRoutingURN()) && 
				message.getRoutingSelector().equals(manager.getRoutingSelector())) {
			return;
		}

		// Add the routing manager
		manager.addLocalPeer(message.getRoutingURN(), message.getRoutingSelector());
	}
}