/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.util;


import ist.palcom.collections.Link;
import ist.palcom.collections.LinkedList;
import ist.palcom.resource.descriptor.Selector;
import ist.palcom.resource.descriptor.URL;
import ist.palcom.resource.descriptor.URN;

import java.util.ArrayList;

/**
 * DSDV-Peer
 * 
 * @author Jacob Mahler-Andersen, jmahle@daimi.au.dk
 */
public class DSDVPeer extends Link {
	
	private URN routingURN;
	private Selector routingSelector;
	private URN nextHopURN;
	private Selector nextHopSelector;
	private int metric;
	private int seq_nr;
	private boolean isActive;
	
	
	/*
	 * Status indicates if this peer reply on our ping messages
	 * 0: Replied with a pong message
	 * 1: We have send one ping, but no pong reply
	 * 2: Send two ping, but no reply
	 */
	private int status;
	
	public DSDVPeer(URN routingURN, Selector routingSelector, URN nextHopURN, Selector nextHopSelector, int metric, int seq_nr) {
		this.routingURN = routingURN;
		this.routingSelector = routingSelector;
		this.nextHopURN = nextHopURN;
		this.nextHopSelector = nextHopSelector;
		this.seq_nr = seq_nr;
		this.metric = metric;
		status = 0;
		isActive = true;
	}
	
	public URN getRoutingURN() {
		return routingURN;
	}
	
	public Selector getRoutingSelector() {
		return routingSelector;
	}
	
	public URN getNextHopURN() {
		return nextHopURN;
	}
	
	public Selector getNextHopSelector() {
		return nextHopSelector;
	}
	
	public void setNextHop(URN nextHopURN, Selector nextHopSelector) {
		this.nextHopURN = nextHopURN;
		this.nextHopSelector = nextHopSelector;
	}
	
	public int getMetric() {
		return metric;
	}
	
	public void setMetric(int new_metric) {
		metric = new_metric;
	}
	
	public int getSequenceNumber() {
		return seq_nr;
	}
	
	public void setSequenceNumber(int new_seqnr) {
		seq_nr = new_seqnr;
	}
	
	public int getStatus() {
		return status;
	}
	
	public void setStatus(int newstatus) {
		if (newstatus == 0)
			isActive = true;
		
		status = newstatus;
	}
	
	public DSDVPeer copy() {
		DSDVPeer copy = new DSDVPeer(routingURN, routingSelector, nextHopURN, nextHopSelector, metric, seq_nr);
		copy.setIsActive(isActive());
		return copy;
	}
	
	public void setIsActive(boolean active) {
		isActive = active;
	}
	
	public boolean isActive() {
		return isActive;
	}
	
	public String toString() {
		return "DSDVPeer - RoutingURN: " + 
		routingURN + " | RoutingSelector: " + routingSelector + " | nextHopURN: " + nextHopURN + " | nextHopSelector " + 
		nextHopSelector + " | Metric: " + metric + " | Sequence nr: " + seq_nr + " | Status: " + status + " | Active: " + 
		isActive;
	}
	
	/**
	 * Marshalling
	 * @return String
	 */
	public String marshall() {
		String output = "";
		
		output += routingURN;
		output += "$";
		
		output += routingSelector;
		output += "$";
		
		output += nextHopURN;
		output += "$";
		
		output += nextHopSelector;
		output += "$";
		
		output += metric;
		output += "$";
		
		output += seq_nr;
		output += "$";
		
		// True
		if (isActive())
			output += "1";
		// False
		else
			output += "0";
		
		return output;
	}
	
	/**
	 * Unmarshalling
	 * @param input The String from the marshalling
	 * @return DSDVPeer
	 */
	public static DSDVPeer unmarshall(String input) {
		int i = input.indexOf("$");
		int j = input.indexOf("$", i+1);
		
		URN newRoutingURN = new URN(input.substring(0, i));
		
		Selector newRoutingSelector = new Selector(input.substring(i+1, j));
		
		i = j;
		j = input.indexOf("$", i+1);
		
		URN newNextHopURN = new URN(input.substring(i+1, j));
		
		i = j;
		j = input.indexOf("$", i+1);
		
		Selector newNextHopSelector = new Selector(input.substring(i+1, j));
		
		i = j;
		j = input.indexOf("$", i+1);
		
		int newmetric = Integer.parseInt(input.substring(i+1, j));
		
		i = j;
		j = input.indexOf("$", i+1);
		
		int newseq_nr = Integer.parseInt(input.substring(i+1, j));
		int newisactive = Integer.parseInt(input.substring(j+1));
		
		DSDVPeer peer = new DSDVPeer(newRoutingURN, newRoutingSelector, newNextHopURN, 
				newNextHopSelector, newmetric, newseq_nr);
		if (newisactive == 0)
			peer.setIsActive(false);
			
		return peer;
	}
}