/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.util;

import ist.palcom.resource.descriptor.PRDDevice;
import ist.palcom.resource.descriptor.Selector;
import ist.palcom.resource.descriptor.URL;
import ist.palcom.resource.descriptor.URN;
import ist.palcom.mal.MediaManager;
import ist.palcom.mal.Message;
import ist.palcom.mal.MessageEvent;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.palcomthreads.PalcomScheduler;
import ist.palcom.palcomthreads.PalcomThread;
import ist.palcom.routing.util.messages.AbstractRoutingMessage;

import java.util.ArrayList;


/**
 * Very simple multicast forwarder. This manager will forward all multicast messages received
 * to the nodes reachable by its mediamanager and deliver the message to the thread the
 * subscribe to the manager.
 */
public class SimpleMulticastRoutingManager extends AbstractRoutingManager{
	private ArrayList receivers = new ArrayList();
	private ReceiverThread receiverThread;
	private URL receivingURL;
	private MediaManager mediaManager;

	public SimpleMulticastRoutingManager(PalcomScheduler scheduler, MediaManager mediaManager, RoutingPubSubManager pubSubManager, URN deviceURN) {
		super(scheduler,mediaManager,pubSubManager, deviceURN);
		
		this.mediaManager = mediaManager;
		this.scheduler = scheduler;
	}
	
	// 
	// Starte receiving messages on the specified url. Messages will be delivered as
	// events to the receiving thread.
	// 
	// @param url - the url to receive messages on
	// @param receivingThread - the thread that receives events when messages are received
	// 
//	public URL startReceiving(PalcomThread receivingThread) {
//		addMessageReceiver(receivingThread);
//		if (receiverThread == null) {
//			receiverThread = new ReceiverThread();
//			URL[] addresses = manager.getDefaultGroupAddresses();
//			for (int i = 0; i < addresses.length; i++) {
//			    manager.startReceiving(addresses[i],
//			            receiverThread);
//			}
//			scheduler.scheduleThread(receiverThread);
//		}
//		return new URL("");
//	}
	
	public boolean send(Message message, URN senderURN, Selector senderSelector, URN receiverURN, Selector receiverSelector) {
		// TODO Auto-generated method stub
		return false;
	}

	// 
	// Remove a receiving thread from the list of receivers.
	// 
	// @param receiver - the receiver that dont want to receive messages anymore.
	// 
//	public void stopReceiving(PalcomThread receiver){
//		removeMessageReceiver(receiver);
//		if(receivers.size() == 0){
//			manager.stopReceiving(receivingURL, receiver);
//		}
//	}
	
	public void send(Message message){
        manager.sendBroadcast(message);
	}
	
	private void addMessageReceiver(PalcomThread receiver) {
        receivers.add(receiver);
    }

    private void removeMessageReceiver(PalcomThread receiver) {
        int index = 0;
        while (index < receivers.size() && receivers.get(index) != receiver) {
            index++;
        }
        if (index < receivers.size()) {
            receivers.remove(index);
        }
    }
	
	private class ReceiverThread extends PalcomPriorityThread {
	    public void run() {
	        while (true) {
	            MessageEvent event = (MessageEvent) waitEvent();
	            Message message = event.getMessage();
                manager.sendBroadcast(message);
	            for(int i = 0; i < receivers.size();i++){
	            	((PalcomThread)receivers.get(i)).putEvent(event);
	            }
	        }
	    }
	}

	protected void handleRoutingMessage(AbstractRoutingMessage message) {
		// Nothing to be done, as this is not using any routingmessages
	}

	protected void startRouting() {
		// No routing is done here.
	}

}
