/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.util.messages;

import ist.palcom.resource.descriptor.Selector;
import ist.palcom.resource.descriptor.URL;
import ist.palcom.resource.descriptor.URN;
import ist.palcom.mal.Message;

/**
 * Contains the RoutingManagers URL 
 * 
 * @author Jacob Mahler-Andersen, jmahle@daimi.au.dk
 *
 */
public class BootstrapMessage extends AbstractRoutingMessage {
	
	public static final String type = "boot";
	private URN routingURN;
	private Selector routingSelector;
	
	public BootstrapMessage(URN routingURN, Selector routingSelector) {
		super((type + " " + routingURN + " " + routingSelector).getBytes());
		this.routingURN = routingURN;
		this.routingSelector = routingSelector;
	}
	
	public URN getRoutingURN() {
		return routingURN;
	}
	
	public Selector getRoutingSelector() {
		return routingSelector;
	}
	
	public static BootstrapMessage unmarshall(Message message) throws IllegalArgumentException {
		
		byte[] receivedData = new byte[message.getTotalLength()];
        message.write(receivedData, 0);
        
        String data = "";
        
        int offset = message.getTotalLength() - message.getDataLength();
        for (int i = offset; i < receivedData.length; i++) {
            data += (char) receivedData[i];
        }

		if (!data.startsWith(type))
			throw new IllegalArgumentException("Input message was not a BootstrapMessage");
		
		int i = data.indexOf(" ");
		i++;
		int j = data.indexOf(" ", i);
		URN newRoutingURN = new URN(data.substring(i, j));
		Selector newRoutingSelector = new Selector(data.substring(j+1));
		
		return new BootstrapMessage(newRoutingURN, newRoutingSelector);
	}

}
