/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.util.messages;

import ist.palcom.resource.descriptor.Selector;
import ist.palcom.resource.descriptor.URL;
import ist.palcom.resource.descriptor.URN;
import ist.palcom.mal.Message;
import ist.palcom.mal.MessageEvent;
import ist.palcom.palcomthreads.Event;
import ist.palcom.routing.util.DSDVTable;

public class DSDVRoutingMessage extends AbstractRoutingMessage {
	
	public static final String type = "dsdvrouting";
	private Message message;
	private URN dsdvSenderURN;
	private Selector dsdvSenderSelector;
	private URN receiverServiceURN;
	private Selector receiverServiceSelector;
	
	public DSDVRoutingMessage(Message message, URN dsdvSenderURN, Selector dsdvSenderSelector, URN receiverServiceURN, Selector receiverServiceSelector) {
		super(getByteArray(message, dsdvSenderURN, dsdvSenderSelector, receiverServiceURN, receiverServiceSelector));
		this.message = message;
		this.dsdvSenderURN = dsdvSenderURN;
		this.dsdvSenderSelector = dsdvSenderSelector;
		this.receiverServiceURN = receiverServiceURN;
		this.receiverServiceSelector = receiverServiceSelector;
	}
	
	private static byte[] getByteArray(Message message, URN dsdvSenderURN, Selector dsdvSenderSelector, URN receiverServiceURN, Selector receiverServiceSelector) {
		byte[] b1 = (type + " " + dsdvSenderURN + " " + dsdvSenderSelector + " " + receiverServiceURN + " " + receiverServiceSelector + " ").getBytes();
		byte[] b2 = new byte[message.getTotalLength()];
		message.write(b2, 0);
		// Merge the two bytearrays
		byte[] b3 = new byte[b1.length + b2.length];
		int i = 0;
		for (int x = 0; x < b1.length; x++) {
			b3[i] = b1[x];
			i++;
		}
		for (int x = 0; x < b2.length; x++) {
			b3[i] = b2[x];
			i++;
		}
		
		return b3;
	}
	
	/**
	 * 
	 * @return The wrapped message 
	 */
	public Message getMessage() {
		return message;
	}
	
	public URN getDSDVSenderURN() {
		return dsdvSenderURN;
	}
	
	public Selector getDSDVSenderSelector() {
		return dsdvSenderSelector;
	}
	
	public URN getReceiverServiceURN() {
		return receiverServiceURN;
	}
	
	public Selector getReceiverServiceSelector() {
		return receiverServiceSelector;
	}
	
	public static DSDVRoutingMessage unmarshall(Message message) throws IllegalArgumentException {		
		byte[] receivedData = new byte[message.getTotalLength()];
        message.write(receivedData, 0);
        
        String data = "";
        int offset = message.getTotalLength() - message.getDataLength() + type.length()+1;
        
        // Types
        URN rSenderURN = null;
        Selector rSenderSelector = null;
        URN rURN = null;
        Selector rSelector = null;
        
        // Parse
        char c;
        int msgStart = -1;
        for (int i = offset; i < receivedData.length; i++) {
            c = (char) receivedData[i];
            if (c != ' ') {
            	data += c;
            	
            // Space: ' '
            } else {
            	
            	if (rSenderURN == null)
            		rSenderURN = new URN(data);
            	else if (rSenderSelector  == null)
            		rSenderSelector = new Selector(data);
            	else if (rURN == null)
            		rURN = new URN(data);
            	else if (rSelector == null) {
            		rSelector = new Selector(data);
            		msgStart = i+1;
            	}
            	else
            		break;
            		
            	data = "";
            }
        }

        // Wrapped message
		Message msg = Message.parse(receivedData, msgStart, receivedData.length - msgStart);
		return new DSDVRoutingMessage(msg, rSenderURN, rSenderSelector, rURN, rSelector);
	}

}
