/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.util.messages;

import ist.palcom.resource.descriptor.Selector;
import ist.palcom.resource.descriptor.URN;
import ist.palcom.mal.Message;
import ist.palcom.routing.util.DSDVTable;

public class DSDVUpdateMessage extends AbstractRoutingMessage {

	public static final String type = "update";
	
	private DSDVTable update;
	
	private URN dsdvSenderURN;
	
	private Selector dsdvSenderSelector;
	
	public DSDVUpdateMessage(URN dsdvSenderURN, Selector dsdvSenderSelector, DSDVTable update) {
		super((type + " " + dsdvSenderURN + " " + dsdvSenderSelector + " " + update.marshall()).getBytes());
		this.dsdvSenderURN = dsdvSenderURN;
		this.dsdvSenderSelector = dsdvSenderSelector;
		this.update = update;
	}
	
	public URN getDSDVSenderURN() {
		return dsdvSenderURN;
	}
	
	public Selector getDSDVSenderSelector() {
		return dsdvSenderSelector;
	}
	
	public DSDVTable getUpdateDSDVTable() {
		return update;
	}
	
	public static DSDVUpdateMessage unmarshall(Message message) throws IllegalArgumentException {
		
		byte[] receivedData = new byte[message.getTotalLength()];
        message.write(receivedData, 0);
        
        String tmpData = "";
        String msgtype = "";
        String senderURN = "";
        String senderSelector = "";
        String dsdvTable = "";
        
        int offset = message.getTotalLength() - message.getDataLength();
        for (int i = offset; i < receivedData.length; i++) {
            char tmp = (char) receivedData[i];
        	if (tmp == ' ') {
        		// The message type
        		if (msgtype.length() == 0) {
        			msgtype = tmpData;
        			tmpData = "";
        		} else if (senderURN.length() == 0) {
        			senderURN = tmpData;
        			tmpData = "";
        		} else if (senderSelector.length() == 0) {
        			senderSelector = tmpData;
        			tmpData = "";
        		}
        		
        	} else {
        		tmpData += tmp;
        	}
        }
        // The rest is the DSDV table
        dsdvTable = tmpData;

		if (!msgtype.startsWith(type))
			throw new IllegalArgumentException("Input message was not an UpdateMessage");
		
		DSDVTable table = DSDVTable.unmarshall(dsdvTable);
		return new DSDVUpdateMessage(new URN(senderURN), new Selector(senderSelector), table);
	}

}
