/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.routing.util.messages;

import ist.palcom.collections.LinkedList;
import ist.palcom.mal.Message;
import ist.palcom.resource.descriptor.URL;
import ist.palcom.routing.util.SMURFNode;

public class SMURFFullAdvertisement extends AbstractRoutingMessage {

	public static final String type = "SMURFFullAdvertisement";

	private LinkedList oneHopNeighbours = new LinkedList();
	
	private LinkedList MPRs = new LinkedList();

	private String sequence = "0";

	private String reluctanceValue = "0";

	private int numberOf1HopNeighbours = 0;

	private int numberOfMPRs = 0;

	private String C ="none"; 
	
	public SMURFFullAdvertisement(byte[] information) {
		super((type+" "+information.toString()).getBytes());
		/*if(super.getInformation().toString().startsWith(type)){
			return true;
		} else {
			return false;
		}*/
		
		/*
		 * Sequence$Reluctance$number of one hop neighbours$number of mprs$
		 * C$list of one hope neighbours - address*$list of MPR neighbours - address*
		 */
		
		String messageContent = information.toString();
		sequence = messageContent.substring(0,messageContent.charAt('$'));
		messageContent = messageContent.substring(messageContent.charAt('$'));
		
		reluctanceValue = messageContent.substring(0,messageContent.charAt('$'));
		messageContent = messageContent.substring(messageContent.charAt('$'));
		
		numberOf1HopNeighbours = Integer.parseInt(messageContent.substring(0,messageContent.charAt('$')));
		messageContent = messageContent.substring(messageContent.charAt('$'));
	
		numberOfMPRs = Integer.parseInt(messageContent.substring(0,messageContent.charAt('$')));
		messageContent = messageContent.substring(messageContent.charAt('$'));
		
		C = messageContent.substring(0,messageContent.charAt('$'));
		messageContent = messageContent.substring(messageContent.charAt('$'));
		
		for(int i=0; i < numberOf1HopNeighbours; i++){
			oneHopNeighbours.insertFirst(new SMURFNode(new URL(messageContent.substring(0,messageContent.charAt('$')))));
			messageContent = messageContent.substring(messageContent.charAt('$'));
		}
		
		for(int i=0; i < numberOfMPRs; i++){
			MPRs.insertFirst(new SMURFNode(new URL(messageContent.substring(0,messageContent.charAt('$')))));
			messageContent = messageContent.substring(messageContent.charAt('$'));
		}
	}
	
	/**
	 * Writing the content of the message into the byte array information
	 *
	 */
	private void updateMessage(){
		String informationString = sequence + "$";
		informationString += reluctanceValue + "$";
		informationString += oneHopNeighbours.getSize()+"$";
		informationString += MPRs.getSize()+"$";
		informationString += C+"$";
		for(int i=0;i<oneHopNeighbours.getSize();i++){
			informationString += ((SMURFNode)oneHopNeighbours.removeFirst()).getAddress()+"$";
		}
		for(int i=0;i<oneHopNeighbours.getSize();i++){
			informationString += ((SMURFNode)MPRs.removeFirst()).getAddress()+"$";
		}
		super.write((type+" "+informationString).getBytes(),0);
	}

	public String getC() {
		return C;
	}

	public void setC(String c) {
		C = c;
		updateMessage();
	}

	public LinkedList getMPRs() {
		return MPRs;
	}

	public void setMPRs(LinkedList rs) {
		MPRs = rs;
		updateMessage();
	}

	public LinkedList getOneHopNeighbours() {
		return oneHopNeighbours;
	}

	public void setOneHopNeighbours(LinkedList oneHopNeighbours) {
		this.oneHopNeighbours = oneHopNeighbours;
		updateMessage();
	}

	public String getReluctanceValue() {
		return reluctanceValue;
	}

	public void setReluctanceValue(String reluctanceValue) {
		this.reluctanceValue = reluctanceValue;
		updateMessage();
	}

	public String getSequence() {
		return sequence;
	}

	public void setSequence(String sequence) {
		this.sequence = sequence;
		updateMessage();
	}
}
