/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

 * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

 * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ist.palcom.service.manager;

import ist.palcom.collections.LinearMap;
import ist.palcom.collections.LinearSet;
import ist.palcom.debugging.DebugDispatcher;
import ist.palcom.device.DeviceContext;
import ist.palcom.device.FullXMLFactory;
import ist.palcom.discovery.CommandEvent;
import ist.palcom.palcomthreads.PalcomPriorityThread;
import ist.palcom.resource.descriptor.Command;
import ist.palcom.resource.descriptor.ControlItem;
import ist.palcom.resource.descriptor.DeviceID;
import ist.palcom.resource.descriptor.Group;
import ist.palcom.resource.descriptor.GroupInfo;
import ist.palcom.resource.descriptor.List;
import ist.palcom.resource.descriptor.LocalSID;
import ist.palcom.resource.descriptor.PRDService;
import ist.palcom.resource.descriptor.PRDSubList;
import ist.palcom.resource.descriptor.Param;
import ist.palcom.resource.descriptor.ServiceID;
import ist.palcom.resource.descriptor.ServiceInstanceID;
import ist.palcom.resource.descriptor.ServiceProxy;
import ist.palcom.resource.descriptor.SubListInfo;
import ist.palcom.resource.descriptor.URN;
import ist.palcom.resource.descriptor.UnboundServiceDecl;
import ist.palcom.resource.descriptor.UnboundServiceDeclList;
import ist.palcom.services.AbstractService;
import ist.palcom.services.CommandEventListener;
import ist.palcom.services.CommandListenerThread;
import ist.palcom.services.SelfTestProvider;
import ist.palcom.services.ServiceManagerConstants;
import ist.palcom.storage.pfs.FileStorage;
import ist.palcom.storage.pfs.Storage;
import ist.palcom.xml.XMLException;
import ist.palcom.xml.XMLRepresentable;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

public class ServiceManager extends AbstractService implements CommandEventListener, ServiceManagerConstants{
	public static final String COMMAND_SERVICES = "component_list";
//	public static final String COMMAND_UNLOAD_ALL = "unload all";
	public static final String COMMAND_LIST_SERVICES = "list_services";
//	private static final String COMMAND_LIST_REGISTERED = "list_registered";
	public static final String COMMAND_LOAD = "load";
//	private static final String COMMAND_UNLOAD = "unload";
	public static final String COMMAND_REGISTER = "register";
//	private static final String COMMAND_UNREGISTER = "unregister";


	private PRDSubList swcList;
	private static ServiceID serviceID = new ServiceID(
			new DeviceID("X:1"), "TF3", new DeviceID("X:1"), "TF3", null, null, null, null);
	private CommandListenerThread inCommandThread;
	private ArrayList loadedServices;
	private LinearMap registeredServices;

	private FileStorage fileStorage;
	private String swListFile;
	private String unboundServicesDir;

	public static boolean isSameService(ServiceID sid) {return serviceID.equals(sid);}

	public ServiceManager(DeviceContext context,
			String serviceName, String root)
	throws IOException {
		super(context, serviceName, true, false, PRDService.UNICAST, "P1",
				false, "1", "ServiceManager", createServiceProxy(serviceName), 
				new LocalSID(context.getDeviceInfo().getDeviceID(), context.getNewLocalSIDString()));

		setServiceInstanceID(new ServiceInstanceID(context.getDeviceInfo().getDeviceID(), serviceID, 
				context.getNewInstanceNumber(serviceID)));
		loadedServices = new ArrayList();
		registeredServices = new LinearMap();
		inCommandThread = new CommandListenerThread(this);
		context.getScheduler().scheduleThread(inCommandThread);
		getServiceProxy().addInCommandReceiver(inCommandThread);

		unboundServicesDir = root;
		fileStorage = new FileStorage(unboundServicesDir);

		ist.palcom.base.System.instance().addLookupPath(unboundServicesDir);
		System.out.println("ServiceManager: Storing unbound services list in="+unboundServicesDir);
		swListFile = unboundServicesDir+"/SoftwareComponents.lst"; 
		parseListFile();
		ist.palcom.base.System.instance().addLookupPath(unboundServicesDir);    
	}

	protected SelfTestProvider getSelfTestService(String urn) {
		return null;
	}

	private void readListFileOld() {
		System.out.println("readListFile");

		String fileContents = "";
		fileStorage.open(swListFile, 0);
		byte[] b = new byte[1];
		do{
			b[0] = (byte) fileStorage.read(swListFile);
			if(b[0]>=0) {
				fileContents += new String(b);
			}
		} while(b[0]>=0);
		fileStorage.close(swListFile);
		String[] tokens = fileContents.split("\n");
		for(int i = 0; i<tokens.length; ++i) {
			String[] listItem = tokens[i].split("=");
			String urn = listItem[0];
			if(listItem.length>=2) {
				String className = listItem[1];
				System.out.println(urn+" = "+className);
				registeredServices.put(new URN(urn), className);	
			} else {
				System.out.println("readFileList: error in file: no className for "+urn);
			}
		}
	}

//	fileStorage.open(swListFile, 0);
//	fileStorage.seek(swListFile, fileStorage.length(swListFile));
//	if(fileStorage.length(swListFile)>0) fileStorage.writeBytes(swListFile, "\n".getBytes(), 0, "\n".getBytes().length);
//	String tmp = siid + "="+className;
//	byte[] byteArray = tmp.getBytes();
//	int result = fileStorage.writeBytes(swListFile, byteArray, 0, byteArray.length);
//	fileStorage.close(swListFile);
//	if(result!=Storage.OK) {
//	System.err.println("Write to file operation for UnboundService failed");
//	}

	private void writeListFile() {
		LinearSet ks = registeredServices.keySet();
		Iterator i = ks.iterator();
		UnboundServiceDeclList result = new UnboundServiceDeclList();
		while(i.hasNext()) {
			ServiceInstanceID siid = (ServiceInstanceID) i.next();
			String className = (String) registeredServices.get(siid);
			result.addUnboundServiceDecl(new UnboundServiceDecl(siid, className));
		}
		fileStorage.open(swListFile, 0);
		byte[] byteArray = result.getXMLData();
		int fileresult = fileStorage.writeBytes(swListFile, byteArray, 0, byteArray.length);
		fileStorage.close(swListFile);
		if(fileresult!=Storage.OK) {
			System.err.println("Write to file operation for UnboundService failed");
		}
	}
	private void parseListFile() {
		String fileContents = "";
		fileStorage.open(swListFile, 0);
		//XXX Is this really the best way to read a file?
		byte[] b = new byte[1];
		do{
			b[0] = (byte) fileStorage.read(swListFile);
			if(b[0]>=0) {
				fileContents += new String(b);
			}
		} while(b[0]>=0);
		fileStorage.close(swListFile);
		UBSXMLFactory fff = new UBSXMLFactory();
		byte[] bytes = fileContents.getBytes();
		try {
			UnboundServiceDeclList tmp = (UnboundServiceDeclList) fff.createFromXMLData(bytes, 0, bytes.length);
			for(int i=0; i<tmp.getNumUnboundServiceDecl();i++) {
				UnboundServiceDecl sd = tmp.getUnboundServiceDecl(i);
				registeredServices.put(sd.getServiceInstanceID(), sd.getClassName());
			}
		} catch (XMLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}   
	}

	private static ServiceProxy createServiceProxy(String controlName) {
		ServiceProxy sd = new ServiceProxy(controlName);
		Group group = new Group(new GroupInfo(), new List());
		sd.addControlItemAndInfo(group);
		Command listCmd = new Command(COMMAND_LIST_SERVICES, Command.DIRECTION_IN);
		group.addControlItemAndInfo(listCmd);

		Command lineCommand = new Command(COMMAND_SERVICES, Command.DIRECTION_OUT);
		lineCommand.addParamAndInfo(new Param("Unbound services list", 
				"text/plain", "Ingen data".getBytes()));
		sd.addControlItemAndInfo(lineCommand);

		Group group2 = new Group(new GroupInfo("Management"), new List());
		sd.addControlItemAndInfo(group2);
		Command loadCmd = new Command(COMMAND_LOAD, Command.DIRECTION_IN);
		loadCmd.addParamAndInfo(new Param("ServiceInstanceID", "text/plain", "".getBytes()));
		group2.addControlItemAndInfo(loadCmd);
		Command regCmd = new Command(COMMAND_REGISTER, Command.DIRECTION_IN);
		regCmd.addParamAndInfo(new Param("ClassName", "text/plain", "".getBytes()));
		regCmd.addParamAndInfo(new Param("InstanceNumber", "text/plain", "".getBytes()));
		group2.addControlItemAndInfo(regCmd);
//		Command killCmd = new Command(COMMAND_UNLOAD_ALL, Command.DIRECTION_IN);
//		group2.addControlItemAndInfo(killCmd);
		return sd;
	}
	public void commandReceived(CommandEvent e) {
		String command = e.getCommand().getID();
		try {
			if (command.equals(COMMAND_LIST_SERVICES)) {
				doSWCCommand();
			} else if(command.equals(COMMAND_LOAD)){
				byte[] data = e.getCommand().getParam(0).getData();
				FullXMLFactory ff = new FullXMLFactory();
				XMLRepresentable tmp = ff.createFromXMLData(data,0,data.length);
				if(tmp instanceof ServiceInstanceID) {
					ServiceInstanceID siid = (ServiceInstanceID) tmp;
					loadUnboundService(siid);
				} else {
					System.err.println("WARNING! ServiceManager.commandReceived: not SIID in XML:"+tmp);
				}
			}else if(command.equals(COMMAND_REGISTER)){
				byte[] data = e.getCommand().getParam(0).getData();
				String className = new String(data);
				String versionString = new String(e.getCommand().getParam(1).getData());
				registerUnboundService(className, versionString);
				doSWCCommand();
			}
			//XXX This is dangerous, as it leaves stale discovery data some places,
			// causing assemblies to stop working: hence it's disabled
//				else if(command.equals(COMMAND_UNLOAD_ALL)){
//				System.out.println("COMMAND_UNLOAD_ALL");
//				removeAllServices();
//				doSWCCommand();
//			}
		} catch (Exception ex) {ex.printStackTrace();}
	}

	private void doSWCCommand() {
		System.out.println("doSWCCommand");
		String line;
		if(loadedServices.isEmpty() && registeredServices.isEmpty())
			line = "no unbound services";
		else {
			line = "Registered services:\n";
			for (Iterator iter = registeredServices.keySet().iterator(); iter.hasNext();) {
				ServiceInstanceID s = (ServiceInstanceID) iter.next();
				line = line + new String(s.getXMLData());
				if(iter.hasNext()) line = line+"\n";
			}
//			line = line  + "\nLoaded components:\n";
//			for (Iterator iter = loadedComponents.iterator(); iter.hasNext();) {
//			SoftwareComponent a = (SoftwareComponent) iter.next();
//			line = line + a.getURN() + ": "+a.getClassName();
//			if(iter.hasNext()) line = line+"\n";
//			}

			//assemblies.get(0).assemblyInfo.getName();
		}
		doCommand1Param(COMMAND_SERVICES, line);
	}

	private void doCommand1Param(String commandID, String text) {
		ControlItem tmp = getServiceProxy().findItem(commandID);
		if(tmp instanceof Command) {
			Command command = (Command) tmp;

			if (command != null) {
				command.getParam(0).setData(text.getBytes());
				command.invoke();
			}
		}
	}

	public ServiceInstanceID registerUnboundService(String className, String versionString) {
		DeviceID did = context.getDeviceInfo().getDeviceID();
		ServiceInstanceID result = null;
		//XXX this is a hack, but there is no way to find the ServiceID from an AbstractService,
		// as that is a constructor parameter. HERE BE DRAGONS!!
		try {
			AbstractService service = (AbstractService) ist.palcom.base.System.instance()
			.newInstance(className,	new Object[] {context});

//			ServiceID sid = new ServiceID(did, versionString, did, versionString, null, null, null, null);
			ServiceID sid = service.getServiceID();
			service = null; // just in case
			ServiceInstanceID siid = new ServiceInstanceID(did,
					sid, context.getNewInstanceNumber(sid));
			System.out.println("registerUnboundService:"+siid+" = "+className);
			if(registeredServices.get(siid) == null) {
				registeredServices.put(siid, className);
				writeListFile();
//				fileStorage.open(swListFile, 0);
//				fileStorage.seek(swListFile, fileStorage.length(swListFile));
//				if(fileStorage.length(swListFile)>0) fileStorage.writeBytes(swListFile, "\n".getBytes(), 0, "\n".getBytes().length);
//				String tmp = siid + "="+className;
//				byte[] byteArray = tmp.getBytes();
//				int result = fileStorage.writeBytes(swListFile, byteArray, 0, byteArray.length);
//				fileStorage.close(swListFile);
//				if(result!=Storage.OK) {
//				System.err.println("Write to file operation for UnboundService failed");
//				}

			} else {
				DebugDispatcher.debug("ServiceManager.registerUS("+siid+"): already registered---ignoring", DebugDispatcher.CAT_SWCOMP);
			}
			result = siid;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}


	protected UnboundService lookupAndCreateUnboundService(ServiceInstanceID siid) {
		//TODO: get an exception here if className not found.
		String className = (String) registeredServices.get(siid);
		System.out.println("lookupSWC("+siid+") :"+className);
		if (className != null) {
			UnboundService service = new UnboundService(
					className, this);
			return service;
		}
		return null;
	}

	public void loadUnboundService(ServiceInstanceID siid) {
		if(findSIIDInLoadedServices(siid)) {
			DebugDispatcher.debug("UnboundService already loaded",DebugDispatcher.CAT_SWCOMP);
			return;
		}
		UnboundService service = lookupAndCreateUnboundService(siid);
		if(service != null) {
			if(swcList == null) {
				swcList = new PRDSubList("Unbound components", context.getNewLocalSID(), PRDSubList.KIND_HIERARCHICAL);
				context.addListOfServicesAtDeviceLevel(new ArrayList(), swcList);
			}
			loadedServices.add(service);
			loadUnboundService(siid,service);
		}
	}

	private boolean findSIIDInLoadedServices(ServiceInstanceID siid) {
		for (Iterator iter = loadedServices.iterator(); iter.hasNext();) {
			UnboundService swc = (UnboundService) iter.next();
			if(swc.getSIID().equals(siid)) 
				return true;
		}
		return false;
	}

	protected void loadUnboundService(ServiceInstanceID siid, UnboundService service) {
		service.loadService(context,siid, swcList);
	}

	public void unloadService(UnboundService c) {
		context.getScheduler().scheduleThread(new StopThread(c));    	
	}

	/** The PalcomThread that calls stopDevice. */
	private class StopThread extends PalcomPriorityThread {
		UnboundService c;
		public StopThread(UnboundService c) {
			this.c = c;
		}
		public void run() {
			c.unloadService();
		}
	}

	public void removeAllServices() {
		for (Iterator iter = loadedServices.iterator(); iter.hasNext();) {
			UnboundService c = (UnboundService) iter.next();

			System.out.println("ServiceManager.removeAllServices: Stopping "+c.getSIID());
			unloadService(c);
		}
		loadedServices.clear();
		System.out.println("** ServiceManager.removeAllServices: done");
	}

	public void serviceUnloaded(UnboundService service) {
		DebugDispatcher.debug("serviceUnloaded: loaded before", DebugDispatcher.CAT_SWCOMP);
		for (Iterator iter = loadedServices.iterator(); iter.hasNext();) {
			UnboundService c = (UnboundService) iter.next();
			DebugDispatcher.debug("  "+c.getSIID().toString(), DebugDispatcher.CAT_SWCOMP);
		}
		loadedServices.remove(service);
		DebugDispatcher.debug("serviceUnloaded: loaded after", DebugDispatcher.CAT_SWCOMP);
		for (Iterator iter = loadedServices.iterator(); iter.hasNext();) {
			UnboundService c = (UnboundService) iter.next();
			DebugDispatcher.debug("  "+c.getSIID().toString(), DebugDispatcher.CAT_SWCOMP);
		}
	}
	public void stop() throws IOException {
		inCommandThread.terminate();
		super.stop();
	}

}
