/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.services;

import ist.palcom.mal.Message;
import ist.palcom.resource.descriptor.ControlInfo;
import ist.palcom.resource.descriptor.ControlItem;
import ist.palcom.resource.function.FunctionInfoXMLFactory;
import ist.palcom.xml.XMLException;
import ist.palcom.xml.XMLFactory;

public class ControlItemFactory {

    private XMLFactory xmlFactory;

    public ControlItemFactory() {
        xmlFactory = new FunctionInfoXMLFactory();
    }

    public ControlItem createFromMessage(Message message) throws XMLException {
        if (message.getType() == Message.TYPE_MULTIPART) {
            Message xmlMessage = message.getPart(0);
            ControlInfo info = (ControlInfo) xmlFactory.createFromXMLData(
                    xmlMessage.getData(), 0, xmlMessage
                            .getDataLength());
            ControlItem item = info.createInstance();
            byte[][] data = new byte[message.getNumberOfParts() - 1][];
            for (int i = 0; i < data.length; i++) {
                Message binMessage = message.getPart(i + 1);
                data[i] = binMessage.getData();
            }
            item.setDataAccordingToDataRefs(data);
            return item;
        } else {
            throw new IllegalArgumentException(
                    "Cannot create ControlItem from non-multipart message.");
        }
    }
}
