/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.services;

import ist.palcom.debugging.DebugDispatcher;
import ist.palcom.device.DeviceContext;
import ist.palcom.discovery.CommandEvent;
import ist.palcom.resource.descriptor.Command;
import ist.palcom.resource.descriptor.ControlItem;
import ist.palcom.resource.descriptor.DeviceID;
import ist.palcom.resource.descriptor.LocalSID;
import ist.palcom.resource.descriptor.PRDService;
import ist.palcom.resource.descriptor.Param;
import ist.palcom.resource.descriptor.ServiceID;
import ist.palcom.resource.descriptor.ServiceInstanceID;
import ist.palcom.resource.descriptor.ServiceProxy;
import ist.palcom.resource.descriptor.URN;

import java.io.IOException;


public class SelfTestProvider extends AbstractService implements CommandEventListener{
	public static final String URN_SUFFIX = "selfTest";
	public static final String SERVICE_NAME="SelfTest";
	public static final String COMMAND_SELFTEST = "do_self_test";
	public static final String COMMAND_RESULT = "result";
    public static final String CONTENT_TYPE = "application/x-palcom-control+selftest+xml";

	
	private CommandListenerThread inCommandThread;

	private SelfTester tester;
	
	public SelfTestProvider(DeviceContext context,
            String serviceName, SelfTester tester) throws IOException {
        super(context, serviceName, true, false, PRDService.UNICAST, "P1",
        		false, "1", "Self test service", createServiceProxy(serviceName),
        		new LocalSID(context.getDeviceInfo().getDeviceID(), context.getNewLocalSIDString()));
        
        ServiceID serviceID = new ServiceID(
        		new DeviceID("X:1"), "TF17", new DeviceID("X:1"), "TF17", null, null, null, null);
		setServiceInstanceID(new ServiceInstanceID(context.getDeviceInfo().getDeviceID(), serviceID, 
				context.getNewInstanceNumber(serviceID)));
        
        // XXX HERE BE DRAGONS!
//        getServiceInfo().setContentType(CONTENT_TYPE);

        inCommandThread = new CommandListenerThread(this);
        context.getScheduler().scheduleThread(inCommandThread);
        getServiceProxy().addInCommandReceiver(inCommandThread);
//        System.out.println("SelfTestService");
        this.tester = tester;

    }
	
	/* Avoid constructor recursion...
	 * 
	 */
	protected SelfTestProvider getSelfTestService(URN urn) {
		return null;
	}
		
	private static ServiceProxy createServiceProxy(String controlName) {
        ServiceProxy sd = new ServiceProxy(controlName);

        Command lineCommand = new Command(COMMAND_SELFTEST, Command.DIRECTION_IN);
        sd.addControlItemAndInfo(lineCommand);       
        Command resultCmd = new Command(COMMAND_RESULT, Command.DIRECTION_OUT);
        resultCmd.addParamAndInfo(new Param("Status", "text/plain", "".getBytes()));
        sd.addControlItemAndInfo(resultCmd);
        return sd;
    }
    public void commandReceived(CommandEvent e) {
    	String command = e.getCommand().getID();
//    	System.out.println("commandReceived:"+command);
    	try {
    		if (command.equals(COMMAND_SELFTEST)) {
    			doSelfTestCmd();
    		}
    	} catch (Exception ex) {ex.printStackTrace();}
	}
	public void doSelfTestCmd() {
		DebugDispatcher.debug("doSelfTestCommand");
		String line = tester.doSelfTest();
		DebugDispatcher.debug("tester returned: "+line);
		doCommand1Param(COMMAND_RESULT, line);
	}

	private void doCommand1Param(String commandID, String text) {
	    ControlItem tmp = getServiceProxy().findItem(commandID);
	    if(tmp instanceof Command) {
	        Command command = (Command) tmp;

	        if (command != null) {
	            command.getParam(0).setData(text.getBytes());
	            command.invoke();
	        }
	    }
	}
    public void stop() throws IOException {
    	inCommandThread.terminate();
    	super.stop();
    }
}
