/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.simulated.device.factory;

import ist.palcom.resource.descriptor.DeviceID;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * @author David Svensson, david@cs.lth.se
 */
class ScenarioDescriptor {

    private final String name;

    private ArrayList<DeviceID> deviceIDs;

    ScenarioDescriptor(String name) {
        this.name = name;
        deviceIDs = new ArrayList<DeviceID>();
    }

    String getName() {
        return name;
    }

    /**
     * Adds the device ID to the scenario. If it is already in the scenario, nothing
     * happens.
     */
    void addDeviceID(DeviceID deviceID) {
        if (!deviceIDs.contains(deviceID)) {
            deviceIDs.add(deviceID);
        }
    }

    void removeDeviceID(DeviceID deviceID) {
        deviceIDs.remove(deviceID);
    }

    int getNumberOfDeviceIDs() {
        return deviceIDs.size();
    }

    DeviceID getDeviceID(int index) {
        return deviceIDs.get(index);
    }

    Iterator<DeviceID> getDeviceIDIterator() {
        return deviceIDs.iterator();
    }

    static ScenarioDescriptor createFromReader(BufferedReader reader)
            throws IOException {
        String line = reader.readLine();
        if (line == null) {
            return null;
        } else {
            String[] items = line.split(",");
            if (items.length == 0) {
                return null;
            }
            String name = URLDecoder.decode(items[0], "UTF-8");
            ScenarioDescriptor descriptor = new ScenarioDescriptor(name);
            for (int i = 1; i < items.length; i++) {
                descriptor.addDeviceID(new DeviceID(items[i]));
            }
            return descriptor;
        }
    }

    void writeToWriter(BufferedWriter writer) throws IOException {
        String encodedName = URLEncoder.encode(name, "UTF-8");
        StringBuffer line = new StringBuffer(encodedName);
        for (int i = 0; i < deviceIDs.size(); i++) {
            line.append("," + deviceIDs.get(i).toString());
        }
        writer.write(line.toString());
        writer.newLine();
        writer.flush();
    }
}
