/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.simulated.device.widgets;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.border.Border;

/**
 * @author David Svensson, david@cs.lth.se
 */
public class ClickableArea extends JLabel {

    protected Border noBorder;

    protected Border mouseOverBorder;

    protected Border mousePressedBorder;

    protected LinkedList<ActionListener> actionListeners;

    public ClickableArea() {
        this(null);
    }

    public ClickableArea(String text) {
        super(text);

        setHorizontalAlignment(JLabel.CENTER);

        noBorder = BorderFactory.createEmptyBorder(1, 1, 1, 1);
        mouseOverBorder = BorderFactory.createLineBorder(Color.GRAY);
        mousePressedBorder = BorderFactory.createLineBorder(Color.LIGHT_GRAY);

        setBorder(noBorder);

        actionListeners = new LinkedList<ActionListener>();

        addMouseListener(new ClickableAreaMouseListener(this));
    }

    public void addActionListener(ActionListener listener) {
        if (!actionListeners.contains(listener)) {
            actionListeners.add(listener);
        }
    }

    public void removeActionListener(ActionListener listener) {
        actionListeners.remove(listener);
    }

    private class ClickableAreaMouseListener implements MouseListener {
        private ClickableArea mother;

        ClickableAreaMouseListener(ClickableArea mother) {
            this.mother = mother;
        }

        public void mouseClicked(MouseEvent e) {
            Iterator iterator = actionListeners.iterator();
            while (iterator.hasNext()) {
                ActionListener listener = (ActionListener) iterator.next();
                ActionEvent event = new ActionEvent(mother, 0, null);
                listener.actionPerformed(event);
            }
        }

        public void mouseEntered(MouseEvent e) {
            setBorder(mouseOverBorder);
        }

        public void mouseExited(MouseEvent e) {
            setBorder(noBorder);
        }

        public void mousePressed(MouseEvent e) {
            setBorder(mousePressedBorder);
        }

        public void mouseReleased(MouseEvent e) {
            setBorder(mouseOverBorder);
        }
    }
}