/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.pfs;

import java.io.IOException;

/**
 * AbstractStorage.java
 * 
 * AbstractStorage defines the minimal implementation of a palcom filesystem
 * ie open, close, read, write. Some implementation extends this to also more 
 * advanced filesystem functions
 * 
 * @author Konrad Tollmar, konrad.tollmar@design.lth.se
 */

public abstract class AbstractStorage implements Storage {
		
	public abstract int open(String file, int mode) throws StorageException; 
	public abstract int close(String file) throws StorageException;

	public abstract int read(String file) throws IOException;
	public abstract int write(String file, int b) throws IOException;

	public int readBytes(String file, byte b[], int off, int len) 
	throws IOException { return NOP;}
	public int writeBytes(String file, byte b[], int off, int len) 
	throws IOException {return NOP;}
	
	public long pos(String file) throws StorageException { return -1; } 
	public int seek(String file, long pos) throws StorageException { return NOP; }
	public int available(String file) throws StorageException { return NOP; }

	public int create(String path) throws StorageException { return NOP; }
	public int delete(String path) throws StorageException { return NOP; }
	public int dirs(String path, String [] files) { return NOP; }
	public int volume() { return NOP; }
}
