/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.pio;

import ist.palcom.storage.pfs.Storage;
import ist.palcom.storage.pfs.StorageException;

import java.io.IOException;

public class StorageAccessStream {

	Storage store;
	String filen;
	
	public StorageAccessStream(Storage store, String filen)  
	throws IOException
	{
		open(store, filen, Storage.READWRITE);
	} 

	private void open(Storage store, String filen, int mode) throws IOException 
	{
		try {
			this.filen = filen;
			this.store = store;
			store.open(filen, mode);
		} catch (StorageException e) {
			throw new IOException(); 
		}
	}
	
	public void close() throws IOException 
	{
		try {
			store.close(filen);
		} catch (StorageException e) {
			throw new IOException(); 
		}
	}

	public void seek(int pos) throws IOException {
		try {
			store.seek(filen, pos);
		} catch (StorageException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public int available() throws IOException {
		try {
			return store.available(filen);
		} catch (StorageException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	public long pos() throws IOException {
		try {
			return store.pos(filen);
		} catch (StorageException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return 0;
	}

	public int read() throws IOException 
	{
		int b = 0;
		if ( (b = store.read(filen)) != Storage.ERROR)
			return b;
		else
			return Storage.ERROR;
	}
	
	public int read(byte b[], int off, int len) throws IOException 
	{
		int result = 0;
		if ( (result = store.readBytes(filen, b, off, len)) != Storage.ERROR)
			return result;
		else
			return Storage.ERROR;
	}

	public int readInt() throws IOException {

		int ch1 = read();
        int ch2 = read();
        int ch3 = read();
        int ch4 = read();
        
        if ((ch1 | ch2 | ch3 | ch4) < 0) {
             throw new IOException("EndOfFileException");
        }	
        return ((ch1 << 24) + (ch2 << 16) + (ch3 << 8) + (ch4 << 0));
		
		/*
		byte []b = new byte[4];
		read(b,0,4);
        return ((b[0] << 24) + (b[1] << 16) + (b[2] << 8) + (b[3] << 0));
		*/
    }

	public void write(int b) throws IOException {
		store.write(filen, b);
    }

	public void write(byte b[], int off, int len) throws IOException {
		store.writeBytes(filen, b, off, len);
	}

	public void writeInt(int v) throws IOException {
		byte []b = new byte[4];
		b[0] = (byte)((v >>> 24) & 0xFF);
		b[1] = (byte)((v >>> 16) & 0xFF);
		b[2] = (byte)((v >>> 8) & 0xFF);
		b[3] = (byte)((v >>> 0) & 0xFF);
		write(b,0,4);		
    }
}
