/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.pio;

/**
 * StorageInputStream.java
 * StorageInputStream enables the use of the Storage as a stream
 * This is (very) simmilar to the std java.io classes
 *  
 * @author Konrad Tollmar, konrad.tollmar@design.lth.se
 */

import ist.palcom.storage.pfs.Storage;
import ist.palcom.storage.pfs.StorageException;

import java.io.*;

public class StorageInputStream extends InputStream {

	Storage store;
	String filen;
	
	/**
	 * Creat an input stream
	 * @param store the Storage 
	 * @param filen the File name
	 * @throws IOException
	 */
	public StorageInputStream(Storage store, String filen)  
	throws IOException
	{
		this.store = store;
		open(filen);
	} 

	/**
	 * Open an input stream
	 * @param filen the Storage
	 * @throws IOException
	 */
	public void open(String filen) 
	throws IOException 
	{
		this.filen = filen;
		try {
			store.open(filen, -1); // todo: implement rw mode
		} catch (StorageException e) {
			throw new IOException(); 
		}
	}
	
	/**
	 * Close an input stream
	 */
	public void close() 
	throws IOException 
	{
		try {
			store.close(filen);
		} catch (StorageException e) {
			throw new IOException(); 
		}
	}

	// Todo: implement these utility methods
	// public long skip(long n) throws IOException {}
	// public int available() throws IOException {}

	/**
	 * Read from the input stream
	 * @return an int
	 * @throws IOException
	 */
	public int read()  
	throws IOException // todo: wrap IOException into StorageException, see open / close
	{
		int b = 0;
		if ( (b = store.read(filen)) != Storage.ERROR)
			return b;
		else
			return Storage.ERROR;
	}
	
	/**
	 * Read from the input stream
	 * @return how many bytes that was read
	 * @param b the byte array to store
	 * @param off the offset
	 * @param len the length of the byte array
	 * @throws IOException
	 */
	public int read(byte b[], int off, int len) 
	throws IOException 
	{
		int result = 0;
		if ( (result = store.readBytes(filen, b, off, len)) != Storage.ERROR)
			return result;
		else
			return Storage.ERROR;
	}
	
	/**
	 * Read an int from the input stream
	 * @return an int
	 * @throws IOException
	 */
	public int readInt() 
	throws IOException {
		
		
        int ch1 = read();
        int ch2 = read();
        int ch3 = read();
        int ch4 = read();
        
        if ((ch1 | ch2 | ch3 | ch4) < 0) {
             throw new IOException("EndOfFileException");
        }	
        return ((ch1 << 24) + (ch2 << 16) + (ch3 << 8) + (ch4 << 0));
		
    }
}
