/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.tags;

import java.io.IOException;
import java.util.Iterator;

import ist.palcom.storage.pfs.Storage;
import ist.palcom.storage.pio.StorageInputStream;
import ist.palcom.storage.pio.StorageOutputStream;

/**
 * Tags.java
 * 
 * Tags demonstrate a simple way of storing tags
 * 
 * @author Konrad Tollmar, konrad.tollmar@design.lth.se
 */

public class Tags {
	
	OrderedMap tags;

	public Tags() {
		tags = new OrderedMap();
	}
	
	public void load(Storage store, String file) {
		StorageInputStream in;
		try {
			in = new StorageInputStream(store,file);
			StringBuilder sb = new StringBuilder(); 
			byte[] buf = new byte[512];
			int count;
			while ((count = in.read(buf,0,buf.length)) > 0) {
				byte[] correctSizedBuf = new byte[count];
				for(int i = 0; i < count; ++i) {
					correctSizedBuf[i] = buf[i];
				}
				sb.append(new String(correctSizedBuf));
			}

			String[] tokens = sb.toString().split("#");
			for(int i = 0; i<tokens.length; ++i) {
				String [] attr = tokens[i].split(":");
				System.out.println(attr[0] + ":" + attr[1]);
				// todo: fix this set2(attr[0],attr[1]);
			}
			in.close();

		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void save(Storage store, String file) {
		StorageOutputStream out;
		try {
			out = new StorageOutputStream(store,file);
			String kvdelim = ":";
			String tgdelim = "#";
			
			for (Iterator iter = tags.keySet().iterator(); iter.hasNext();) {
				String tag = (String) iter.next();
				String val = ""; // todo: (String) tags.get(tag);
				out.write(tag.getBytes(), 0, tag.getBytes().length);			
				out.write(kvdelim.getBytes(), 0, kvdelim.getBytes().length);			
				out.write(val.getBytes(), 0, val.getBytes().length);			
				out.write(tgdelim.getBytes(), 0, tgdelim.getBytes().length);			
			}			
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}		
	}

	public void set(Taggable tobj, String tag) {
		tags.put(tag, new Integer(tobj.getid()));
	}
		
	public int [] get(String tag) {
		Object [] objID = tags.get(tag);
		int [] ids = new int[objID.length];
		for (int i = 0; i < objID.length; i++) {
			Integer id = (Integer)objID[i];
			ids[i] = id.intValue();
		}
		return ids; 
	}

	public String getKeys() {
		StringBuffer keys = new StringBuffer();
		for (Iterator iter = tags.keySet().iterator(); iter.hasNext();) {
			String tag = (String) iter.next();
			keys.append(tag);
		}		
		return keys.toString();
	}
	
	public String toString() {
		StringBuffer keys = new StringBuffer();
		for (Iterator iter = tags.keySet().iterator(); iter.hasNext();) {
			String tag = (String) iter.next();
			int [] ind = get(tag);
			keys.append(tag + " [" );
			for (int i = 0; i< ind.length; i++) 
				keys.append(ind[i]+" ");	
			keys.append("] ");
		}		
		return keys.toString();
	}
	
	public void setStringTag(String tid, String tag) {
		tags.put(tag, tid);
	}
	
	public String [] getStringTag(String tag) {
		Object [] objID = tags.get(tag);
		String [] objs = new String[objID.length];
		for (int i = 0; i < objID.length; i++) {
			objs[i] = (String)objID[i];
		}
		return objs;
	}
}
