/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.test;

import ist.palcom.storage.pfs.ByteStorage;
import ist.palcom.storage.pfs.FileStorage;
import ist.palcom.storage.pfs.Storage;
import ist.palcom.storage.pfs.StorageException;
import ist.palcom.storage.pio.StorageInputStream;
import ist.palcom.storage.pio.StorageOutputStream;

import java.io.IOException;

public class StorageStreamsTest {


	public void testStorageStreams(Storage store) {
		try {

			System.out.println("testStorageStreams: ");
			System.out.println();
			
			StorageOutputStream out = new StorageOutputStream(store,"file.txt");

			String [] fields = {"Name:Lone Malm","#", "Address:Main Street 1, Sommervile" };

			for (int i = 0; i < fields.length; i++)
				out.write(fields[i].getBytes(), 0, fields[i].getBytes().length);

			out.close();

			StorageInputStream in = new StorageInputStream(store,"file.txt");

			StringBuilder sb = new StringBuilder(); 
			byte[] buf = new byte[512];
			int count;
			while ((count = in.read(buf,0,buf.length)) > 0) {
				byte[] correctSizedBuf = new byte[count];
				for(int i = 0; i < count; ++i) {
					correctSizedBuf[i] = buf[i];
				}
				sb.append(new String(correctSizedBuf));
			}

			String[] tokens = sb.toString().split("#");
			for(int i = 0; i<tokens.length; ++i) {
				String [] attr = tokens[i].split(":");
				System.out.println(attr[0] + ":" + attr[1]);
			}
			
			in.close();
			
			try {
				store.delete("file.txt");
			} catch (StorageException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			System.out.println("--");

		} catch (IOException e) {}
	}

	public void testStorageStreams2(Storage store) { 
		
		try {
			System.out.println("testStorageStreams2: ");
			System.out.println();
			StorageOutputStream out = new StorageOutputStream(store,"file2.txt");

			out.writeInt(-1200);

			String str = new String("Hej-Hopp");
			out.write(str.getBytes(), 0, str.length());
			
			out.close();

			StorageInputStream in = new StorageInputStream(store,"file2.txt");

			//int bint = 0; 
			int bint = in.readInt();
			
			byte[] buf = new byte[512];
			int count = in.read(buf,0,buf.length);
			
			byte[] correctSizedBuf = new byte[count];
			for(int i = 0; i < count; ++i) {
				correctSizedBuf[i] = buf[i];
			}
			
			str = new String(correctSizedBuf);
			System.out.println(bint + " " + str);
			
			in.close();
			
			try {
				store.delete("file2.txt");
			} catch (StorageException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			System.out.println("--");

		} catch (IOException e) {
			System.err.println("ioexception");
		}

	}
	public static void main(String[] args) {
		Storage store; 
		
		StorageStreamsTest main = new StorageStreamsTest();

		store = new ByteStorage("/"); 
		main.testStorageStreams(store);
		main.testStorageStreams2(store);
		
 		store = new FileStorage("/");
		main.testStorageStreams(store);
		main.testStorageStreams2(store);
	}
}
