/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.storage.test;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import ist.palcom.storage.tags.Taggable;
import ist.palcom.storage.test.Employee;

public class TaggableEmployee extends Employee implements Taggable {

	int tagid = 0;
    public TaggableEmployee(){}

    public TaggableEmployee( int employeeID, String firstName,
            String lastName, int managerID ) {
    	super(employeeID, firstName, lastName, managerID);
    	setid(employeeID);
    }
        
	public int getid() {
		return tagid;
	}

	public void setid(int id) {
		this.tagid = id;
	}

	public String toString() {
		return " (tid:"+tagid + ") " + super.toString();
	}
	
    public byte[] persist() throws IOException {
        ByteArrayOutputStream bout = 
                           new ByteArrayOutputStream();
        
        bout.write(tagid);
        
        bout.write(getID());
        bout.write(getFirstName().getBytes().length); 
        bout.write(getFirstName().getBytes());
        bout.write(getLastName().getBytes().length); 
        bout.write(getLastName().getBytes());
        bout.write(getManagerID());
        bout.flush();
        
        return bout.toByteArray();
    }

    public void resurrect( byte[] data ) 
                                   throws IOException {
        ByteArrayInputStream bin = 
                      new ByteArrayInputStream( data );
        
        tagid = bin.read();
        employeeID = bin.read();
        int stringLength = bin.read();
        byte[] b = new byte[stringLength];
        int result = bin.read(b);
        if(result>=0) {
        	firstName = new String(b);
        } //else EOF reached
        stringLength = bin.read();
        b = new byte[stringLength];
        result = bin.read(b);
        if(result>=0) {
        	lastName = new String(b);
        } //else EOF reached
        managerID = bin.read();    
    }
}
