/*
   Copyright (c) 2004-2007, The PalCom Project consortium.
   See http://www.ist-palcom.org/consortium.

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

       * Redistributions of source code must retain the above copyright
         notice, this list of conditions and the following disclaimer.

       * Redistributions in binary form must reproduce the above
         copyright notice, this list of conditions and the following
         disclaimer in the documentation and/or other materials provided
         with the distribution.

       * Neither the name of the PalCom Project nor the names of its
         contributors may be used to endorse or promote products derived
         from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package ist.palcom.types;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;

import org.kxml2.io.KXmlParser;
import org.kxml2.io.KXmlSerializer;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlSerializer;

import ist.palcom.hmap.HmapDir;
import ist.palcom.hmap.HmapLeaf;
import ist.palcom.hmap.HmapNode;

public abstract class AbstractTypeManager extends HmapDir{

	protected AbstractTypeManager(String name, String typeName, String typeVersion, String typeFullyQualifiedVersion){
		super(name, typeName, typeVersion, typeFullyQualifiedVersion);
	}
	
	public HmapNode deSerialize(byte[] dataIn){
		return deSerialize(new ByteArrayInputStream(dataIn));
	}
	public HmapNode deSerialize(InputStream dataIn){
		XmlPullParser parser = new KXmlParser();
		try {
			parser.setInput(new InputStreamReader(dataIn));
			return xmlreader(parser);
		} catch (XmlPullParserException e) {
			e.printStackTrace();
			return null;
		}
	}
	public byte[] serialize(HmapNode node){
		 ByteArrayOutputStream outStream = new ByteArrayOutputStream();
		 OutputStreamWriter writer = new OutputStreamWriter(outStream);
		 XmlSerializer serializer = new KXmlSerializer();
		 try {
			serializer.setOutput(writer);
			serializer.startDocument("UTF-8",new Boolean(true));
			xmlwriter(serializer, node);
			serializer.endDocument();
			outStream.flush();
			return outStream.toByteArray();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalStateException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	private void xmlwriter(XmlSerializer serializer, HmapNode node){
		try {
			if(node.isDir()){
				serializeDirRecursively(node.asDir(), serializer);
			}else if(node.isLeaf()){
				serializeLeaf(node.asLeaf(), serializer);
			}
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalStateException e) {
			e.printStackTrace();
		}
	}

	
	private HmapNode xmlreader(XmlPullParser parser){
		Stack stack = new Stack();
		boolean first = true;
		try {
			while (parser.getEventType() != XmlPullParser.END_DOCUMENT) {
				switch (parser.getEventType()) {
				case XmlPullParser.START_TAG:
					HmapNode tmp = deSerializeNode(parser);
					if (tmp != null) {
						if (first) {
							stack.push(tmp);
							stack.push(tmp);
							first = false;
						}else{
							((HmapDir) stack.peek()).addNode(tmp);
							stack.push(tmp);
						}
					}
					break;
				case XmlPullParser.END_TAG:
					stack.pop();
					break;	
				case XmlPullParser.TEXT:
					break;	
				default:
					break;
				}
				try {
					parser.nextToken();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		} catch (XmlPullParserException e) {
			e.printStackTrace();
		}
		return (HmapNode) stack.pop();
	}
	
	protected abstract HmapNode deSerializeNode(XmlPullParser parser);
	protected abstract void serializeNode(HmapNode node, XmlSerializer serializer);
	protected abstract void serializeDirRecursively(HmapDir dir, XmlSerializer serializer) ;
	protected abstract void serializeLeaf(HmapLeaf leaf, XmlSerializer serializer);
	
	private class Stack extends ArrayList{
		public void push(Object obj){
			add(size(), obj);
		}
		public Object pop(){
			if(size() > 0){
				Object obj = get(size() - 1);
				remove(obj);
				return obj;
			}else{
				return null;
			}
		}
		public Object peek(){
			return get(size() - 1);
		}
	}
}
